@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Function_Type.cxx,v 1.15.2.16.2.16 2004/11/20 16:19:04 easysw Exp $"
+// "$Id: Fl_Function_Type.cxx 6013 2008-01-04 22:18:40Z matt $"
 //
 // C function type code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2006 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,10 +20,14 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
+#include <FL/Fl_Preferences.H>
+#include <FL/Fl_File_Chooser.H>
 #include "Fl_Type.h"
 #include <FL/fl_show_input.H>
 #include "../src/flstring.h"
@@ -37,6 +41,8 @@ extern const char* i18n_file;
 extern const char* i18n_set;
 extern char i18n_program[];
 
+extern void redraw_browser();
+
 ////////////////////////////////////////////////////////////////
 // quick check of any C code for legality, returns an error message
 
@@ -130,7 +136,10 @@ Fl_Type *Fl_Function_Type::make() {
 
 void Fl_Function_Type::write_properties() {
   Fl_Type::write_properties();
-  if (!public_) write_string("private");
+  switch (public_) {
+    case 0: write_string("private"); break;
+    case 2: write_string("protected"); break;
+  }
   if (cdecl_) write_string("C");
   if (return_type) {
     write_string("return_type");
@@ -141,6 +150,8 @@ void Fl_Function_Type::write_properties() {
 void Fl_Function_Type::read_property(const char *c) {
   if (!strcmp(c,"private")) {
     public_ = 0;
+  } else if (!strcmp(c,"protected")) {
+    public_ = 2;
   } else if (!strcmp(c,"C")) {
     cdecl_ = 1;
   } else if (!strcmp(c,"return_type")) {
@@ -157,7 +168,15 @@ void Fl_Function_Type::open() {
   if (!function_panel) make_function_panel();
   f_return_type_input->static_value(return_type);
   f_name_input->static_value(name());
-  f_public_button->value(public_);
+  if (is_in_class()) {
+    f_public_member_choice->value(public_);
+    f_public_member_choice->show();
+    f_public_choice->hide();
+  } else {
+    f_public_choice->value(public_>0);
+    f_public_choice->show();
+    f_public_member_choice->hide();
+  }
   f_c_button->value(cdecl_);
   function_panel->show();
   const char* message = 0;
@@ -177,12 +196,29 @@ void Fl_Function_Type::open() {
     if (*c && *d != '(') {
       message = "must be name(arguments), try again:"; continue;
     }
+    int mod = 0;
     c = f_return_type_input->value();
     message = c_check(c); if (message) continue;
     name(f_name_input->value());
     storestring(c, return_type);
-    public_ = f_public_button->value();
-    cdecl_ = f_c_button->value();
+    if (is_in_class()) {
+      if (public_ != f_public_member_choice->value()) {
+        mod = 1;
+        public_ = f_public_member_choice->value();
+        redraw_browser();
+      }
+    } else {
+      if (public_ != f_public_choice->value()) {
+        mod = 1;
+        public_ = f_public_choice->value();
+        redraw_browser();
+      }
+    }
+    if (cdecl_ != f_c_button->value()) {
+      mod = 1;
+      cdecl_ = f_c_button->value();
+    }
+    if (mod) set_modflag(1);
     break;
   }
  BREAK2:
@@ -322,20 +358,37 @@ void Fl_Function_Type::write_code1() {
       write_c("%s%s %s {\n", rtype, star, s);
     }
   }
-  if (havewidgets) write_c("  %s* w;\n",subclassname(child));
+
+  if (havewidgets && !child->name()) write_c("  %s* w;\n", subclassname(child));
   indentation += 2;
 }
 
 void Fl_Function_Type::write_code2() {
+  Fl_Type *child;
+  const char *var = "w";
+  for (child = next; child && child->level > level; child = child->next)
+    if (child->is_window() && child->name()) var = child->name();
+
   if (ismain()) {
-    if (havewidgets) write_c("  w->show(argc, argv);\n");
+    if (havewidgets) write_c("  %s->show(argc, argv);\n", var);
     write_c("  return Fl::run();\n");
-  } else if (havewidgets && !constructor && !return_type)
-    write_c("  return w;\n");
+  } else if (havewidgets && !constructor && !return_type) {
+    write_c("  return %s;\n", var);
+  }
   write_c("}\n");
   indentation = 0;
 }
 
+int Fl_Function_Type::has_signature(const char *rtype, const char *sig) const {
+  if (rtype && !return_type) return 0;
+  if (!name()) return 0;
+  if ( (rtype==0L || strcmp(return_type, rtype)==0) 
+      && fl_filename_match(name(), sig)) {
+    return 1;
+  }
+  return 0;
+}
+
 ////////////////////////////////////////////////////////////////
 
 Fl_Type *Fl_Code_Type::make() {
@@ -463,7 +516,7 @@ void Fl_CodeBlock_Type::write_code2() {
 
 int Fl_Decl_Type::is_public() const 
 {
- Fl_Type *p = Fl_Type::current;
+ Fl_Type *p = parent;
  while (p && !p->is_decl_block()) p = p->parent;
  if(p && p->is_public() && public_)
    return public_;
@@ -477,6 +530,7 @@ Fl_Type *Fl_Decl_Type::make() {
   while (p && !p->is_decl_block()) p = p->parent;
   Fl_Decl_Type *o = new Fl_Decl_Type();
   o->public_ = 0;
+  o->static_ = 1;
   o->name("int x;");
   o->add(p);
   o->factory = this;
@@ -485,12 +539,20 @@ Fl_Type *Fl_Decl_Type::make() {
 
 void Fl_Decl_Type::write_properties() {
   Fl_Type::write_properties();
-  if (public_) write_string("public");
+  switch (public_) {
+    case 1: write_string("public"); break;
+    case 2: write_string("protected"); break;
+  }
+  if (!static_) write_string("global");
 }
 
 void Fl_Decl_Type::read_property(const char *c) {
   if (!strcmp(c,"public")) {
     public_ = 1;
+  } else if (!strcmp(c,"protected")) {
+    public_ = 2;
+  } else if (!strcmp(c,"global")) {
+    static_ = 0;
   } else {
     Fl_Type::read_property(c);
   }
@@ -499,7 +561,15 @@ void Fl_Decl_Type::read_property(const char *c) {
 void Fl_Decl_Type::open() {
   if (!decl_panel) make_decl_panel();
   decl_input->static_value(name());
-  decl_public_button->value(public_);
+  if (is_in_class()) {
+    decl_class_choice->value(public_);
+    decl_class_choice->show();
+    decl_choice->hide();
+  } else {
+    decl_choice->value((public_&1)|((static_&1)<<1));
+    decl_choice->show();
+    decl_class_choice->hide();
+  }
   decl_panel->show();
   const char* message = 0;
   for (;;) { // repeat as long as there are errors
@@ -515,7 +585,21 @@ void Fl_Decl_Type::open() {
     message = c_check(c&&c[0]=='#' ? c+1 : c);
     if (message) continue;
     name(c);
-    public_ = decl_public_button->value();
+    if (is_in_class()) {
+      if (public_!=decl_class_choice->value()) {
+        set_modflag(1);
+        public_ = decl_class_choice->value();
+      }
+    } else {
+      if (public_!=(decl_choice->value()&1)) {
+        set_modflag(1);
+        public_ = (decl_choice->value()&1);
+      }
+      if (static_!=((decl_choice->value()>>1)&1)) {
+        set_modflag(1);
+        static_ = ((decl_choice->value()>>1)&1);
+      }
+    }
     break;
   }
  BREAK2:
@@ -527,6 +611,17 @@ Fl_Decl_Type Fl_Decl_type;
 void Fl_Decl_Type::write_code1() {
   const char* c = name();
   if (!c) return;
+  // handle a few keywords differently if inside a class
+  if (is_in_class() && (
+         !strncmp(c,"class",5) && isspace(c[5])
+      || !strncmp(c,"typedef",7) && isspace(c[7])
+      || !strncmp(c,"FL_EXPORT",9) && isspace(c[9])
+      || !strncmp(c,"struct",6) && isspace(c[6])
+      ) ) {
+    write_public(public_);
+    write_h("  %s\n", c);
+    return;
+  }
   // handle putting #include, extern, using or typedef into decl:
   if (!isalpha(*c) && *c != '~'
       || !strncmp(c,"extern",6) && isspace(c[6])
@@ -542,18 +637,27 @@ void Fl_Decl_Type::write_code1() {
       write_c("%s\n", c);
     return;
   }
+  // find the first C++ style comment
+  const char* e = c+strlen(c), *csc = c;
+  while (csc<e && (csc[0]!='/' || csc[1]!='/')) csc++;
+  if (csc!=e) e = csc; // comment found
   // lose all trailing semicolons so I can add one:
-  const char* e = c+strlen(c);
+  while (e>c && e[-1]==' ') e--;
   while (e>c && e[-1]==';') e--;
   if (class_name(1)) {
     write_public(public_);
-    write_h("  %.*s;\n", e-c, c);
+    write_h("  %.*s; %s\n", (int)(e-c), c, csc);
   } else {
     if (public_) {
-      write_h("extern %.*s;\n", e-c, c);
-      write_c("%.*s;\n", e-c, c);
+      if (static_) 
+        write_h("extern ");
+      write_h("%.*s; %s\n", (int)(e-c), c, csc);
+      if (static_)
+        write_c("%.*s; %s\n", (int)(e-c), c, csc);
     } else {
-      write_c("static %.*s;\n", e-c, c);
+      if (static_) 
+        write_c("static ");
+      write_c("%.*s; %s\n", (int)(e-c), c, csc);
     }
   }
 }
@@ -578,7 +682,10 @@ Fl_Type *Fl_DeclBlock_Type::make() {
 
 void Fl_DeclBlock_Type::write_properties() {
   Fl_Type::write_properties();
-  if (public_) write_string("public");
+  switch (public_) {
+    case 1: write_string("public"); break;
+    case 2: write_string("protected"); break;
+  }
   write_string("after");
   write_word(after);
 }
@@ -586,6 +693,8 @@ void Fl_DeclBlock_Type::write_properties() {
 void Fl_DeclBlock_Type::read_property(const char *c) {
   if(!strcmp(c,"public")) {
     public_ = 1;
+  } else if(!strcmp(c,"protected")) {
+    public_ = 2;
   } else  if (!strcmp(c,"after")) {
     storestring(read_word(),after);
   } else {
@@ -596,7 +705,7 @@ void Fl_DeclBlock_Type::read_property(const char *c) {
 void Fl_DeclBlock_Type::open() {
   if (!declblock_panel) make_declblock_panel();
   decl_before_input->static_value(name());
-  declblock_public_button->value(public_);
+  declblock_public_choice->value((public_>0));
   decl_after_input->static_value(after);
   declblock_panel->show();
   const char* message = 0;
@@ -618,7 +727,11 @@ void Fl_DeclBlock_Type::open() {
     message = c_check(c&&c[0]=='#' ? c+1 : c);
     if (message) continue;
     storestring(c,after);
-    public_ = declblock_public_button->value();
+    if (public_ != declblock_public_choice->value()) {
+      set_modflag(1);
+      public_ = declblock_public_choice->value();
+      redraw_browser();
+    }
     break;
   }
  BREAK2:
@@ -643,6 +756,241 @@ void Fl_DeclBlock_Type::write_code2() {
 
 ////////////////////////////////////////////////////////////////
 
+Fl_Type *Fl_Comment_Type::make() {
+  Fl_Type *p = Fl_Type::current;
+  while (p && !p->is_code_block()) p = p->parent;
+  Fl_Comment_Type *o = new Fl_Comment_Type();
+  o->in_c_ = 1;
+  o->in_h_ = 1;
+  o->style_ = 0;
+  o->name("my comment");
+  o->add(p);
+  o->factory = this;
+  o->title_buf[0] = 0;
+  return o;
+}
+
+void Fl_Comment_Type::write_properties() {
+  Fl_Type::write_properties();
+  if (in_c_) write_string("in_source"); else write_string("not_in_source"); 
+  if (in_h_) write_string("in_header"); else write_string("not_in_header");
+}
+
+void Fl_Comment_Type::read_property(const char *c) {
+  if (!strcmp(c,"in_source")) {
+    in_c_ = 1;
+  } else if (!strcmp(c,"not_in_source")) {
+    in_c_ = 0;
+  } else if (!strcmp(c,"in_header")) {
+    in_h_ = 1;
+  } else if (!strcmp(c,"not_in_header")) {
+    in_h_ = 0;
+  } else {
+    Fl_Type::read_property(c);
+  }
+}
+
+#include "comments.h"
+
+static void load_comments_preset(Fl_Preferences &menu) {
+  static const char * const predefined_comment[] = {
+    "GNU Public License/GPL Header",  "GNU Public License/GPL Footer",
+    "GNU Public License/LGPL Header", "GNU Public License/LGPL Footer",
+    "FLTK/Header", "FLTK/Footer" };
+  int i;
+  menu.set("n", 6);
+  Fl_Preferences db(Fl_Preferences::USER, "fltk.org", "fluid_comments");
+  for (i=0; i<6; i++) {
+    menu.set(Fl_Preferences::Name(i), predefined_comment[i]);
+    db.set(predefined_comment[i], comment_text[i]);
+  }
+}
+
+void Fl_Comment_Type::open() {
+  if (!comment_panel) make_comment_panel();
+  const char *text = name();
+  {
+    int i=0, n=0;
+    Fl_Preferences menu(Fl_Preferences::USER, "fltk.org", "fluid_comments_menu");
+    comment_predefined->clear();
+    comment_predefined->add("_Edit/Add current comment...");
+    comment_predefined->add("_Edit/Remove last selection...");
+    menu.get("n", n, -1);
+    if (n==-1) load_comments_preset(menu);
+    menu.get("n", n, 0);
+    for (i=0;i<n;i++) {
+      char *text;
+      menu.get(Fl_Preferences::Name(i), text, "");
+      comment_predefined->add(text);
+      free(text);
+    }
+  }
+  comment_input->buffer()->text( text ? text : "" );
+  comment_in_source->value(in_c_);
+  comment_in_header->value(in_h_);
+  comment_panel->show();
+  const char* message = 0;
+  char itempath[256]; itempath[0] = 0;
+  int last_selected_item = 0;
+  for (;;) { // repeat as long as there are errors
+    if (message) fl_alert(message);
+    for (;;) {
+      Fl_Widget* w = Fl::readqueue();
+      if (w == comment_panel_cancel) goto BREAK2;
+      else if (w == comment_panel_ok) break;
+      else if (w == comment_predefined) {
+        if (comment_predefined->value()==1) {
+          // add the current comment to the database
+          const char *xname = fl_input(
+            "Please enter a name to reference the current\ncomment in your database.\n\n"
+            "Use forward slashes '/' to create submenus.", 
+            "My Comment");
+          if (xname) {
+            char *name = strdup(xname);
+            for (char*s=name;*s;s++) if (*s==':') *s = ';';
+            int n;
+            Fl_Preferences db(Fl_Preferences::USER, "fltk.org", "fluid_comments");
+            db.set(name, comment_input->buffer()->text());
+            Fl_Preferences menu(Fl_Preferences::USER, "fltk.org", "fluid_comments_menu");
+            menu.get("n", n, 0);
+            menu.set(Fl_Preferences::Name(n), name);
+            menu.set("n", ++n);
+            comment_predefined->add(name);
+            free(name);
+          }
+        } else if (comment_predefined->value()==2) {
+          // remove the last selected comment from the database
+          if (itempath[0]==0 || last_selected_item==0) {
+            fl_message("Please select an entry form this menu first.");
+          } else if (fl_choice("Are you sure that you want to delete the entry\n"
+	                       "\"%s\"\nfrom the database?", "Cancel", "Delete",
+			       NULL, itempath)) {
+            Fl_Preferences db(Fl_Preferences::USER, "fltk.org", "fluid_comments");
+            db.deleteEntry(itempath);
+            comment_predefined->remove(last_selected_item);
+            Fl_Preferences menu(Fl_Preferences::USER, "fltk.org", "fluid_comments_menu");
+            int i, n;
+            for (i=4, n=0; i<comment_predefined->size(); i++) {
+              const Fl_Menu_Item *mi = comment_predefined->menu()+i;
+              if (comment_predefined->item_pathname(itempath, 255, mi)==0) {
+                if (itempath[0]=='/') memmove(itempath, itempath+1, 255);
+                if (itempath[0]) menu.set(Fl_Preferences::Name(n++), itempath);
+              }
+            }
+            menu.set("n", n);
+          }
+        } else {
+          // load the selected comment from the database
+          if (comment_predefined->item_pathname(itempath, 255)==0) {
+            if (itempath[0]=='/') memmove(itempath, itempath+1, 255);
+            Fl_Preferences db(Fl_Preferences::USER, "fltk.org", "fluid_comments");
+            char *text; 
+            db.get(itempath, text, "(no text found in data base)");
+            comment_input->buffer()->text(text);
+            free(text);
+            last_selected_item = comment_predefined->value();
+          }
+        }
+      }
+      else if (w == comment_load) {
+        // load a comment from disk
+	fl_file_chooser_ok_label("Use File");
+        const char *fname = fl_file_chooser("Pick a comment", 0L, 0L);
+	fl_file_chooser_ok_label(NULL);
+        if (fname) {
+          if (comment_input->buffer()->loadfile(fname)) {
+            fl_alert("Error loading file\n%s", fname);
+          }
+        }
+      }
+      else if (!w) Fl::wait();
+    }
+    char*c = comment_input->buffer()->text();
+    name(c);
+    free(c);
+    int mod = 0;
+    if (in_c_ != comment_in_source->value()) {
+      in_c_ = comment_in_source->value();
+      mod = 1;
+    }
+    if (in_h_ != comment_in_header->value()) {
+      in_h_ = comment_in_header->value();
+      mod = 1;
+    }
+    if (mod) set_modflag(1);
+    break;
+  }
+ BREAK2:
+  title_buf[0] = 0;
+  comment_panel->hide();
+}
+
+const char *Fl_Comment_Type::title() {
+  const char* n = name(); 
+  if (!n || !*n) return type_name();
+  if (title_buf[0]==0) {
+    const char *s = n;
+    char *d = title_buf;
+    int i = 50;
+    while (--i > 0) {
+      char n = *s++;
+      if (n==0) break;
+      if (n=='\r') { *d++ = '\\'; *d++ = 'r'; i--; }
+      else if (n=='\n') { *d++ = '\\'; *d++ = 'n'; i--; }
+      else if (n<32) { *d++ = '^'; *d++ = 'A'+n; i--; }
+      else *d++ = n;
+    }
+    if (i<=0) { *d++ = '.'; *d++ = '.'; *d++ = '.'; }
+    *d++ = 0;
+  }
+  return title_buf;
+}
+
+Fl_Comment_Type Fl_Comment_type;
+
+void Fl_Comment_Type::write_code1() {
+  const char* c = name();
+  if (!c) return;
+  if (!in_c_ && !in_h_) return;
+  // find out if there is already a valid comment:
+  const char *s = c;
+  while (isspace(*s)) s++;
+  // if this seems to be a C style comment, copy the block as is
+  // (it's up to the user to correctly close the comment)
+  if (s[0]=='/' && s[1]=='*') {
+    if (in_h_) write_h("%s\n", c);
+    if (in_c_) write_c("%s\n", c);
+    return;
+  }
+  // copy the comment line by line, add the double slash if needed
+  char *txt = strdup(c);
+  char *b = txt, *e = txt;
+  for (;;) {
+    // find the end of the line and set it to NUL
+    while (*e && *e!='\n') e++;
+    char eol = *e;
+    *e = 0;
+    // check if there is a C++ style comment at the beginning of the line
+    char *s = b;
+    while (isspace(*s)) s++;
+    if (s!=e && ( s[0]!='/' || s[1]!='/') ) {
+      // if no comment marker was found, we add one ourselves
+      if (in_h_) write_h("// ");
+      if (in_c_) write_c("// ");
+    }
+    // now copy the rest of the line
+    if (in_h_) write_h("%s\n", b);
+    if (in_c_) write_c("%s\n", b);
+    if (eol==0) break;
+    *e++ = eol;
+    b = e;
+  }
+}
+
+void Fl_Comment_Type::write_code2() {}
+
+////////////////////////////////////////////////////////////////
+
 const char* Fl_Type::class_name(const int need_nest) const {
   Fl_Type* p = parent;
   while (p) {
@@ -665,6 +1013,20 @@ const char* Fl_Type::class_name(const int need_nest) const {
   return 0;
 }
 
+/**
+ * If this Type resides inside a class, this function returns the class type, or null.
+ */
+const Fl_Class_Type *Fl_Type::is_in_class() const {
+  Fl_Type* p = parent;
+  while (p) {
+    if (p->is_class()) {
+      return (Fl_Class_Type*)p;
+    }
+    p = p->parent;
+  }
+  return 0;
+}
+
 int Fl_Class_Type::is_public() const {return public_;}
 
 void Fl_Class_Type::prefix(const char*p) {
@@ -691,12 +1053,17 @@ void Fl_Class_Type::write_properties() {
     write_string(":");
     write_word(subclass_of);
   }
-  if (!public_) write_string("private");
+  switch (public_) {
+    case 0: write_string("private"); break;
+    case 2: write_string("protected"); break;
+  }
 }
 
 void Fl_Class_Type::read_property(const char *c) {
   if (!strcmp(c,"private")) {
     public_ = 0;
+  } else if (!strcmp(c,"protected")) {
+    public_ = 2;
   } else if (!strcmp(c,":")) {
     storestring(read_word(), subclass_of);
   } else {
@@ -753,7 +1120,10 @@ void Fl_Class_Type::open() {
     prefix(pr);
     free((void*)s);
     storestring(c, subclass_of);
-    public_ = c_public_button->value();
+    if (public_ != c_public_button->value()) {
+      public_ = c_public_button->value();
+      set_modflag(1);
+    }
     break;
   }
  BREAK2:
@@ -762,13 +1132,19 @@ void Fl_Class_Type::open() {
 
 Fl_Class_Type Fl_Class_type;
 
-static Fl_Class_Type *current_class;
-extern int varused_test;
+Fl_Class_Type *current_class;
+extern Fl_Widget_Class_Type *current_widget_class;
 void write_public(int state) {
-  if (!current_class || varused_test) return;
-  if (current_class->write_public_state == state) return;
-  current_class->write_public_state = state;
-  write_h(state ? "public:\n" : "private:\n");
+  if (!current_class && !current_widget_class) return;
+  if (current_class && current_class->write_public_state == state) return;
+  if (current_widget_class && current_widget_class->write_public_state == state) return;
+  if (current_class) current_class->write_public_state = state;
+  if (current_widget_class) current_widget_class->write_public_state = state;
+  switch (state) {
+    case 0: write_h("private:\n"); break;
+    case 1: write_h("public:\n"); break;
+    case 2: write_h("protected:\n"); break;
+  }
 }
 
 void Fl_Class_Type::write_code1() {
@@ -788,6 +1164,21 @@ void Fl_Class_Type::write_code2() {
   current_class = parent_class;
 }
 
+/**
+ * Return 1 if this class contains a function with the given signature.
+ */
+int Fl_Class_Type::has_function(const char *rtype, const char *sig) const {
+  Fl_Type *child;
+  for (child = next; child && child->level > level; child = child->next) {
+    if (child->level == level+1 && strcmp(child->type_name(), "Function")==0) {
+      const Fl_Function_Type *fn = (const Fl_Function_Type*)child;
+      if (fn->has_signature(rtype, sig))
+        return 1;
+    }
+  }
+  return 0;
+}
+
 //
-// End of "$Id: Fl_Function_Type.cxx,v 1.15.2.16.2.16 2004/11/20 16:19:04 easysw Exp $".
+// End of "$Id: Fl_Function_Type.cxx 6013 2008-01-04 22:18:40Z matt $".
 //
