@@ -1,9 +1,9 @@
 //
-// "$Id: Fluid_Image.cxx 4607 2005-10-26 09:08:01Z matt $"
+// "$Id: Fluid_Image.cxx 5266 2006-07-26 19:52:28Z mike $"
 //
 // Pixmap label support for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2006 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -118,9 +118,9 @@ void Fluid_Image::write_static() {
   }
 }
 
-void Fluid_Image::write_code(int inactive) {
+void Fluid_Image::write_code(const char *var, int inactive) {
   if (!img) return;
-  write_c("%so->%s(%s);\n", indent(), inactive ? "deimage" : "image",
+  write_c("%s%s->%s(%s);\n", indent(), var, inactive ? "deimage" : "image",
 	  unique_id(this, "image", fl_filename_name(name()), 0));
 }
 
@@ -225,5 +225,5 @@ Fluid_Image *ui_find_image(const char *oldname) {
 
 
 //
-// End of "$Id: Fluid_Image.cxx 4607 2005-10-26 09:08:01Z matt $".
+// End of "$Id: Fluid_Image.cxx 5266 2006-07-26 19:52:28Z mike $".
 //
