@@ -1,5 +1,5 @@
 //
-// "$Id: Fluid_Image.h,v 1.3.2.4.2.5 2004/04/11 04:38:56 easysw Exp $"
+// "$Id: Fluid_Image.h 5266 2006-07-26 19:52:28Z mike $"
 //
 // Image header file for the Fast Light Tool Kit (FLTK).
 //
@@ -7,7 +7,7 @@
 // not a class in FLTK itself, and will produce different types of
 // code depending on what the image type is.
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -24,7 +24,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #ifndef FLUID_IMAGE_H
@@ -48,7 +50,7 @@ public:
   void image(Fl_Widget *); // set the image of this widget
   void deimage(Fl_Widget *); // set the deimage of this widget
   void write_static();
-  void write_code(int inactive = 0);
+  void write_code(const char *var, int inactive = 0);
   const char *name() const {return name_;}
 };
 
@@ -60,5 +62,5 @@ extern const char *ui_find_image_name;
 #endif
 
 //
-// End of "$Id: Fluid_Image.h,v 1.3.2.4.2.5 2004/04/11 04:38:56 easysw Exp $".
+// End of "$Id: Fluid_Image.h 5266 2006-07-26 19:52:28Z mike $".
 //
