@@ -1,9 +1,9 @@
 //
-// "$Id: code.cxx,v 1.9.2.9 2001/01/22 15:13:39 easysw Exp $"
+// "$Id: code.cxx 6013 2008-01-04 22:18:40Z matt $"
 //
 // Code output routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2006 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,13 +20,14 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
-#include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include "../src/flstring.h"
 #include <stdarg.h>
 
 #include <FL/Fl.H>
@@ -136,7 +137,7 @@ int write_declare(const char *format, ...) {
   va_list args;
   char buf[1024];
   va_start(args, format);
-  vsprintf(buf, format, args);
+  vsnprintf(buf, sizeof(buf), format, args);
   va_end(args);
   included **p = &included_root;
   while (*p) {
@@ -154,13 +155,16 @@ int write_declare(const char *format, ...) {
 
 // silly thing to prevent declaring unused variables:
 // When this symbol is on, all attempts to write code don't write
-// anything, but set a variable if it looks like the varaible "o" is used:
+// anything, but set a variable if it looks like the variable "o" is used:
 int varused_test;
 int varused;
 
 // write an array of C characters (adds a null):
 void write_cstring(const char *w, int length) {
-  if (varused_test) return;
+  if (varused_test) {
+    varused = 1;
+    return;
+  }
   const char *e = w+length;
   int linelength = 1;
   putc('\"', code_file);
@@ -225,8 +229,33 @@ void write_cstring(const char *w, int length) {
 // write a C string, quoting characters if necessary:
 void write_cstring(const char *w) {write_cstring(w,strlen(w));}
 
+// write an array of C binary data (does not add a null):
+void write_cdata(const char *s, int length) {
+  if (varused_test) {
+    varused = 1;
+    return;
+  }
+  const unsigned char *w = (const unsigned char *)s;
+  const unsigned char *e = w+length;
+  int linelength = 1;
+  putc('{', code_file);
+  for (; w < e;) {
+    unsigned char c = *w++;
+    if (c>99) linelength += 4;
+    else if (c>9) linelength += 3;
+    else linelength += 2;
+    if (linelength >= 77) {fputs("\n",code_file); linelength = 0;}
+    fprintf(code_file, "%d", c);
+    if (w<e) putc(',', code_file);
+  }
+  putc('}', code_file);
+}
+
 void write_c(const char* format,...) {
-  if (varused_test) {varused = 1; return;}
+  if (varused_test) {
+    varused = 1;
+    return;
+  }
   va_list args;
   va_start(args, format);
   vfprintf(code_file, format, args);
@@ -243,40 +272,111 @@ void write_h(const char* format,...) {
 
 #include <FL/filename.H>
 int write_number;
+int write_sourceview;
+extern Fl_Widget_Class_Type *current_widget_class;
 
 // recursively dump code, putting children between the two parts
 // of the parent code:
 static Fl_Type* write_code(Fl_Type* p) {
-  p->write_code1();
+  if (write_sourceview) {
+    p->code_line = (int)ftell(code_file);
+    if (p->header_line_end==-1)
+      p->header_line = (int)ftell(header_file);
+  }
+  // write all code that come before the children code
+  // (but don't write the last comment until the very end)
+  if (!(p==Fl_Type::last && p->is_comment()))
+    p->write_code1();
+  // recursively write the code of all children
   Fl_Type* q;
-  for (q = p->next; q && q->level > p->level;) q = write_code(q);
-  p->write_code2();
+  if (p->is_widget() && p->is_class()) {
+    // Handle widget classes specially
+    for (q = p->next; q && q->level > p->level;) {
+      if (strcmp(q->type_name(), "Function")) q = write_code(q);
+      else {
+        int level = q->level;
+	do {
+	  q = q->next;
+	} while (q && q->level > level);
+      }
+    }
+
+    // write all code that come after the children 
+    p->write_code2();
+
+    for (q = p->next; q && q->level > p->level;) {
+      if (!strcmp(q->type_name(), "Function")) q = write_code(q);
+      else {
+        int level = q->level;
+	do {
+	  q = q->next;
+	} while (q && q->level > level);
+      }
+    }
+
+    write_h("};\n");
+    current_widget_class = 0L;
+  } else {
+    for (q = p->next; q && q->level > p->level;) q = write_code(q);
+    // write all code that come after the children 
+    p->write_code2();
+  }
+  if (write_sourceview) {
+    p->code_line_end = (int)ftell(code_file);
+    if (p->header_line_end==-1)
+      p->header_line_end = (int)ftell(header_file);
+  }
   return q;
 }
 
+extern const char* header_file_name;
+extern Fl_Class_Type *current_class;
+
 int write_code(const char *s, const char *t) {
+  const char *filemode = "w";
+  if (write_sourceview) 
+    filemode = "wb";
   write_number++;
   delete id_root; id_root = 0;
   indentation = 0;
+  current_class = 0L;
+  current_widget_class = 0L;
   if (!s) code_file = stdout;
   else {
-    FILE *f = fopen(s,"w");
+    FILE *f = fopen(s, filemode);
     if (!f) return 0;
     code_file = f;
   }
   if (!t) header_file = stdout;
   else {
-    FILE *f = fopen(t,"w");
+    FILE *f = fopen(t, filemode);
     if (!f) {fclose(code_file); return 0;}
     header_file = f;
   }
+  // if the first entry in the Type tree is a comment, then it is probably 
+  // a copyright notice. We print that before anything else in the file!
+  Fl_Type* first_type = Fl_Type::first;
+  if (first_type && first_type->is_comment()) {
+    if (write_sourceview) {
+      first_type->code_line = (int)ftell(code_file);
+      first_type->header_line = (int)ftell(header_file);
+    }
+    // it is ok to write non-recusive code here, because comments have no children or code2 blocks
+    first_type->write_code1();
+    if (write_sourceview) {
+      first_type->code_line_end = (int)ftell(code_file);
+      first_type->header_line_end = (int)ftell(header_file);
+    }
+    first_type = first_type->next;
+  }
+
   const char *hdr = "\
 // generated by Fast Light User Interface Designer (fluid) version %.4f\n\n";
   fprintf(header_file, hdr, FL_VERSION);
   fprintf(code_file, hdr, FL_VERSION);
 
   {char define_name[102];
-  const char* a = filename_name(t);
+  const char* a = fl_filename_name(t);
   char* b = define_name;
   if (!isalpha(*a)) {*b++ = '_';}
   while (*a) {*b++ = isalnum(*a) ? *a : '_'; a++;}
@@ -303,13 +403,29 @@ int write_code(const char *s, const char *t) {
       }
     }
   }
-  if (t && include_H_from_C)
-    write_c("#include \"%s\"\n", filename_name(t));
-  for (Fl_Type* p = Fl_Type::first; p;) {
+  if (t && include_H_from_C) {
+    if (*header_file_name == '.' && strchr(header_file_name, '/') == NULL) {
+      write_c("#include \"%s\"\n", fl_filename_name(t));
+    } else {
+      write_c("#include \"%s\"\n", t);
+    }
+  }
+  for (Fl_Type* p = first_type; p;) {
     // write all static data for this & all children first
+    if (write_sourceview) p->header_line = (int)ftell(header_file);
     p->write_static();
-    for (Fl_Type* q = p->next; q && q->level > p->level; q = q->next)
+    if (write_sourceview) {
+      p->header_line_end = (int)ftell(header_file);
+      if (p->header_line==p->header_line_end) p->header_line_end = -1;
+    }
+    for (Fl_Type* q = p->next; q && q->level > p->level; q = q->next) {
+      if (write_sourceview) q->header_line = (int)ftell(header_file);
       q->write_static();
+      if (write_sourceview) {
+        q->header_line_end = (int)ftell(header_file);
+        if (q->header_line==q->header_line_end) q->header_line_end = -1;
+      }
+    }
     // then write the nested code:
     p = write_code(p);
   }
@@ -317,9 +433,24 @@ int write_code(const char *s, const char *t) {
   delete included_root; included_root = 0;
 
   if (!s) return 1;
+
+  fprintf(header_file, "#endif\n");
+
+  Fl_Type* last_type = Fl_Type::last;
+  if (last_type && last_type->is_comment()) {
+    if (write_sourceview) {
+      last_type->code_line = (int)ftell(code_file);
+      last_type->header_line = (int)ftell(header_file);
+    }
+    last_type->write_code1();
+    if (write_sourceview) {
+      last_type->code_line_end = (int)ftell(code_file);
+      last_type->header_line_end = (int)ftell(header_file);
+    }
+  }
+
   int x = fclose(code_file);
   code_file = 0;
-  fprintf(header_file, "#endif\n");
   int y = fclose(header_file);
   header_file = 0;
   return x >= 0 && y >= 0;
@@ -328,6 +459,7 @@ int write_code(const char *s, const char *t) {
 int write_strings(const char *sfile) {
   FILE *fp = fopen(sfile, "w");
   Fl_Type *p;
+  Fl_Widget_Type *w;
   int i;
 
   if (!fp) return 1;
@@ -337,13 +469,26 @@ int write_strings(const char *sfile) {
       fprintf(fp, "# generated by Fast Light User Interface Designer (fluid) version %.4f\n",
 	      FL_VERSION);
       for (p = Fl_Type::first; p; p = p->next) {
-        if (p->is_widget() && p->label() && !((Fl_Widget_Type *)p)->image) {
-	  for (const char *s = p->label(); *s; s ++)
-	    if (*s < 32 || *s > 126 || *s == '\"')
-	      fprintf(fp, "\\%03o", *s);
-	    else
-	      putc(*s, fp);
-          putc('\n', fp);
+        if (p->is_widget()) {
+	  w = (Fl_Widget_Type *)p;
+
+	  if (w->label()) {
+	    for (const char *s = w->label(); *s; s ++)
+	      if (*s < 32 || *s > 126 || *s == '\"')
+		fprintf(fp, "\\%03o", *s);
+	      else
+		putc(*s, fp);
+            putc('\n', fp);
+	  }
+
+	  if (w->tooltip()) {
+	    for (const char *s = w->tooltip(); *s; s ++)
+	      if (*s < 32 || *s > 126 || *s == '\"')
+		fprintf(fp, "\\%03o", *s);
+	      else
+		putc(*s, fp);
+            putc('\n', fp);
+	  }
 	}
       }
       break;
@@ -351,24 +496,48 @@ int write_strings(const char *sfile) {
       fprintf(fp, "# generated by Fast Light User Interface Designer (fluid) version %.4f\n",
 	      FL_VERSION);
       for (p = Fl_Type::first; p; p = p->next) {
-        if (p->is_widget() && p->label() && !((Fl_Widget_Type *)p)->image) {
-	  const char *s;
+        if (p->is_widget()) {
+	  w = (Fl_Widget_Type *)p;
 
-	  fputs("msgid \"", fp);
-	  for (s = p->label(); *s; s ++)
-	    if (*s < 32 || *s > 126 || *s == '\"')
-	      fprintf(fp, "\\%03o", *s);
-	    else
-	      putc(*s, fp);
-          fputs("\"\n", fp);
+	  if (w->label()) {
+	    const char *s;
 
-	  fputs("msgstr \"", fp);
-	  for (s = p->label(); *s; s ++)
-	    if (*s < 32 || *s > 126 || *s == '\"')
-	      fprintf(fp, "\\%03o", *s);
-	    else
-	      putc(*s, fp);
-          fputs("\"\n", fp);
+	    fputs("msgid \"", fp);
+	    for (s = w->label(); *s; s ++)
+	      if (*s < 32 || *s > 126 || *s == '\"')
+		fprintf(fp, "\\%03o", *s);
+	      else
+		putc(*s, fp);
+            fputs("\"\n", fp);
+
+	    fputs("msgstr \"", fp);
+	    for (s = w->label(); *s; s ++)
+	      if (*s < 32 || *s > 126 || *s == '\"')
+		fprintf(fp, "\\%03o", *s);
+	      else
+		putc(*s, fp);
+            fputs("\"\n", fp);
+	  }
+
+	  if (w->tooltip()) {
+	    const char *s;
+
+	    fputs("msgid \"", fp);
+	    for (s = w->tooltip(); *s; s ++)
+	      if (*s < 32 || *s > 126 || *s == '\"')
+		fprintf(fp, "\\%03o", *s);
+	      else
+		putc(*s, fp);
+            fputs("\"\n", fp);
+
+	    fputs("msgstr \"", fp);
+	    for (s = w->tooltip(); *s; s ++)
+	      if (*s < 32 || *s > 126 || *s == '\"')
+		fprintf(fp, "\\%03o", *s);
+	      else
+		putc(*s, fp);
+            fputs("\"\n", fp);
+	  }
 	}
       }
       break;
@@ -379,14 +548,28 @@ int write_strings(const char *sfile) {
       fputs("$quote \"\n", fp);
 
       for (i = 1, p = Fl_Type::first; p; p = p->next) {
-        if (p->is_widget() && p->label() && !((Fl_Widget_Type *)p)->image) {
-	  fprintf(fp, "%d \"", i ++);
-	  for (const char *s = p->label(); *s; s ++)
-	    if (*s < 32 || *s > 126 || *s == '\"')
-	      fprintf(fp, "\\%03o", *s);
-	    else
-	      putc(*s, fp);
-          fputs("\"\n", fp);
+        if (p->is_widget()) {
+	  w = (Fl_Widget_Type *)p;
+
+	  if (w->label()) {
+	    fprintf(fp, "%d \"", i ++);
+	    for (const char *s = w->label(); *s; s ++)
+	      if (*s < 32 || *s > 126 || *s == '\"')
+		fprintf(fp, "\\%03o", *s);
+	      else
+		putc(*s, fp);
+            fputs("\"\n", fp);
+	  }
+
+	  if (w->tooltip()) {
+	    fprintf(fp, "%d \"", i ++);
+	    for (const char *s = w->tooltip(); *s; s ++)
+	      if (*s < 32 || *s > 126 || *s == '\"')
+		fprintf(fp, "\\%03o", *s);
+	      else
+		putc(*s, fp);
+            fputs("\"\n", fp);
+	  }
 	}
       }
       break;
@@ -405,5 +588,5 @@ void Fl_Type::write_code1() {
 void Fl_Type::write_code2() {}
 
 //
-// End of "$Id: code.cxx,v 1.9.2.9 2001/01/22 15:13:39 easysw Exp $".
+// End of "$Id: code.cxx 6013 2008-01-04 22:18:40Z matt $".
 //
