@@ -1,5 +1,5 @@
 //
-// "$Id: file.cxx 4288 2005-04-16 00:13:17Z mike $"
+// "$Id: file.cxx 5808 2007-05-10 12:06:31Z matt $"
 //
 // Fluid file routines for the Fast Light Tool Kit (FLTK).
 //
@@ -8,7 +8,7 @@
 // They are somewhat similar to tcl, using matching { and }
 // to quote strings.
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2006 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -101,7 +101,7 @@ void write_string(const char *format, ...) {
   if (needspace) fputc(' ',fout);
   vfprintf(fout, format, args);
   va_end(args);
-  needspace = !isspace(format[strlen(format)-1]);
+  needspace = !isspace(format[strlen(format)-1] & 255);
 }
 
 // start a new line and indent it for a given nesting level:
@@ -238,7 +238,7 @@ const char *read_word(int wantbrace) {
   // skip all the whitespace before it:
   for (;;) {
     x = getc(fin);
-    if (x < 0) {	// eof
+    if (x < 0 && feof(fin)) {	// eof
       return 0;
     } else if (x == '#') {	// comment
       do x = getc(fin); while (x >= 0 && x != '\n');
@@ -246,7 +246,7 @@ const char *read_word(int wantbrace) {
       continue;
     } else if (x == '\n') {
       lineno++;
-    } else if (!isspace(x)) {
+    } else if (!isspace(x & 255)) {
       break;
     }
   }
@@ -287,7 +287,7 @@ const char *read_word(int wantbrace) {
     int length = 0;
     for (;;) {
       if (x == '\\') {x = read_quoted(); if (x<0) continue;}
-      else if (x<0 || isspace(x) || x=='{' || x=='}' || x=='#') break;
+      else if (x<0 || isspace(x & 255) || x=='{' || x=='}' || x=='#') break;
       buffer[length++] = x;
       expand_buffer(length);
       x = getc(fin);
@@ -323,6 +323,8 @@ int write_file(const char *filename, int selected_only) {
 	       "version %.4f",FL_VERSION);
   if(!include_H_from_C)
     write_string("\ndo_not_include_H_from_C");
+  if(use_FL_COMMAND)
+    write_string("\nuse_FL_COMMAND");
   if (i18n_type) {
     write_string("\ni18n_type %d", i18n_type);
     write_string("\ni18n_include %s", i18n_include);
@@ -404,6 +406,10 @@ static void read_children(Fl_Type *p, int paste) {
       include_H_from_C=0;
       goto CONTINUE;
     }
+    if (!strcmp(c,"use_FL_COMMAND")) {
+      use_FL_COMMAND=1;
+      goto CONTINUE;
+    }
     if (!strcmp(c,"i18n_type")) {
       i18n_type = atoi(read_word());
       goto CONTINUE;
@@ -504,6 +510,7 @@ int read_file(const char *filename, int merge) {
     if (o->is_menu_button()) o->add_child(0,0);
   for (o = Fl_Type::first; o; o = o->next)
     if (o->selected) {Fl_Type::current = o; break;}
+  selection_changed(Fl_Type::current);
   return close_read();
 }
 
@@ -517,9 +524,9 @@ int read_fdesign_line(const char*& name, const char*& value) {
   // find a colon:
   for (;;) {
     x = getc(fin);
-    if (x < 0) return 0;
+    if (x < 0 && feof(fin)) return 0;
     if (x == '\n') {length = 0; continue;} // no colon this line...
-    if (!isspace(x)) {
+    if (!isspace(x & 255)) {
       buffer[length++] = x;
       expand_buffer(length);
     }
@@ -531,7 +538,7 @@ int read_fdesign_line(const char*& name, const char*& value) {
   // skip to start of value:
   for (;;) {
     x = getc(fin);
-    if (x < 0 || x == '\n' || !isspace(x)) break;
+    if ((x < 0 && feof(fin)) || x == '\n' || !isspace(x & 255)) break;
   }
 
   // read the value:
@@ -639,5 +646,5 @@ void read_fdesign() {
 }
 
 //
-// End of "$Id: file.cxx 4288 2005-04-16 00:13:17Z mike $".
+// End of "$Id: file.cxx 5808 2007-05-10 12:06:31Z matt $".
 //
