@@ -1,9 +1,9 @@
 //
-// "$Id: fluid.cxx 4660 2005-11-27 14:45:48Z mike $"
+// "$Id: fluid.cxx 6783 2009-04-27 15:50:29Z AlbrechtS $"
 //
 // FLUID main entry for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2007 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -59,11 +59,13 @@
 #  include <io.h>
 #  include <commdlg.h>
 #  include <FL/x.H>
+#  ifndef __WATCOMC__
 // Visual C++ 2005 incorrectly displays a warning about the use of POSIX APIs
 // on Windows, which is supposed to be POSIX compliant...
-#  define access _access
-#  define chdir _chdir
-#  define getcwd _getcwd
+#    define access _access
+#    define chdir _chdir
+#    define getcwd _getcwd
+#  endif // !__WATCOMC__
 #else
 #  include <unistd.h>
 #endif
@@ -1014,6 +1016,14 @@ void print_menu_cb(Fl_Widget *, void *) {
     win    = (Fl_Window *)(windows[winpage]->o);
     pixels = windows[winpage]->read_image(w, h);
 
+    // Swap colors: FLTK uses R-G-B --> Windows GDI uses B-G-R
+
+    { uchar *p = pixels;
+      for (int i=0; i<w*h; i++, p+=3) {
+	uchar temp = p[0]; p[0] = p[2]; p[2] = temp;
+      }
+    }
+
     // Figure out the window size, first at 100 PPI and then scaled
     // down if that is too big...
     ww = w * xdpi / 100;
@@ -1567,48 +1577,48 @@ void toggle_sourceview_cb(Fl_Double_Window *, void *);
 
 Fl_Menu_Item Main_Menu[] = {
 {"&File",0,0,0,FL_SUBMENU},
-  {"&New...", FL_CTRL+'n', new_cb, 0},
-  {"&Open...", FL_CTRL+'o', open_cb, 0},
-  {"&Insert...", FL_CTRL+'i', open_cb, (void*)1, FL_MENU_DIVIDER},
+  {"&New...", FL_COMMAND+'n', new_cb, 0},
+  {"&Open...", FL_COMMAND+'o', open_cb, 0},
+  {"&Insert...", FL_COMMAND+'i', open_cb, (void*)1, FL_MENU_DIVIDER},
 #define SAVE_ITEM 4
-  {"&Save", FL_CTRL+'s', save_cb, 0},
-  {"Save &As...", FL_CTRL+FL_SHIFT+'s', save_cb, (void*)1},
+  {"&Save", FL_COMMAND+'s', save_cb, 0},
+  {"Save &As...", FL_COMMAND+FL_SHIFT+'s', save_cb, (void*)1},
   {"Sa&ve A Copy...", 0, save_cb, (void*)2},
   {"Save &Template...", 0, save_template_cb},
   {"&Revert...", 0, revert_cb, 0, FL_MENU_DIVIDER},
-  {"&Print...", FL_CTRL+'p', print_menu_cb},
-  {"Write &Code...", FL_CTRL+FL_SHIFT+'c', write_cb, 0},
-  {"&Write Strings...", FL_CTRL+FL_SHIFT+'w', write_strings_cb, 0, FL_MENU_DIVIDER},
+  {"&Print...", FL_COMMAND+'p', print_menu_cb},
+  {"Write &Code...", FL_COMMAND+FL_SHIFT+'c', write_cb, 0},
+  {"&Write Strings...", FL_COMMAND+FL_SHIFT+'w', write_strings_cb, 0, FL_MENU_DIVIDER},
 #define HISTORY_ITEM 12
-  {relative_history[0], FL_CTRL+'0', open_history_cb, absolute_history[0]},
-  {relative_history[1], FL_CTRL+'1', open_history_cb, absolute_history[1]},
-  {relative_history[2], FL_CTRL+'2', open_history_cb, absolute_history[2]},
-  {relative_history[3], FL_CTRL+'3', open_history_cb, absolute_history[3]},
-  {relative_history[4], FL_CTRL+'4', open_history_cb, absolute_history[4]},
-  {relative_history[5], FL_CTRL+'5', open_history_cb, absolute_history[5]},
-  {relative_history[6], FL_CTRL+'6', open_history_cb, absolute_history[6]},
-  {relative_history[7], FL_CTRL+'7', open_history_cb, absolute_history[7]},
-  {relative_history[8], FL_CTRL+'8', open_history_cb, absolute_history[8]},
-  {relative_history[9], FL_CTRL+'9', open_history_cb, absolute_history[9], FL_MENU_DIVIDER},
-  {"&Quit", FL_CTRL+'q', exit_cb},
+  {relative_history[0], FL_COMMAND+'0', open_history_cb, absolute_history[0]},
+  {relative_history[1], FL_COMMAND+'1', open_history_cb, absolute_history[1]},
+  {relative_history[2], FL_COMMAND+'2', open_history_cb, absolute_history[2]},
+  {relative_history[3], FL_COMMAND+'3', open_history_cb, absolute_history[3]},
+  {relative_history[4], FL_COMMAND+'4', open_history_cb, absolute_history[4]},
+  {relative_history[5], FL_COMMAND+'5', open_history_cb, absolute_history[5]},
+  {relative_history[6], FL_COMMAND+'6', open_history_cb, absolute_history[6]},
+  {relative_history[7], FL_COMMAND+'7', open_history_cb, absolute_history[7]},
+  {relative_history[8], FL_COMMAND+'8', open_history_cb, absolute_history[8]},
+  {relative_history[9], FL_COMMAND+'9', open_history_cb, absolute_history[9], FL_MENU_DIVIDER},
+  {"&Quit", FL_COMMAND+'q', exit_cb},
   {0},
 {"&Edit",0,0,0,FL_SUBMENU},
-  {"&Undo", FL_CTRL+'z', undo_cb},
-  {"&Redo", FL_CTRL+FL_SHIFT+'z', redo_cb, 0, FL_MENU_DIVIDER},
-  {"C&ut", FL_CTRL+'x', cut_cb},
-  {"&Copy", FL_CTRL+'c', copy_cb},
-  {"&Paste", FL_CTRL+'v', paste_cb},
-  {"Dup&licate", FL_CTRL+'u', duplicate_cb},
+  {"&Undo", FL_COMMAND+'z', undo_cb},
+  {"&Redo", FL_COMMAND+FL_SHIFT+'z', redo_cb, 0, FL_MENU_DIVIDER},
+  {"C&ut", FL_COMMAND+'x', cut_cb},
+  {"&Copy", FL_COMMAND+'c', copy_cb},
+  {"&Paste", FL_COMMAND+'v', paste_cb},
+  {"Dup&licate", FL_COMMAND+'u', duplicate_cb},
   {"&Delete", FL_Delete, delete_cb, 0, FL_MENU_DIVIDER},
-  {"Select &All", FL_CTRL+'a', select_all_cb},
-  {"Select &None", FL_CTRL+FL_SHIFT+'a', select_none_cb, 0, FL_MENU_DIVIDER},
+  {"Select &All", FL_COMMAND+'a', select_all_cb},
+  {"Select &None", FL_COMMAND+FL_SHIFT+'a', select_none_cb, 0, FL_MENU_DIVIDER},
   {"Pr&operties...", FL_F+1, openwidget_cb},
   {"&Sort",0,sort_cb},
   {"&Earlier", FL_F+2, earlier_cb},
   {"&Later", FL_F+3, later_cb},
   {"&Group", FL_F+7, group_cb},
   {"Ung&roup", FL_F+8, ungroup_cb,0, FL_MENU_DIVIDER},
-  {"Hide O&verlays",FL_CTRL+FL_SHIFT+'o',toggle_overlays},
+  {"Hide O&verlays",FL_COMMAND+FL_SHIFT+'o',toggle_overlays},
 #define WIDGETBIN_ITEM 41
   {"Show Widget &Bin...",FL_ALT+'b',toggle_widgetbin_cb},
 #define SOURCEVIEW_ITEM 42
@@ -1647,7 +1657,7 @@ Fl_Menu_Item Main_Menu[] = {
     {"&Large",FL_ALT+'5',(Fl_Callback *)widget_size_cb,(void*)24,0,FL_NORMAL_LABEL,FL_HELVETICA,24},
     {"&Huge",FL_ALT+'6',(Fl_Callback *)widget_size_cb,(void*)32,0,FL_NORMAL_LABEL,FL_HELVETICA,32},
     {0},
-  {"&Grid and Size Settings...",FL_CTRL+'g',show_grid_cb},
+  {"&Grid and Size Settings...",FL_COMMAND+'g',show_grid_cb},
   {0},
 {"&Shell",0,0,0,FL_SUBMENU},
   {"Execute &Command...",FL_ALT+'x',(Fl_Callback *)show_shell_window},
@@ -1682,6 +1692,9 @@ void scheme_cb(Fl_Choice *, void *) {
     case 2 : // Plastic
       Fl::scheme("plastic");
       break;
+    case 3 : // GTK+
+      Fl::scheme("gtk+");
+      break;
   }
 
   fluid_prefs.set("scheme", scheme_choice->value());
@@ -2236,5 +2249,5 @@ int main(int argc,char **argv) {
 }
 
 //
-// End of "$Id: fluid.cxx 4660 2005-11-27 14:45:48Z mike $".
+// End of "$Id: fluid.cxx 6783 2009-04-27 15:50:29Z AlbrechtS $".
 //
