@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Adjuster.cxx,v 1.5.2.3.2.10 2004/04/11 04:38:57 easysw Exp $"
+// "$Id: Fl_Adjuster.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // Adjuster widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 
@@ -159,11 +161,11 @@ Fl_Adjuster::Fl_Adjuster(int X, int Y, int W, int H, const char* l)
   : Fl_Valuator(X, Y, W, H, l) {
   box(FL_UP_BOX);
   step(1, 10000);
-  selection_color(FL_BLACK);
+  selection_color(FL_SELECTION_COLOR);
   drag = 0;
   soft_ = 1;
 }
 
 //
-// End of "$Id: Fl_Adjuster.cxx,v 1.5.2.3.2.10 2004/04/11 04:38:57 easysw Exp $".
+// End of "$Id: Fl_Adjuster.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
