@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Browser.cxx,v 1.9.2.12.2.13 2004/11/20 03:19:58 easysw Exp $"
+// "$Id: Fl_Browser.cxx 5987 2007-11-20 21:57:17Z mike $"
 //
 // Browser widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2007 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
@@ -470,7 +472,7 @@ void Fl_Browser::show(int line) {
   if (t->flags & NOTDISPLAYED) {
     t->flags &= ~NOTDISPLAYED;
     full_height_ += item_height(t);
-    if (Fl_Browser_::displayed(t)) redraw_lines();
+    if (Fl_Browser_::displayed(t)) redraw();
   }
 }
 
@@ -479,7 +481,7 @@ void Fl_Browser::hide(int line) {
   if (!(t->flags & NOTDISPLAYED)) {
     full_height_ -= item_height(t);
     t->flags |= NOTDISPLAYED;
-    if (Fl_Browser_::displayed(t)) redraw_lines();
+    if (Fl_Browser_::displayed(t)) redraw();
   }
 }
 
@@ -501,6 +503,7 @@ int Fl_Browser::value() const {
 void Fl_Browser::swap(FL_BLINE *a, FL_BLINE *b) {
 
   if ( a == b || !a || !b) return;          // nothing to do
+  swapping(a, b);
   FL_BLINE *aprev  = a->prev;
   FL_BLINE *anext  = a->next;
   FL_BLINE *bprev  = b->prev;
@@ -533,9 +536,7 @@ void Fl_Browser::swap(FL_BLINE *a, FL_BLINE *b) {
   }
   // Disable cache -- we played around with positions
   cacheline = 0;
-  // Redraw modified lines
-  redraw_line(a);
-  redraw_line(b);
+  cache = 0;
 }
 
 void Fl_Browser::swap(int ai, int bi) {
@@ -546,5 +547,5 @@ void Fl_Browser::swap(int ai, int bi) {
 }
 
 //
-// End of "$Id: Fl_Browser.cxx,v 1.9.2.12.2.13 2004/11/20 03:19:58 easysw Exp $".
+// End of "$Id: Fl_Browser.cxx 5987 2007-11-20 21:57:17Z mike $".
 //
