@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Button.cxx,v 1.4.2.6.2.23 2004/07/27 16:02:20 easysw Exp $"
+// "$Id: Fl_Button.cxx 6589 2008-12-17 10:47:09Z fabien $"
 //
 // Button widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2006 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
@@ -38,7 +40,8 @@ int Fl_Button::value(int v) {
   clear_changed();
   if (value_ != v) {
     value_ = v;
-    redraw();
+    if (box()) redraw();
+    else redraw_label();
     return 1;
   } else {
     return 0;
@@ -58,9 +61,13 @@ void Fl_Button::setonly() { // set this radio button on, turn others off
 void Fl_Button::draw() {
   if (type() == FL_HIDDEN_BUTTON) return;
   Fl_Color col = value() ? selection_color() : color();
-//if (col == FL_GRAY && Fl::belowmouse()==this) col = FL_LIGHT1;
   draw_box(value() ? (down_box()?down_box():fl_down(box())) : box(), col);
-  draw_label();
+  if (labeltype() == FL_NORMAL_LABEL && value()) {
+    Fl_Color c = labelcolor();
+    labelcolor(fl_contrast(c, col));
+    draw_label();
+    labelcolor(c);
+  } else draw_label();
   if (Fl::focus() == this) draw_focus();
 }
 
@@ -78,7 +85,10 @@ int Fl_Button::handle(int event) {
       if (type() == FL_RADIO_BUTTON) newval = 1;
       else newval = !oldval;
     } else
+    {
+      clear_changed();
       newval = oldval;
+    }
     if (newval != value_) {
       value_ = newval;
       set_changed();
@@ -96,6 +106,7 @@ int Fl_Button::handle(int event) {
     else if (type() == FL_TOGGLE_BUTTON) oldval = value_;
     else {
       value(oldval);
+      set_changed();
       if (when() & FL_WHEN_CHANGED) do_callback();
     }
     if (when() & FL_WHEN_RELEASE) do_callback();
@@ -109,11 +120,13 @@ int Fl_Button::handle(int event) {
     if (type() == FL_RADIO_BUTTON && !value_) {
       setonly();
       set_changed();
-      if (when() & FL_WHEN_CHANGED) do_callback();
+      if (when() & (FL_WHEN_CHANGED|FL_WHEN_RELEASE)) 
+	do_callback();
     } else if (type() == FL_TOGGLE_BUTTON) {
       value(!value());
       set_changed();
-      if (when() & FL_WHEN_CHANGED) do_callback();
+      if (when() & (FL_WHEN_CHANGED|FL_WHEN_RELEASE)) 
+	do_callback();
     } else if (when() & FL_WHEN_RELEASE) do_callback();
     return 1;
   case FL_FOCUS :
@@ -158,5 +171,5 @@ Fl_Button::Fl_Button(int X, int Y, int W, int H, const char *l)
 }
 
 //
-// End of "$Id: Fl_Button.cxx,v 1.4.2.6.2.23 2004/07/27 16:02:20 easysw Exp $".
+// End of "$Id: Fl_Button.cxx 6589 2008-12-17 10:47:09Z fabien $".
 //
