@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Chart.cxx,v 1.5.2.6 2001/01/22 15:13:39 easysw Exp $"
+// "$Id: Fl_Chart.cxx 5942 2007-10-06 17:33:17Z matt $"
 //
 // Forms-compatible chart widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,19 +20,21 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/math.h>
 #include <FL/Fl.H>
 #include <FL/Fl_Chart.H>
 #include <FL/fl_draw.H>
-#include <string.h>
+#include "flstring.h"
 #include <stdlib.h>
 
 #define ARCINC	(2.0*M_PI/360.0)
 
-// this function is in fl_boxtype.C:
+// this function is in fl_boxtype.cxx:
 void fl_rectbound(int x,int y,int w,int h, Fl_Color color);
 
 /* Widget specific information */
@@ -53,9 +55,9 @@ static void draw_barchart(int x,int y,int w,int h,
       incr = (h - lh + min*incr)/(max-min);
       zeroh = int(y+h-lh);
   } else {
-      zeroh = int(y+h+min * incr + .5);
+      zeroh = (int)rint(y+h+min * incr);
   }
-  int bwidth = int(w/double(autosize?numb:maxnumb)+.5);
+  int bwidth = (int)rint(w/double(autosize?numb:maxnumb));
   /* Draw base line */
   fl_color(textcolor);
   fl_line(x, zeroh, x+w, zeroh);
@@ -63,11 +65,11 @@ static void draw_barchart(int x,int y,int w,int h,
   int i;
   /* Draw the bars */
   for (i=0; i<numb; i++) {
-      int h = int(entries[i].val*incr+.5);
-      if (h < 0)
-	fl_rectbound(x+i*bwidth,zeroh,bwidth+1,-h+1, (Fl_Color)entries[i].col);
-      else if (h > 0)
-	fl_rectbound(x+i*bwidth,zeroh-h,bwidth+1,h+1,(Fl_Color)entries[i].col);
+      int hh = (int)rint(entries[i].val*incr);
+      if (hh < 0)
+	fl_rectbound(x+i*bwidth,zeroh,bwidth+1,-hh+1, (Fl_Color)entries[i].col);
+      else if (hh > 0)
+	fl_rectbound(x+i*bwidth,zeroh-hh,bwidth+1,hh+1,(Fl_Color)entries[i].col);
   }
   /* Draw the labels */
   fl_color(textcolor);
@@ -98,22 +100,22 @@ static void draw_horbarchart(int x,int y,int w,int h,
   else incr = w/(max-min);
   if ( (-min*incr) < lw) {
       incr = (w - lw + min*incr)/(max-min);
-      zeroh = x+int(lw+.5);
+      zeroh = x+(int)rint(lw);
   } else {
-      zeroh = int(x-min * incr + .5);
+      zeroh = (int)rint(x-min * incr);
   }
-  int bwidth = int(h/double(autosize?numb:maxnumb)+.5);
+  int bwidth = (int)rint(h/double(autosize?numb:maxnumb));
   /* Draw base line */
   fl_color(textcolor);
   fl_line(zeroh, y, zeroh, y+h);
   if (min == 0.0 && max == 0.0) return; /* Nothing else to draw */
   /* Draw the bars */
   for (i=0; i<numb; i++) {
-      int w = int(entries[i].val*incr+.5);
-      if (w > 0)
-	fl_rectbound(zeroh,y+i*bwidth,w+1,bwidth+1, (Fl_Color)entries[i].col);
-      else if (w < 0)
-	fl_rectbound(zeroh+w,y+i*bwidth,-w+1,bwidth+1,(Fl_Color)entries[i].col);
+      int ww = (int)rint(entries[i].val*incr);
+      if (ww > 0)
+	fl_rectbound(zeroh,y+i*bwidth,ww+1,bwidth+1, (Fl_Color)entries[i].col);
+      else if (ww < 0)
+	fl_rectbound(zeroh+ww,y+i*bwidth,-ww+1,bwidth+1,(Fl_Color)entries[i].col);
   }
   /* Draw the labels */
   fl_color(textcolor);
@@ -135,32 +137,32 @@ static void draw_linechart(int type, int x,int y,int w,int h,
   double incr;
   if (max == min) incr = h-2.0*lh;
   else incr = (h-2.0*lh)/ (max-min);
-  int zeroh = int(y+h-lh+min * incr + .5);
+  int zeroh = (int)rint(y+h-lh+min * incr);
   double bwidth = w/double(autosize?numb:maxnumb);
   /* Draw the values */
   for (i=0; i<numb; i++) {
-      int x0 = x + int((i-.5)*bwidth+.5);
-      int x1 = x + int((i+.5)*bwidth+.5);
-      int y0 = i ? zeroh - int(entries[i-1].val*incr+.5) : 0;
-      int y1 = zeroh - int(entries[i].val*incr+.5);
+      int x0 = x + (int)rint((i-.5)*bwidth);
+      int x1 = x + (int)rint((i+.5)*bwidth);
+      int yy0 = i ? zeroh - (int)rint(entries[i-1].val*incr) : 0;
+      int yy1 = zeroh - (int)rint(entries[i].val*incr);
       if (type == FL_SPIKE_CHART) {
 	  fl_color((Fl_Color)entries[i].col);
-	  fl_line(x1, zeroh, x1, y1);
+	  fl_line(x1, zeroh, x1, yy1);
       } else if (type == FL_LINE_CHART && i != 0) {
 	  fl_color((Fl_Color)entries[i-1].col);
-	  fl_line(x0,y0,x1,y1);
+	  fl_line(x0,yy0,x1,yy1);
       } else if (type == FL_FILLED_CHART && i != 0) {
 	  fl_color((Fl_Color)entries[i-1].col);
 	  if ((entries[i-1].val>0.0)!=(entries[i].val>0.0)) {
 	      double ttt = entries[i-1].val/(entries[i-1].val-entries[i].val);
-	      int xt = x + int((i-.5+ttt)*bwidth+.5);
-	      fl_polygon(x0,zeroh, x0,y0, xt,zeroh);
-	      fl_polygon(xt,zeroh, x1,y1, x1,zeroh);
+	      int xt = x + (int)rint((i-.5+ttt)*bwidth);
+	      fl_polygon(x0,zeroh, x0,yy0, xt,zeroh);
+	      fl_polygon(xt,zeroh, x1,yy1, x1,zeroh);
 	  } else {
-	      fl_polygon(x0,zeroh, x0,y0, x1,y1, x1,zeroh);
+	      fl_polygon(x0,zeroh, x0,yy0, x1,yy1, x1,zeroh);
 	  }
 	  fl_color(textcolor);
-	  fl_line(x0,y0,x1,y1);
+	  fl_line(x0,yy0,x1,yy1);
       }
   }
   /* Draw base line */
@@ -169,7 +171,7 @@ static void draw_linechart(int type, int x,int y,int w,int h,
   /* Draw the labels */
   for (i=0; i<numb; i++)
       fl_draw(entries[i].str,
-	      x+int((i+.5)*bwidth+.5), zeroh - int(entries[i].val*incr+.5),0,0,
+	      x+(int)rint((i+.5)*bwidth), zeroh - (int)rint(entries[i].val*incr),0,0,
 	      entries[i].val>=0 ? FL_ALIGN_BOTTOM : FL_ALIGN_TOP);
 }
 
@@ -187,9 +189,9 @@ static void draw_piechart(int x,int y,int w,int h,
   double txc,tyc;	/* temporary center */
   double lh = fl_height();
   /* compute center and radius */
-  xc = x+w/2.0; yc = y+h/2.0;
-  rad = h/2.0 - lh;
-  if (special) { yc += 0.1*rad; rad = 0.9*rad;}
+  double h_denom = (special ? 2.3 : 2.0);
+  rad = (h - 2*lh)/h_denom/1.1;
+  xc = x+w/2.0; yc = y+h-1.1*rad-lh;
   /* compute sum of values */
   tot = 0.0;
   for (i=0; i<numb; i++)
@@ -220,8 +222,8 @@ static void draw_piechart(int x,int y,int w,int h,
       /* draw the label */
       double xl = txc + 1.1*rad*cos(ARCINC*curang);
       fl_draw(entries[i].str,
-	      int(xl+.5),
-	      int(tyc - 1.1*rad*sin(ARCINC*curang)+.5),
+	      (int)rint(xl),
+	      (int)rint(tyc - 1.1*rad*sin(ARCINC*curang)),
 	      0, 0,
 	      xl<txc ? FL_ALIGN_RIGHT : FL_ALIGN_LEFT);
       curang += 0.5 * incr * entries[i].val;
@@ -229,31 +231,35 @@ static void draw_piechart(int x,int y,int w,int h,
 }
 
 void Fl_Chart::draw() {
-    int xx,yy,ww,hh;
-    int i;
 
-    xx = x()+9;
-    yy = y()+9;
-    ww = w()-2*9;
-    hh = h()-2*9;
+    draw_box();
+    Fl_Boxtype b = box();
+    int xx = x()+Fl::box_dx(b); // was 9 instead of dx...
+    int yy = y()+Fl::box_dy(b);
+    int ww = w()-Fl::box_dw(b);
+    int hh = h()-Fl::box_dh(b);
+    fl_push_clip(xx, yy, ww, hh);
+
+    ww--; hh--; // adjust for line thickness
 
     if (min >= max) {
 	min = max = 0.0;
-	for (i=0; i<numb; i++) {
+	for (int i=0; i<numb; i++) {
 	    if (entries[i].val < min) min = entries[i].val;
 	    if (entries[i].val > max) max = entries[i].val;
 	}
     }
 
-    draw_box();
     fl_font(textfont(),textsize());
 
     switch (type()) {
     case FL_BAR_CHART:
+	ww++; // makes the bars fill box correctly
 	draw_barchart(xx,yy,ww,hh, numb, entries, min, max,
 			autosize(), maxnumb, textcolor());
 	break;
     case FL_HORBAR_CHART:
+	hh++; // makes the bars fill box correctly
 	draw_horbarchart(xx,yy,ww,hh, numb, entries, min, max,
 			autosize(), maxnumb, textcolor());
 	break;
@@ -269,6 +275,7 @@ void Fl_Chart::draw() {
 	break;
     }
     draw_label();
+    fl_pop_clip();
 }
 
 /*------------------------------*/
@@ -278,8 +285,8 @@ void Fl_Chart::draw() {
 #define FL_CHART_LCOL		FL_LCOL
 #define FL_CHART_ALIGN		FL_ALIGN_BOTTOM
 
-Fl_Chart::Fl_Chart(int x,int y,int w,int h,const char *l) :
-Fl_Widget(x,y,w,h,l) {
+Fl_Chart::Fl_Chart(int X, int Y, int W, int H,const char *l) :
+Fl_Widget(X,Y,W,H,l) {
   box(FL_BORDER_BOX);
   align(FL_ALIGN_BOTTOM);
   numb       = 0;
@@ -289,16 +296,20 @@ Fl_Widget(x,y,w,h,l) {
   min = max  = 0;
   textfont_  = FL_HELVETICA;
   textsize_  = 10;
-  textcolor_ = FL_BLACK;
+  textcolor_ = FL_FOREGROUND_COLOR;
   entries    = (FL_CHART_ENTRY *)calloc(sizeof(FL_CHART_ENTRY), FL_CHART_MAX + 1);
 }
 
+Fl_Chart::~Fl_Chart() {
+  free(entries);
+}
+
 void Fl_Chart::clear() {
   numb = 0;
   redraw();
 }
 
-void Fl_Chart::add(double val, const char *str, uchar col) {
+void Fl_Chart::add(double val, const char *str, unsigned col) {
   /* Allocate more entries if required */
   if (numb >= sizenumb) {
     sizenumb += FL_CHART_MAX;
@@ -306,14 +317,13 @@ void Fl_Chart::add(double val, const char *str, uchar col) {
   }
   // Shift entries as needed
   if (numb >= maxnumb && maxnumb > 0) {
-    memcpy(entries, entries + 1, sizeof(FL_CHART_ENTRY) * (numb - 1));
+    memmove(entries, entries + 1, sizeof(FL_CHART_ENTRY) * (numb - 1));
     numb --;
   }
   entries[numb].val = float(val);
   entries[numb].col = col;
     if (str) {
-	strncpy(entries[numb].str,str,FL_CHART_LABEL_MAX+1);
-	entries[numb].str[FL_CHART_LABEL_MAX] = 0;
+	strlcpy(entries[numb].str,str,FL_CHART_LABEL_MAX + 1);
     } else {
 	entries[numb].str[0] = 0;
     }
@@ -321,38 +331,36 @@ void Fl_Chart::add(double val, const char *str, uchar col) {
   redraw();
 }
 
-void Fl_Chart::insert(int index, double val, const char *str, uchar col) {
+void Fl_Chart::insert(int ind, double val, const char *str, unsigned col) {
   int i;
-  if (index < 1 || index > numb+1) return;
+  if (ind < 1 || ind > numb+1) return;
   /* Allocate more entries if required */
   if (numb >= sizenumb) {
     sizenumb += FL_CHART_MAX;
     entries = (FL_CHART_ENTRY *)realloc(entries, sizeof(FL_CHART_ENTRY) * (sizenumb + 1));
   }
   // Shift entries as needed
-  for (i=numb; i >= index; i--) entries[i] = entries[i-1];
+  for (i=numb; i >= ind; i--) entries[i] = entries[i-1];
   if (numb < maxnumb || maxnumb == 0) numb++;
   /* Fill in the new entry */
-  entries[index-1].val = float(val);
-  entries[index-1].col = col;
+  entries[ind-1].val = float(val);
+  entries[ind-1].col = col;
   if (str) {
-      strncpy(entries[index-1].str,str,FL_CHART_LABEL_MAX+1);
-      entries[index-1].str[FL_CHART_LABEL_MAX] = 0;
+      strlcpy(entries[ind-1].str,str,FL_CHART_LABEL_MAX+1);
   } else {
-      entries[index-1].str[0] = 0;
+      entries[ind-1].str[0] = 0;
   }
   redraw();
 }
 
-void Fl_Chart::replace(int index,double val, const char *str, uchar col) {
-  if (index < 1 || index > numb) return;
-  entries[index-1].val = float(val);
-  entries[index-1].col = col;
+void Fl_Chart::replace(int ind,double val, const char *str, unsigned col) {
+  if (ind < 1 || ind > numb) return;
+  entries[ind-1].val = float(val);
+  entries[ind-1].col = col;
   if (str) {
-      strncpy(entries[index-1].str,str,FL_CHART_LABEL_MAX+1);
-      entries[index-1].str[FL_CHART_LABEL_MAX] = 0;
+      strlcpy(entries[ind-1].str,str,FL_CHART_LABEL_MAX+1);
   } else {
-      entries[index-1].str[0] = 0;
+      entries[ind-1].str[0] = 0;
   }
   redraw();
 }
@@ -378,5 +386,5 @@ void Fl_Chart::maxsize(int m) {
 }
 
 //
-// End of "$Id: Fl_Chart.cxx,v 1.5.2.6 2001/01/22 15:13:39 easysw Exp $".
+// End of "$Id: Fl_Chart.cxx 5942 2007-10-06 17:33:17Z matt $".
 //
