@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Dial.cxx,v 1.12.2.3.2.7 2004/04/11 04:38:57 easysw Exp $"
+// "$Id: Fl_Dial.cxx 5472 2006-09-20 03:03:14Z mike $"
 //
 // Circular dial widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2006 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
@@ -42,25 +44,30 @@ void Fl_Dial::draw(int X, int Y, int W, int H) {
     // foo: draw this nicely in certain round box types
     int foo = (box() > _FL_ROUND_UP_BOX && Fl::box_dx(box()));
     if (foo) {X--; Y--; W+=2; H+=2;}
-    fl_color(color());
-    fl_pie(X, Y, W-1, H-1, 270-a1, angle > a1 ? 360+270-angle : 270-360-angle);
-    fl_color(selection_color());
-    fl_pie(X, Y, W-1, H-1, 270-angle, 270-a1);
+    if (active_r()) fl_color(color());
+    else fl_color(fl_inactive(color()));
+    fl_pie(X, Y, W, H, 270-a1, angle > a1 ? 360+270-angle : 270-360-angle);
+    if (active_r()) fl_color(selection_color());
+    else fl_color(fl_inactive(selection_color()));
+    fl_pie(X, Y, W, H, 270-angle, 270-a1);
     if (foo) {
-      fl_color(FL_FOREGROUND_COLOR);
+      if (active_r()) fl_color(FL_FOREGROUND_COLOR);
+      else fl_color(fl_inactive(FL_FOREGROUND_COLOR));
       fl_arc(X, Y, W, H, 0, 360);
     }
     return;
   }
   if (!(damage()&FL_DAMAGE_ALL)) {
-    fl_color(color());
+    if (active_r()) fl_color(color());
+    else fl_color(fl_inactive(color()));
     fl_pie(X+1, Y+1, W-2, H-2, 0, 360);
   }
   fl_push_matrix();
   fl_translate(X+W/2-.5, Y+H/2-.5);
   fl_scale(W-1, H-1);
   fl_rotate(45-angle);
-  fl_color(selection_color());
+  if (active_r()) fl_color(selection_color());
+  else fl_color(fl_inactive(selection_color()));
   if (type()) { // FL_LINE_DIAL
     fl_begin_polygon();
     fl_vertex(0.0,   0.0);
@@ -68,7 +75,8 @@ void Fl_Dial::draw(int X, int Y, int W, int H) {
     fl_vertex(-0.25, 0.25);
     fl_vertex(0.0,   0.04);
     fl_end_polygon();
-    fl_color(FL_FOREGROUND_COLOR);
+    if (active_r()) fl_color(FL_FOREGROUND_COLOR);
+    else fl_color(fl_inactive(FL_FOREGROUND_COLOR));
     fl_begin_loop();
     fl_vertex(0.0,   0.0);
     fl_vertex(-0.04, 0.0);
@@ -77,7 +85,8 @@ void Fl_Dial::draw(int X, int Y, int W, int H) {
     fl_end_loop();
   } else {
     fl_begin_polygon(); fl_circle(-0.20, 0.20, 0.07); fl_end_polygon();
-    fl_color(FL_FOREGROUND_COLOR);
+    if (active_r()) fl_color(FL_FOREGROUND_COLOR);
+    else fl_color(fl_inactive(FL_FOREGROUND_COLOR));
     fl_begin_loop(); fl_circle(-0.20, 0.20, 0.07); fl_end_loop();
   }
   fl_pop_matrix();
@@ -134,5 +143,5 @@ Fl_Dial::Fl_Dial(int X, int Y, int W, int H, const char* l)
 }
 
 //
-// End of "$Id: Fl_Dial.cxx,v 1.12.2.3.2.7 2004/04/11 04:38:57 easysw Exp $".
+// End of "$Id: Fl_Dial.cxx 5472 2006-09-20 03:03:14Z mike $".
 //
