@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_File_Browser.cxx 4526 2005-08-18 14:08:17Z mike $"
+// "$Id: Fl_File_Browser.cxx 5635 2007-01-23 15:02:00Z mike $"
 //
 // Fl_File_Browser routines.
 //
-// Copyright 1999-2005 by Michael Sweet.
+// Copyright 1999-2006 by Michael Sweet.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -167,13 +167,16 @@ Fl_File_Browser::item_width(void *p) const	// I - List item data
   const int	*columns;		// Columns
 
 
-  // Set the font and size...
-  fl_font(textfont(), textsize());
-
   // Scan for newlines...
   line    = (FL_BLINE *)p;
   columns = column_widths();
 
+  // Set the font and size...
+  if (line->txt[strlen(line->txt) - 1] == '/')
+    fl_font(textfont() | FL_BOLD, textsize());
+  else
+    fl_font(textfont(), textsize());
+
   if (strchr(line->txt, '\n') == NULL &&
       strchr(line->txt, column_char()) == NULL)
   {
@@ -467,7 +470,7 @@ Fl_File_Browser::load(const char     *directory,// I - Directory to load
       {
         sprintf(filename, "%c:/", i);
 
-	if (i < 'C')
+	if (i < 'C') // see also: GetDriveType and GetVolumeInformation in WIN32
 	  add(filename, icon);
 	else
 	  add(filename, icon);
@@ -597,7 +600,7 @@ Fl_File_Browser::load(const char     *directory,// I - Directory to load
 
         icon = Fl_File_Icon::find(filename);
 	if ((icon && icon->type() == Fl_File_Icon::DIRECTORY) ||
-	     fl_filename_isdir(filename)) {
+	     _fl_filename_isdir_quick(filename)) {
           num_dirs ++;
           insert(num_dirs, files[i]->d_name, icon);
 	} else if (filetype_ == FILES &&
@@ -632,5 +635,5 @@ Fl_File_Browser::filter(const char *pattern)	// I - Pattern string
 
 
 //
-// End of "$Id: Fl_File_Browser.cxx 4526 2005-08-18 14:08:17Z mike $".
+// End of "$Id: Fl_File_Browser.cxx 5635 2007-01-23 15:02:00Z mike $".
 //
