@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_File_Chooser2.cxx,v 1.1.2.38 2004/09/07 20:59:16 easysw Exp $"
+// "$Id: Fl_File_Chooser2.cxx 6092 2008-04-11 12:57:37Z matt $"
 //
 // More Fl_File_Chooser routines.
 //
-// Copyright 1999-2004 by Michael Sweet.
+// Copyright 1999-2005 by Michael Sweet.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 // Contents:
 //
@@ -35,6 +37,7 @@
 //                                          in the Fl_File_Browser.
 //   Fl_File_Chooser::fileNameCB()        - Handle text entry in the FileBrowser.
 //   Fl_File_Chooser::showChoiceCB()      - Handle show selections.
+//   compare_dirnames()                   - Compare two directory names.
 //   quote_pathname()                     - Quote a pathname for a menu.
 //   unquote_pathname()                   - Unquote a pathname from a menu.
 //
@@ -59,6 +62,10 @@
 #if defined(WIN32) && ! defined (__CYGWIN__)
 #  include <direct.h>
 #  include <io.h>
+// Visual C++ 2005 incorrectly displays a warning about the use of POSIX APIs
+// on Windows, which is supposed to be POSIX compliant...
+#  define access _access
+#  define mkdir _mkdir
 // Apparently Borland C++ defines DIRECTORY in <direct.h>, which
 // interfers with the Fl_File_Icon enumeration of the same name.
 #  ifdef DIRECTORY
@@ -91,6 +98,7 @@ const char	*Fl_File_Chooser::manage_favorites_label = "Manage Favorites";
 const char	*Fl_File_Chooser::new_directory_label = "New Directory?";
 const char	*Fl_File_Chooser::new_directory_tooltip = "Create a new directory.";
 const char	*Fl_File_Chooser::preview_label = "Preview";
+const char	*Fl_File_Chooser::save_label = "Save";
 const char	*Fl_File_Chooser::show_label = "Show:";
 Fl_File_Sort_F	*Fl_File_Chooser::sort = fl_numericsort;
 
@@ -99,6 +107,7 @@ Fl_File_Sort_F	*Fl_File_Chooser::sort = fl_numericsort;
 // Local functions...
 //
 
+static int	compare_dirnames(const char *a, const char *b);
 static void	quote_pathname(char *, const char *, int);
 static void	unquote_pathname(char *, const char *, int);
 
@@ -108,38 +117,35 @@ static void	unquote_pathname(char *, const char *, int);
 //
 
 int				// O - Number of selected files
-Fl_File_Chooser::count()
-{
+Fl_File_Chooser::count() {
   int		i;		// Looping var
   int		fcount;		// Number of selected files
   const char	*filename;	// Filename in input field or list
 
 
-  if (!(type_ & MULTI))
-  {
+  filename = fileName->value();
+
+  if (!(type_ & MULTI)) {
     // Check to see if the file name input field is blank...
-    filename = fileName->value();
-
-//    printf("Fl_File_Chooser::count(): filename=\"%s\"\n", filename);
-
-    if (!filename || !filename[0])
-      return (0);
-
-    // Is the file name just the current directory?
-    return (strcmp(filename, directory_) != 0);
+    if (!filename || !filename[0]) return 0;
+    else return 1;
   }
 
   for (i = 1, fcount = 0; i <= fileList->size(); i ++)
-    if (fileList->selected(i))
-    {
+    if (fileList->selected(i)) {
       // See if this file is a directory...
-      filename = (char *)fileList->text(i);
+      // matt: why would we do that? It is perfectly legal to select multiple
+      // directories in a DIR chooser. They are visually selected and value(i)
+      // returns all of them as expected
+      //filename = (char *)fileList->text(i);
 
-      if (filename[strlen(filename) - 1] != '/')
+      //if (filename[strlen(filename) - 1] != '/')
 	fcount ++;
     }
 
-  return (fcount);
+  if (fcount) return fcount;
+  else if (!filename || !filename[0]) return 0;
+  else return 1;
 }
 
 
@@ -159,6 +165,21 @@ Fl_File_Chooser::directory(const char *d)// I - Directory to change to
   if (d == NULL)
     d = ".";
 
+#ifdef WIN32
+  // See if the filename contains backslashes...
+  char	*slash;				// Pointer to slashes
+  char	fixpath[1024];			// Path with slashes converted
+  if (strchr(d, '\\')) {
+    // Convert backslashes to slashes...
+    strlcpy(fixpath, d, sizeof(fixpath));
+
+    for (slash = strchr(fixpath, '\\'); slash; slash = strchr(slash + 1, '\\'))
+      *slash = '/';
+
+    d = fixpath;
+  }
+#endif // WIN32
+
   if (d[0] != '\0')
   {
     // Make the directory absolute...
@@ -196,8 +217,10 @@ Fl_File_Chooser::directory(const char *d)// I - Directory to change to
   else
     directory_[0] = '\0';
 
-  // Rescan the directory...
-  rescan();
+  if (shown()) {
+    // Rescan the directory...
+    rescan();
+  }
 }
 
 
@@ -223,6 +246,7 @@ Fl_File_Chooser::favoritesButtonCB()
     sprintf(menuname, "favorite%02d", v);
 
     prefs_.set(menuname, directory_);
+    prefs_.flush();
 
     quote_pathname(menuname, directory_, sizeof(menuname));
     favoritesButton->add(menuname);
@@ -359,6 +383,7 @@ Fl_File_Chooser::favoritesCB(Fl_Widget *w)
     }
 
     update_favorites();
+    prefs_.flush();
 
     favWindow->hide();
   }
@@ -392,9 +417,9 @@ Fl_File_Chooser::fileListCB()
   if (Fl::event_clicks()) {
 #if (defined(WIN32) && ! defined(__CYGWIN__)) || defined(__EMX__)
     if ((strlen(pathname) == 2 && pathname[1] == ':') ||
-        fl_filename_isdir(pathname))
+        _fl_filename_isdir_quick(pathname))
 #else
-    if (fl_filename_isdir(pathname))
+    if (_fl_filename_isdir_quick(pathname))
 #endif /* WIN32 || __EMX__ */
     {
       // Change directories...
@@ -414,8 +439,36 @@ Fl_File_Chooser::fileListCB()
   }
   else
   {
-    // Strip any trailing slash from the directory name...
+    // Check if the user clicks on a directory when picking files;
+    // if so, make sure only that item is selected...
     filename = pathname + strlen(pathname) - 1;
+
+    if ((type_ & MULTI) && !(type_ & DIRECTORY)) {
+      if (*filename == '/') {
+	// Clicked on a directory, deselect everything else...
+	int i = fileList->value();
+	fileList->deselect();
+	fileList->select(i);
+      } else {
+        // Clicked on a file - see if there are other directories selected...
+        int i;
+	const char *temp;
+	for (i = 1; i <= fileList->size(); i ++) {
+	  if (i != fileList->value() && fileList->selected(i)) {
+	    temp = fileList->text(i);
+	    temp += strlen(temp) - 1;
+	    if (*temp == '/') break;	// Yes, selected directory
+	  }
+	}
+
+        if (i <= fileList->size()) {
+	  i = fileList->value();
+	  fileList->deselect();
+	  fileList->select(i);
+	}
+      }
+    }
+    // Strip any trailing slash from the directory name...
     if (*filename == '/') *filename = '\0';
 
 //    puts("Setting fileName from fileListCB...");
@@ -429,8 +482,10 @@ Fl_File_Chooser::fileListCB()
     if (callback_) (*callback_)(this, data_);
 
     // Activate the OK button as needed...
-    if (!fl_filename_isdir(pathname) || (type_ & DIRECTORY))
+    if (!_fl_filename_isdir_quick(pathname) || (type_ & DIRECTORY))
       okButton->activate();
+    else
+      okButton->deactivate();
   }
 }
 
@@ -453,8 +508,8 @@ Fl_File_Chooser::fileNameCB()
 		first_line;	// First matching line
   const char	*file;		// File from directory
 
-
 //  puts("fileNameCB()");
+//  printf("Event: %s\n", fl_eventnames[Fl::event()]);
 
   // Get the filename from the text field...
   filename = (char *)fileName->value();
@@ -494,25 +549,24 @@ Fl_File_Chooser::fileNameCB()
     // Enter pressed - select or change directory...
 #if (defined(WIN32) && ! defined(__CYGWIN__)) || defined(__EMX__)
     if ((isalpha(pathname[0] & 255) && pathname[1] == ':' && !pathname[2]) ||
-        fl_filename_isdir(pathname)) {
+        _fl_filename_isdir_quick(pathname) &&
+	compare_dirnames(pathname, directory_)) {
 #else
-    if (fl_filename_isdir(pathname)) {
+    if (_fl_filename_isdir_quick(pathname) &&
+	compare_dirnames(pathname, directory_)) {
 #endif /* WIN32 || __EMX__ */
       directory(pathname);
     } else if ((type_ & CREATE) || access(pathname, 0) == 0) {
-      // New file or file exists...  If we are in multiple selection mode,
-      // switch to single selection mode...
-      if (type_ & MULTI)
-        type(SINGLE);
+      if (!_fl_filename_isdir_quick(pathname) || (type_ & DIRECTORY)) {
+	// Update the preview box...
+	update_preview();
 
-      // Update the preview box...
-      update_preview();
+	// Do any callback that is registered...
+	if (callback_) (*callback_)(this, data_);
 
-      // Do any callback that is registered...
-      if (callback_) (*callback_)(this, data_);
-
-      // Hide the window to signal things are done...
-      window->hide();
+	// Hide the window to signal things are done...
+	window->hide();
+      }
     } else {
       // File doesn't exist, so beep at and alert the user...
       fl_alert(existing_file_label);
@@ -563,7 +617,7 @@ Fl_File_Chooser::fileNameCB()
       file = fileList->text(i);
 
 #if (defined(WIN32) && ! defined(__CYGWIN__)) || defined(__EMX__)
-      if (strnicmp(filename, file, min_match) == 0) {
+      if (strncasecmp(filename, file, min_match) == 0) {
 #else
       if (strncmp(filename, file, min_match) == 0) {
 #endif // WIN32 || __EMX__
@@ -586,7 +640,7 @@ Fl_File_Chooser::fileNameCB()
 	  // Succeeding match; compare to find maximum string match...
 	  while (max_match > min_match)
 #if (defined(WIN32) && ! defined(__CYGWIN__)) || defined(__EMX__)
-	    if (strnicmp(file, matchname, max_match) == 0)
+	    if (strncasecmp(file, matchname, max_match) == 0)
 #else
 	    if (strncmp(file, matchname, max_match) == 0)
 #endif // WIN32 || __EMX__
@@ -742,21 +796,25 @@ Fl_File_Chooser::preview(int e)// I - 1 = enable preview, 0 = disable preview
 {
   previewButton->value(e);
   prefs_.set("preview", e);
+  prefs_.flush();
 
+  Fl_Group *p = previewBox->parent();
   if (e) {
-    int w = previewBox->h() * 2 / 3;
+    int w = p->w() * 2 / 3;
     fileList->resize(fileList->x(), fileList->y(),
-                     window->w() - 20 - w, fileList->h());
-    previewBox->resize(window->w() - 10 - w, previewBox->y(),
-                       w, previewBox->h());
-
+                     w, fileList->h());
+    previewBox->resize(fileList->x()+w, previewBox->y(),
+                       p->w()-w, previewBox->h());
+    previewBox->show();
     update_preview();
   } else {
     fileList->resize(fileList->x(), fileList->y(),
-                     window->w() - 20, fileList->h());
-    previewBox->resize(window->w() - 10, previewBox->y(),
+                     p->w(), fileList->h());
+    previewBox->resize(p->x()+p->w(), previewBox->y(),
                        0, previewBox->h());
+    previewBox->hide();
   }
+  p->init_sizes();
 
   fileList->parent()->redraw();
 }
@@ -802,6 +860,57 @@ Fl_File_Chooser::rescan()
   update_preview();
 }
 
+//
+// 'Fl_File_Chooser::rescan_keep_filename()' - Rescan the current directory
+// without clearing the filename, then select the file if it is in the list
+//
+
+void
+Fl_File_Chooser::rescan_keep_filename()
+{
+  // if no filename was set, this is likely a diretory browser
+  const char *fn = fileName->value();
+  if (!fn || !*fn || fn[strlen(fn) - 1]=='/') {
+    rescan();
+    return;
+  }
+
+  int   i;
+  char	pathname[1024];		// New pathname for filename field
+  strlcpy(pathname, fn, sizeof(pathname));
+
+  // Build the file list...
+  fileList->load(directory_, sort);
+
+  // Update the preview box...
+  update_preview();
+
+  // and select the chosen file
+  char found = 0;
+  char *slash = strrchr(pathname, '/');
+  if (slash) 
+    slash++;
+  else
+    slash = pathname;
+  for (i = 1; i <= fileList->size(); i ++)
+#if defined(WIN32) || defined(__EMX__)
+    if (strcasecmp(fileList->text(i), slash) == 0) {
+#else
+    if (strcmp(fileList->text(i), slash) == 0) {
+#endif // WIN32 || __EMX__
+      fileList->topline(i);
+      fileList->select(i);
+      found = 1;
+      break;
+    }
+
+  // update OK button activity
+  if (found || type_ & CREATE)
+    okButton->activate();
+  else
+    okButton->deactivate();
+}
+
 
 //
 // 'Fl_File_Chooser::showChoiceCB()' - Handle show selections.
@@ -834,7 +943,11 @@ Fl_File_Chooser::showChoiceCB()
   }
 
   fileList->filter(pattern_);
-  rescan();
+
+  if (shown()) {
+    // Rescan the directory...
+    rescan_keep_filename();
+  }
 }
 
 
@@ -894,7 +1007,7 @@ Fl_File_Chooser::update_preview()
 
   if (!previewButton->value()) return;
 
-  if ((filename = value()) == NULL) image = NULL;
+  if ((filename = value()) == NULL || fl_filename_isdir(filename)) image = NULL;
   else {
     window->cursor(FL_CURSOR_WAIT);
     Fl::check();
@@ -996,46 +1109,39 @@ Fl_File_Chooser::value(int f)	// I - File number
   int		i;		// Looping var
   int		fcount;		// Number of selected files
   const char	*name;		// Current filename
-  char		*slash;		// Trailing slash, if any
   static char	pathname[1024];	// Filename + directory
 
 
+  name = fileName->value();
+
   if (!(type_ & MULTI)) {
-    name = fileName->value();
+    // Return the filename in the filename field...
     if (!name || !name[0]) return NULL;
-    else if (fl_filename_isdir(name)) {
-      if (type_ & DIRECTORY) {
-        // Strip trailing slash, if any...
-        strlcpy(pathname, name, sizeof(pathname));
-	slash = pathname + strlen(pathname) - 1;
-	if (*slash == '/') *slash = '\0';
-        return pathname;
-      } else return NULL;
-    } else return name;
+    else return name;
   }
 
+  // Return a filename from the list...
   for (i = 1, fcount = 0; i <= fileList->size(); i ++)
     if (fileList->selected(i)) {
-      // See if this file is a directory...
+      // See if this file is a selected file/directory...
       name = fileList->text(i);
 
-      if (name[strlen(name) - 1] != '/') {
-        // Not a directory, see if this this is "the one"...
-	fcount ++;
+      fcount ++;
 
-	if (fcount == f) {
-	  if (directory_[0]) {
-	    snprintf(pathname, sizeof(pathname), "%s/%s", directory_, name);
-	  } else {
-	    strlcpy(pathname, name, sizeof(pathname));
-	  }
-
-	  return (pathname);
+      if (fcount == f) {
+	if (directory_[0]) {
+	  snprintf(pathname, sizeof(pathname), "%s/%s", directory_, name);
+	} else {
+	  strlcpy(pathname, name, sizeof(pathname));
 	}
+
+	return pathname;
       }
     }
 
-  return (NULL);
+  // If nothing is selected, use the filename field...
+  if (!name || !name[0]) return NULL;
+  else return name;
 }
 
 
@@ -1044,12 +1150,13 @@ Fl_File_Chooser::value(int f)	// I - File number
 //
 
 void
-Fl_File_Chooser::value(const char *filename)	// I - Filename + directory
+Fl_File_Chooser::value(const char *filename)
+					// I - Filename + directory
 {
-  int	i,					// Looping var
-  	fcount;					// Number of items in list
-  char	*slash;					// Directory separator
-  char	pathname[1024];				// Local copy of filename
+  int	i,				// Looping var
+  	fcount;				// Number of items in list
+  char	*slash;				// Directory separator
+  char	pathname[1024];			// Local copy of filename
 
 
 //  printf("Fl_File_Chooser::value(\"%s\")\n", filename == NULL ? "(null)" : filename);
@@ -1063,17 +1170,24 @@ Fl_File_Chooser::value(const char *filename)	// I - Filename + directory
     return;
   }
 
-  // Switch to single-selection mode as needed
-  if (type_ & MULTI)
-    type(SINGLE);
+#ifdef WIN32
+  // See if the filename contains backslashes...
+  char	fixpath[1024];			// Path with slashes converted
+  if (strchr(filename, '\\')) {
+    // Convert backslashes to slashes...
+    strlcpy(fixpath, filename, sizeof(fixpath));
+
+    for (slash = strchr(fixpath, '\\'); slash; slash = strchr(slash + 1, '\\'))
+      *slash = '/';
+
+    filename = fixpath;
+  }
+#endif // WIN32
 
   // See if there is a directory in there...
   fl_filename_absolute(pathname, sizeof(pathname), filename);
 
-  if ((slash = strrchr(pathname, '/')) == NULL)
-    slash = strrchr(pathname, '\\');
-
-  if (slash != NULL) {
+  if ((slash = strrchr(pathname, '/')) != NULL) {
     // Yes, change the display to the directory... 
     if (!fl_filename_isdir(pathname)) *slash++ = '\0';
 
@@ -1111,6 +1225,36 @@ Fl_File_Chooser::value(const char *filename)	// I - Filename + directory
 }
 
 
+//
+// 'compare_dirnames()' - Compare two directory names.
+//
+
+static int
+compare_dirnames(const char *a, const char *b) {
+  int alen, blen;
+
+  // Get length of each string...
+  alen = strlen(a) - 1;
+  blen = strlen(b) - 1;
+
+  if (alen < 0 || blen < 0) return alen - blen;
+
+  // Check for trailing slashes...
+  if (a[alen] != '/') alen ++;
+  if (b[blen] != '/') blen ++;
+
+  // If the lengths aren't the same, then return the difference...
+  if (alen != blen) return alen - blen;
+
+  // Do a comparison of the first N chars (alen == blen at this point)...
+#ifdef WIN32
+  return strncasecmp(a, b, alen);
+#else
+  return strncmp(a, b, alen);
+#endif // WIN32
+}
+
+
 //
 // 'quote_pathname()' - Quote a pathname for a menu.
 //
@@ -1160,5 +1304,5 @@ unquote_pathname(char       *dst,	// O - Destination string
 
 
 //
-// End of "$Id: Fl_File_Chooser2.cxx,v 1.1.2.38 2004/09/07 20:59:16 easysw Exp $".
+// End of "$Id: Fl_File_Chooser2.cxx 6092 2008-04-11 12:57:37Z matt $".
 //
