@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Gl_Window.cxx,v 1.12.2.21 2001/03/14 17:20:01 spitzak Exp $"
+// "$Id: Fl_Gl_Window.cxx 6807 2009-06-30 13:38:52Z matt $"
 //
 // OpenGL window code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2006 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,10 +20,12 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
-#include <config.h>
+#include "flstring.h"
 #if HAVE_GL
 
 #include <FL/Fl.H>
@@ -31,7 +33,6 @@
 #include "Fl_Gl_Choice.H"
 #include <FL/Fl_Gl_Window.H>
 #include <stdlib.h>
-#include <string.h>
 
 ////////////////////////////////////////////////////////////////
 
@@ -54,7 +55,7 @@
 #define COPY 3		// unchanged
 #define NODAMAGE 4	// unchanged even by X expose() events
 
-static char SWAP_TYPE; // 0 = determine it from environment variable
+static char SWAP_TYPE = 0 ; // 0 = determine it from environment variable
 
 ////////////////////////////////////////////////////////////////
 
@@ -66,34 +67,63 @@ void Fl_Gl_Window::show() {
   if (!shown()) {
     if (!g) {
       g = Fl_Gl_Choice::find(mode_,alist);
-      if (!g) {Fl::error("Insufficient GL support"); return;}
+
+      if (!g && (mode_ & FL_DOUBLE) == FL_SINGLE) {
+        g = Fl_Gl_Choice::find(mode_ | FL_DOUBLE,alist);
+	if (g) mode_ |= FL_FAKE_SINGLE;
+      }
+
+      if (!g) {
+        Fl::error("Insufficient GL support");
+	return;
+      }
     }
-#ifndef WIN32
+#if !defined(WIN32) && !defined(__APPLE__)
     Fl_X::make_xid(this, g->vis, g->colormap);
     if (overlay && overlay != this) ((Fl_Gl_Window*)overlay)->show();
 #endif
   }
   Fl_Window::show();
+
+#ifdef __APPLE__
+  set_visible();
+#endif /* __APPLE__ */
 }
 
 void Fl_Gl_Window::invalidate() {
   valid(0);
+  context_valid(0);
 #ifndef WIN32
-  if (overlay) ((Fl_Gl_Window*)overlay)->valid(0);
+  if (overlay) {
+    ((Fl_Gl_Window*)overlay)->valid(0);
+    ((Fl_Gl_Window*)overlay)->context_valid(0);
+  }
 #endif
 }
 
 int Fl_Gl_Window::mode(int m, const int *a) {
   if (m == mode_ && a == alist) return 0;
-#ifndef WIN32
+#ifndef __APPLE__
   int oldmode = mode_;
+#endif // !__APPLE__
+#if !defined(WIN32) && !defined(__APPLE__)
   Fl_Gl_Choice* oldg = g;
-#endif
+#endif // !WIN32 && !__APPLE__
   context(0);
   mode_ = m; alist = a;
   if (shown()) {
     g = Fl_Gl_Choice::find(m, a);
-#ifndef WIN32
+#if defined(WIN32)
+    if (!g || (oldmode^m)&(FL_DOUBLE|FL_STEREO)) {
+      hide();
+      show();
+    }
+#elif defined(__APPLE_QD__)
+    redraw();
+#elif defined(__APPLE_QUARTZ__)
+    // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+    redraw();
+#else
     // under X, if the visual changes we must make a new X window (yuck!):
     if (!g || g->vis->visualid!=oldg->vis->visualid || (oldmode^m)&FL_DOUBLE) {
       hide();
@@ -109,12 +139,37 @@ int Fl_Gl_Window::mode(int m, const int *a) {
 #define NON_LOCAL_CONTEXT 0x80000000
 
 void Fl_Gl_Window::make_current() {
+//  puts("Fl_Gl_Window::make_current()");
+//  printf("make_current: context_=%p\n", context_);
   if (!context_) {
     mode_ &= ~NON_LOCAL_CONTEXT;
     context_ = fl_create_gl_context(this, g);
     valid(0);
+    context_valid(0);
   }
   fl_set_gl_context(this, context_);
+
+#ifdef __APPLE__
+  // Set the buffer rectangle here, since in resize() we won't have the
+  // correct parent window size to work with...
+  GLint xywh[4];
+
+  if (window()) {
+    xywh[0] = x();
+    xywh[1] = window()->h() - y() - h();
+  } else {
+    xywh[0] = 0;
+    xywh[1] = 0;
+  }
+
+  xywh[2] = w();
+  xywh[3] = h();
+
+  aglSetInteger(context_, AGL_BUFFER_RECT, xywh);
+  aglEnable(context_, AGL_BUFFER_RECT);
+//  printf("make_current: xywh=[%d %d %d %d]\n", xywh[0], xywh[1], xywh[2], xywh[3]);
+#endif // __APPLE__
+
 #if defined(WIN32) && USE_COLORMAP
   if (fl_palette) {
     fl_GetDC(fl_xid(this));
@@ -122,6 +177,10 @@ void Fl_Gl_Window::make_current() {
     RealizePalette(fl_gc);
   }
 #endif // USE_COLORMAP
+  if (mode_ & FL_FAKE_SINGLE) {
+    glDrawBuffer(GL_FRONT);
+    glReadBuffer(GL_FRONT);
+  }
   current_ = this;
 }
 
@@ -142,12 +201,18 @@ void Fl_Gl_Window::ortho() {
 
 void Fl_Gl_Window::swap_buffers() {
 #ifdef WIN32
-#if HAVE_GL_OVERLAY
+#  if HAVE_GL_OVERLAY
   // Do not swap the overlay, to match GLX:
-  wglSwapLayerBuffers(Fl_X::i(this)->private_dc, WGL_SWAP_MAIN_PLANE);
-#else
+  BOOL ret = wglSwapLayerBuffers(Fl_X::i(this)->private_dc, WGL_SWAP_MAIN_PLANE);
+  DWORD err = GetLastError();;
+#  else
   SwapBuffers(Fl_X::i(this)->private_dc);
-#endif
+#  endif
+#elif defined(__APPLE_QD__)
+  aglSwapBuffers((AGLContext)context_);
+#elif defined(__APPLE_QUARTZ__)
+  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+  aglSwapBuffers((AGLContext)context_);
 #else
   glXSwapBuffers(fl_display, fl_xid(this));
 #endif
@@ -159,18 +224,42 @@ int fl_overlay_depth = 0;
 #endif
 
 void Fl_Gl_Window::flush() {
-  uchar save_valid = valid_;
+  uchar save_valid = valid_f_ & 1;
+#if HAVE_GL_OVERLAY && defined(WIN32)
+  uchar save_valid_f = valid_f_;
+#endif
+
+#ifdef __APPLE_QD__
+  //: clear previous clipping in this shared port
+  GrafPtr port = GetWindowPort( fl_xid(this) );
+  Rect rect; SetRect( &rect, 0, 0, 0x7fff, 0x7fff );
+  GrafPtr old; GetPort( &old );
+  SetPort( port );
+  ClipRect( &rect );
+  SetPort( old );
+#elif defined(__APPLE_QUARTZ__)
+  // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+  //: clear previous clipping in this shared port
+  GrafPtr port = GetWindowPort( fl_xid(this) );
+  Rect rect; SetRect( &rect, 0, 0, 0x7fff, 0x7fff );
+  GrafPtr old; GetPort( &old );
+  SetPort( port );
+  ClipRect( &rect );
+  SetPort( old );
+#endif
 
 #if HAVE_GL_OVERLAY && defined(WIN32)
 
-  // SGI 320 messes up overlay with user-defined cursors:
-  bool fixcursor =
-    Fl_X::i(this)->cursor && Fl_X::i(this)->cursor != fl_default_cursor;
-  if (fixcursor) SetCursor(0);
+  bool fixcursor = false; // for fixing the SGI 320 bug
 
-  // Draw into hardware overlay planes:
+  // Draw into hardware overlay planes if they are damaged:
   if (overlay && overlay != this
       && (damage()&(FL_DAMAGE_OVERLAY|FL_DAMAGE_EXPOSE) || !save_valid)) {
+    // SGI 320 messes up overlay with user-defined cursors:
+    if (Fl_X::i(this)->cursor && Fl_X::i(this)->cursor != fl_default_cursor) {
+      fixcursor = true; // make it restore cursor later
+      SetCursor(0);
+    }
     fl_set_gl_context(this, (GLContext)overlay);
     if (fl_overlay_depth)
       wglRealizeLayerPalette(Fl_X::i(this)->private_dc, 1, TRUE);
@@ -179,9 +268,10 @@ void Fl_Gl_Window::flush() {
     fl_overlay = 1;
     draw_overlay();
     fl_overlay = 0;
-    valid(save_valid);
+    valid_f_ = save_valid_f;
     wglSwapLayerBuffers(Fl_X::i(this)->private_dc, WGL_SWAP_OVERLAY1);
-    if (damage() == FL_DAMAGE_OVERLAY) { // main layer is undamaged
+    // if only the overlay was damaged we are done, leave main layer alone:
+    if (damage() == FL_DAMAGE_OVERLAY) {
       if (fixcursor) SetCursor(Fl_X::i(this)->cursor);
       return;
     }
@@ -195,11 +285,19 @@ void Fl_Gl_Window::flush() {
     glDrawBuffer(GL_BACK);
 
     if (!SWAP_TYPE) {
+#ifdef __APPLE_QD__
+      SWAP_TYPE = COPY;
+#elif defined __APPLE_QUARTZ__
+      // warning: the Quartz version should probably use Core GL (CGL) instead of AGL
+      SWAP_TYPE = COPY;
+#else
       SWAP_TYPE = UNDEFINED;
+#endif
       const char* c = getenv("GL_SWAP_TYPE");
       if (c) {
 	if (!strcmp(c,"COPY")) SWAP_TYPE = COPY;
 	else if (!strcmp(c, "NODAMAGE")) SWAP_TYPE = NODAMAGE;
+	else if (!strcmp(c, "SWAP")) SWAP_TYPE = SWAP;
       }
     }
 
@@ -227,10 +325,10 @@ void Fl_Gl_Window::flush() {
 	// and depth test needs to be off:
 	static GLContext ortho_context = 0;
 	static Fl_Gl_Window* ortho_window = 0;
-	int init = !ortho_context;
-	if (init) ortho_context = fl_create_gl_context(this, g);
+	int orthoinit = !ortho_context;
+	if (orthoinit) ortho_context = fl_create_gl_context(this, g);
 	fl_set_gl_context(this, ortho_context);
-	if (init || !save_valid || ortho_window != this) {
+	if (orthoinit || !save_valid || ortho_window != this) {
 	  glDisable(GL_DEPTH_TEST);
 	  glReadBuffer(GL_BACK);
 	  glDrawBuffer(GL_FRONT);
@@ -245,11 +343,9 @@ void Fl_Gl_Window::flush() {
 	damage1_ = 0;
 
       } else {
-
 	damage1_ = damage();
-	clear_damage(~0); draw();
+	clear_damage(0xff); draw();
 	swap_buffers();
-
       }
 
     }
@@ -273,16 +369,23 @@ void Fl_Gl_Window::flush() {
   if (fixcursor) SetCursor(Fl_X::i(this)->cursor);
 #endif
   valid(1);
+  context_valid(1);
 }
 
 void Fl_Gl_Window::resize(int X,int Y,int W,int H) {
-  if (W != w() || H != h()) {
-    valid(0);
-#ifndef WIN32
-    if (!resizable() && overlay && overlay != this)
-      ((Fl_Gl_Window*)overlay)->resize(0,0,W,H);
-#endif
+//  printf("Fl_Gl_Window::resize(X=%d, Y=%d, W=%d, H=%d)\n", X, Y, W, H);
+//  printf("current: x()=%d, y()=%d, w()=%d, h()=%d\n", x(), y(), w(), h());
+
+  if (W != w() || H != h()) valid(0);
+
+#ifdef __APPLE__
+  if (X != x() || Y != y() || W != w() || H != h()) aglUpdateContext(context_);
+#elif !defined(WIN32)
+  if ((W != w() || H != h()) && !resizable() && overlay && overlay != this) {
+    ((Fl_Gl_Window*)overlay)->resize(0,0,W,H);
   }
+#endif
+
   Fl_Window::resize(X,Y,W,H);
 }
 
@@ -312,11 +415,20 @@ Fl_Gl_Window::~Fl_Gl_Window() {
 void Fl_Gl_Window::init() {
   end(); // we probably don't want any children
   box(FL_NO_BOX);
-  mode_ = FL_RGB | FL_DEPTH | FL_DOUBLE;
-  alist = 0;
+
+  mode_    = FL_RGB | FL_DEPTH | FL_DOUBLE;
+  alist    = 0;
   context_ = 0;
-  g = 0;
-  overlay = 0;
+  g        = 0;
+  overlay  = 0;
+  valid_f_ = 0;
+  damage1_ = 0;
+
+#if 0 // This breaks resizing on Linux/X11
+  int H = h();
+  h(1); // Make sure we actually do something in resize()...
+  resize(x(), y(), w(), H);
+#endif // 0
 }
 
 void Fl_Gl_Window::draw_overlay() {}
@@ -324,5 +436,5 @@ void Fl_Gl_Window::draw_overlay() {}
 #endif
 
 //
-// End of "$Id: Fl_Gl_Window.cxx,v 1.12.2.21 2001/03/14 17:20:01 spitzak Exp $".
+// End of "$Id: Fl_Gl_Window.cxx 6807 2009-06-30 13:38:52Z matt $".
 //
