@@ -1,5 +1,5 @@
 //
-// "$Id: Fl_Image.cxx 5888 2007-06-07 17:23:41Z matt $"
+// "$Id: Fl_Image.cxx 6805 2009-06-29 07:45:48Z AlbrechtS $"
 //
 // Image drawing code for the Fast Light Tool Kit (FLTK).
 //
@@ -411,7 +411,7 @@ void Fl_RGB_Image::draw(int XP, int YP, int WP, int HP, int cx, int cy) {
     CGDataProviderRelease(src);
 #elif defined(WIN32)
     id = fl_create_offscreen(w(), h());
-    if (d() == 2 || d() == 4 && fl_can_do_alpha_blending()) {
+    if ((d() == 2 || d() == 4) && fl_can_do_alpha_blending()) {
       fl_begin_offscreen((Fl_Offscreen)id);
       fl_draw_image(array, 0, 0, w(), h(), d()|FL_IMAGE_WITH_ALPHA, ld());
       fl_end_offscreen();
@@ -521,5 +521,5 @@ void Fl_RGB_Image::label(Fl_Menu_Item* m) {
 
 
 //
-// End of "$Id: Fl_Image.cxx 5888 2007-06-07 17:23:41Z matt $".
+// End of "$Id: Fl_Image.cxx 6805 2009-06-29 07:45:48Z AlbrechtS $".
 //
