@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Light_Button.cxx 4288 2005-04-16 00:13:17Z mike $"
+// "$Id: Fl_Light_Button.cxx 5721 2007-02-27 19:23:24Z matt $"
 //
 // Lighted button widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2006 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -35,6 +35,7 @@
 #include <FL/Fl.H>
 #include <FL/Fl_Light_Button.H>
 #include <FL/fl_draw.H>
+#include "flstring.h"
 
 void Fl_Light_Button::draw() {
   if (box()) draw_box(this==Fl::pushed() ? fl_down(box()) : box(), color());
@@ -58,7 +59,11 @@ void Fl_Light_Button::draw() {
         // Check box...
         draw_box(down_box(), x()+dx, y()+dy, W, W, FL_BACKGROUND2_COLOR);
 	if (value()) {
-	  fl_color(col);
+	  if (Fl::scheme() && !strcmp(Fl::scheme(), "gtk+")) {
+	    fl_color(FL_SELECTION_COLOR);
+	  } else {
+	    fl_color(col);
+	  }
 	  int tx = x() + dx + 3;
 	  int tw = W - 6;
 	  int d1 = tw/3;
@@ -75,12 +80,19 @@ void Fl_Light_Button::draw() {
         // Radio button...
         draw_box(down_box(), x()+dx, y()+dy, W, W, FL_BACKGROUND2_COLOR);
 	if (value()) {
-	  fl_color(col);
 	  int tW = (W - Fl::box_dw(down_box())) / 2 + 1;
 	  if ((W - tW) & 1) tW++; // Make sure difference is even to center
 	  int tdx = dx + (W - tW) / 2;
 	  int tdy = dy + (W - tW) / 2;
 
+	  if (Fl::scheme() && !strcmp(Fl::scheme(), "gtk+")) {
+	    fl_color(FL_SELECTION_COLOR);
+	    tW --;
+	    fl_pie(x() + tdx - 1, y() + tdy - 1, tW + 3, tW + 3, 0.0, 360.0);
+	    fl_arc(x() + tdx - 1, y() + tdy - 1, tW + 3, tW + 3, 0.0, 360.0);
+	    fl_color(fl_color_average(FL_WHITE, FL_SELECTION_COLOR, 0.2f));
+	  } else fl_color(col);
+
 	  switch (tW) {
 	    // Larger circles draw fine...
 	    default :
@@ -106,6 +118,11 @@ void Fl_Light_Button::draw() {
 	      fl_rectf(x() + tdx, y() + tdy, tW, tW);
 	      break;
 	  }
+
+	  if (Fl::scheme() && !strcmp(Fl::scheme(), "gtk+")) {
+	    fl_color(fl_color_average(FL_WHITE, FL_SELECTION_COLOR, 0.5));
+	    fl_arc(x() + tdx, y() + tdy, tW + 1, tW + 1, 60.0, 180.0);
+	  }
 	}
         break;
       default :
@@ -118,7 +135,7 @@ void Fl_Light_Button::draw() {
     int ww = W/2+1;
     int xx = dx;
     if (w()<ww+2*xx) xx = (w()-ww)/2;
-    if (Fl::scheme()) {
+    if (Fl::scheme() && !strcmp(Fl::scheme(), "plastic")) {
       col = active_r() ? selection_color() : fl_inactive(selection_color());
       fl_color(value() ? col : fl_color_average(col, FL_BLACK, 0.5f));
       fl_pie(x()+xx, y()+dy+1, ww, hh, 0, 360);
@@ -148,5 +165,5 @@ Fl_Light_Button::Fl_Light_Button(int X, int Y, int W, int H, const char* l)
 }
 
 //
-// End of "$Id: Fl_Light_Button.cxx 4288 2005-04-16 00:13:17Z mike $".
+// End of "$Id: Fl_Light_Button.cxx 5721 2007-02-27 19:23:24Z matt $".
 //
