@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Positioner.cxx,v 1.4.2.3 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_Positioner.cxx 5347 2006-08-23 12:57:42Z matt $"
 //
 // Positioner widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // The positioner widget from Forms, gives 2D input
@@ -36,14 +38,14 @@ static double flinear(double val, double smin, double smax, double gmin, double
   else return gmin + (gmax - gmin) * (val - smin) / (smax - smin);
 }
 
-void Fl_Positioner::draw(int x, int y, int w, int h) {
-  int x1 = x + 4;
-  int y1 = y + 4;
-  int w1 = w - 2 * 4;
-  int h1 = h - 2 * 4;
+void Fl_Positioner::draw(int X, int Y, int W, int H) {
+  int x1 = X + 4;
+  int y1 = Y + 4;
+  int w1 = W - 2 * 4;
+  int h1 = H - 2 * 4;
   int xx = int(flinear(xvalue(), xmin, xmax, x1, x1+w1-1)+.5);
   int yy = int(flinear(yvalue(), ymin, ymax, y1, y1+h1-1)+.5);
-  draw_box(box(), x, y, w, h, color());
+  draw_box(box(), X, Y, W, H, color());
   fl_color(selection_color());
   fl_xyline(x1, yy, x1+w1);
   fl_yxline(xx, y1, y1+h1);
@@ -70,28 +72,44 @@ int Fl_Positioner::yvalue(double Y) {
   return(value(xvalue_, Y));
 }
 
-int Fl_Positioner::handle(int event, int x, int y, int w, int h) {
+int Fl_Positioner::handle(int event, int X, int Y, int W, int H) {
   switch (event) {
   case FL_PUSH:
   case FL_DRAG:
   case FL_RELEASE: {
-    double x1 = x + 4;
-    double y1 = y + 4;
-    double w1 = w - 2 * 4;
-    double h1 = h - 2 * 4;
-    double X = flinear(Fl::event_x(), x1, x1+w1-1.0, xmin, xmax);
-    if (xstep_) X = int(X/xstep_+0.5) * xstep_;
-    if (X < xmin) X = xmin;
-    if (X > xmax) X = xmax;
-    double Y = flinear(Fl::event_y(), y1, y1+h1-1.0, ymin, ymax);
-    if (ystep_) Y = int(Y/ystep_+0.5) * ystep_;
-    if (Y < ymin) Y = ymin;
-    if (Y > ymax) Y = ymax;
-    if (value(X, Y)) set_changed();}
+    double x1 = X + 4;
+    double y1 = Y + 4;
+    double w1 = W - 2 * 4;
+    double h1 = H - 2 * 4;
+    double xx = flinear(Fl::event_x(), x1, x1+w1-1.0, xmin, xmax);
+    if (xstep_) xx = int(xx/xstep_+0.5) * xstep_;
+    if (xmin < xmax) {
+      if (xx < xmin) xx = xmin;
+      if (xx > xmax) xx = xmax;
+    } else {
+      if (xx > xmin) xx = xmin;
+      if (xx < xmax) xx = xmax;
+    }
+    double yy = flinear(Fl::event_y(), y1, y1+h1-1.0, ymin, ymax);
+    if (ystep_) yy = int(yy/ystep_+0.5) * ystep_;
+    if (ymin < ymax) {
+      if (yy < ymin) yy = ymin;
+      if (yy > ymax) yy = ymax;
+    } else {
+      if (yy > ymin) yy = ymin;
+      if (yy < ymax) yy = ymax;
+    }
+    if (xx != xvalue_ || yy != yvalue_) {
+      xvalue_ = xx; yvalue_ = yy;
+      set_changed();
+      redraw();
+                   } }
     if (!(when() & FL_WHEN_CHANGED ||
-	  when() & FL_WHEN_RELEASE && event == FL_RELEASE)) return 1;
+	  (when() & FL_WHEN_RELEASE && event == FL_RELEASE))) return 1;
     if (changed() || when()&FL_WHEN_NOT_CHANGED) {
-      clear_changed(); do_callback();}
+      if (event == FL_RELEASE) clear_changed();
+      do_callback();
+    }
     return 1;
   default:
     return 0;
@@ -102,8 +120,8 @@ int Fl_Positioner::handle(int e) {
   return handle(e, x(), y(), w(), h());
 }
 
-Fl_Positioner::Fl_Positioner(int x, int y, int w, int h, const char* l)
-: Fl_Widget(x, y, w, h, l) {
+Fl_Positioner::Fl_Positioner(int X, int Y, int W, int H, const char* l)
+: Fl_Widget(X, Y, W, H, l) {
   box(FL_DOWN_BOX);
   selection_color(FL_RED);
   align(FL_ALIGN_BOTTOM);
@@ -129,5 +147,5 @@ void Fl_Positioner::ybounds(double a, double b) {
 }
 
 //
-// End of "$Id: Fl_Positioner.cxx,v 1.4.2.3 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_Positioner.cxx 5347 2006-08-23 12:57:42Z matt $".
 //
