@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Scrollbar.cxx,v 1.7.2.14.2.17 2004/07/27 16:02:21 easysw Exp $"
+// "$Id: Fl_Scrollbar.cxx 6042 2008-02-25 13:00:53Z matt $"
 //
 // Scroll bar widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2006 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,13 +20,16 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
 #include <FL/Fl_Scrollbar.H>
 #include <FL/fl_draw.H>
 #include <math.h>
+#include "flstring.h"
 
 #define INITIALREPEAT .5
 #define REPEAT .05
@@ -86,6 +89,7 @@ int Fl_Scrollbar::handle(int event) {
   else {
     int S = int(slider_size()*ww+.5);
     int T = (horizontal() ? H : W)/2+1;
+    if (type()==FL_VERT_NICE_SLIDER || type()==FL_HOR_NICE_SLIDER) T += 4;
     if (S < T) S = T;
     double val =
       (maximum()-minimum()) ? (value()-minimum())/(maximum()-minimum()) : 0.5;
@@ -126,9 +130,15 @@ int Fl_Scrollbar::handle(int event) {
     if (pushed_) return 1;
     return Fl_Slider::handle(event, X,Y,W,H);
   case FL_MOUSEWHEEL :
-    if (horizontal()) return 0;
-    handle_drag(clamp(value() + linesize_ * Fl::e_dy));
-    return 1;
+    if (horizontal()) {
+      if (Fl::e_dx==0) return 0;
+      handle_drag(clamp(value() + linesize_ * Fl::e_dx));
+      return 1;
+    } else {
+      if (Fl::e_dy==0) return 0;
+      handle_drag(clamp(value() + linesize_ * Fl::e_dy));
+      return 1;
+    }
   case FL_SHORTCUT:
   case FL_KEYBOARD: {
     int v = value();
@@ -205,9 +215,15 @@ void Fl_Scrollbar::draw() {
       int w1 = (H-4)/3; if (w1 < 1) w1 = 1;
       int x1 = X+(H-w1-1)/2;
       int yy1 = Y+(H-2*w1-1)/2;
-      fl_polygon(x1, yy1+w1, x1+w1, yy1+2*w1, x1+w1, yy1);
-      x1 += (W-H);
-      fl_polygon(x1, yy1, x1, yy1+2*w1, x1+w1, yy1+w1);
+      if (Fl::scheme_ && !strcmp(Fl::scheme_, "gtk+")) {
+	fl_polygon(x1, yy1+w1, x1+w1, yy1+2*w1, x1+w1-1, yy1+w1, x1+w1, yy1);
+	x1 += (W-H);
+	fl_polygon(x1, yy1, x1+1, yy1+w1, x1, yy1+2*w1, x1+w1, yy1+w1);
+      } else {
+	fl_polygon(x1, yy1+w1, x1+w1, yy1+2*w1, x1+w1, yy1);
+	x1 += (W-H);
+	fl_polygon(x1, yy1, x1, yy1+2*w1, x1+w1, yy1+w1);
+      }
     }
   } else { // vertical
     if (H < 3*W) {Fl_Slider::draw(X,Y,W,H); return;}
@@ -224,9 +240,16 @@ void Fl_Scrollbar::draw() {
       int w1 = (W-4)/3; if (w1 < 1) w1 = 1;
       int x1 = X+(W-2*w1-1)/2;
       int yy1 = Y+(W-w1-1)/2;
-      fl_polygon(x1, yy1+w1, x1+2*w1, yy1+w1, x1+w1, yy1);
-      yy1 += H-W;
-      fl_polygon(x1, yy1, x1+w1, yy1+w1, x1+2*w1, yy1);
+      if (Fl::scheme_ && !strcmp(Fl::scheme_, "gtk+")) {
+	fl_polygon(x1, yy1+w1, x1+w1, yy1+w1-1, x1+2*w1, yy1+w1, x1+w1, yy1);
+	yy1 += H-W;
+	fl_polygon(x1, yy1, x1+w1, yy1+1, x1+w1, yy1+w1);
+	fl_polygon(x1+w1, yy1+1, x1+2*w1, yy1, x1+w1, yy1+w1);
+      } else {
+	fl_polygon(x1, yy1+w1, x1+2*w1, yy1+w1, x1+w1, yy1);
+	yy1 += H-W;
+	fl_polygon(x1, yy1, x1+w1, yy1+w1, x1+2*w1, yy1);
+      }
     }
   }
 }
@@ -242,6 +265,13 @@ Fl_Scrollbar::Fl_Scrollbar(int X, int Y, int W, int H, const char* L)
   step(1);
 }
 
+Fl_Scrollbar::~Fl_Scrollbar()
+{
+  if (pushed_)
+    Fl::remove_timeout(timeout_cb, this);
+}
+
+
 //
-// End of "$Id: Fl_Scrollbar.cxx,v 1.7.2.14.2.17 2004/07/27 16:02:21 easysw Exp $".
+// End of "$Id: Fl_Scrollbar.cxx 6042 2008-02-25 13:00:53Z matt $".
 //
