@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Shared_Image.cxx,v 1.23.2.21 2004/09/24 16:00:11 easysw Exp $"
+// "$Id: Fl_Shared_Image.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // Shared image code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <stdio.h>
@@ -41,9 +43,9 @@ Fl_Shared_Image **Fl_Shared_Image::images_ = 0;	// Shared images
 int	Fl_Shared_Image::num_images_ = 0;	// Number of shared images
 int	Fl_Shared_Image::alloc_images_ = 0;	// Allocated shared images
 
-Fl_Shared_Handler *Fl_Shared_Image::handlers_;	// Additional format handlers
-int	Fl_Shared_Image::num_handlers_;		// Number of format handlers
-int	Fl_Shared_Image::alloc_handlers_;	// Allocated format handlers
+Fl_Shared_Handler *Fl_Shared_Image::handlers_ = 0;// Additional format handlers
+int	Fl_Shared_Image::num_handlers_ = 0;	// Number of format handlers
+int	Fl_Shared_Image::alloc_handlers_ = 0;	// Allocated format handlers
 
 
 //
@@ -225,7 +227,7 @@ Fl_Shared_Image::reload() {
     fread(header, 1, sizeof(header), fp);
     fclose(fp);
   } else {
-    memset(header, 0, sizeof(header));
+    return;
   }
 
   // Load the image as appropriate...
@@ -245,14 +247,15 @@ Fl_Shared_Image::reload() {
   if (img) {
     if (alloc_image_) delete image_;
 
-    image_       = img;
     alloc_image_ = 1;
 
     if ((img->w() != w() && w()) || (img->h() != h() && h())) {
       // Make sure the reloaded image is the same size as the existing one.
       Fl_Image *temp = img->copy(w(), h());
       delete img;
-      img = temp;
+      image_ = temp;
+    } else {
+      image_ = img;
     }
 
     update();
@@ -460,5 +463,5 @@ Fl_Shared_Image::remove_handler(Fl_Shared_Handler f) {
 
 
 //
-// End of "$Id: Fl_Shared_Image.cxx,v 1.23.2.21 2004/09/24 16:00:11 easysw Exp $".
+// End of "$Id: Fl_Shared_Image.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
