@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Sys_Menu_Bar.cxx,v 1.1.2.10 2004/11/21 14:32:22 easysw Exp $"
+// "$Id: Fl_Sys_Menu_Bar.cxx 6016 2008-01-09 21:32:40Z matt $"
 //
 // MacOS system menu bar widget for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 /**
@@ -46,6 +48,8 @@
  * ::menu(bar) should only be called once!
  * Many other calls of the parent class don't work.
  * Changing the menu items has no effect on the menu bar.
+ * Starting with OS X 10.5, FLTK applications must be created as
+ * a bundle for the System Menu Bar (and maybe other features) to work!
  */
 
 #if defined(__APPLE__)
@@ -131,17 +135,19 @@ static void setMenuShortcut( MenuHandle mh, int miCnt, const Fl_Menu_Item *m )
   if ( !isalnum( key ) )
     return;
   
-  long macMod = kMenuNoModifiers;
+  long macMod = kMenuNoCommandModifier;
+  if ( m->shortcut_ & FL_META ) macMod = kMenuNoModifiers;
   if ( m->shortcut_ & FL_SHIFT || isupper(key) ) macMod |= kMenuShiftModifier;
   if ( m->shortcut_ & FL_ALT ) macMod |= kMenuOptionModifier;
-  if ( m->shortcut_ & FL_META ) macMod |= kMenuControlModifier;
-  if ( !(m->shortcut_ & FL_CTRL) ) macMod |= kMenuNoCommandModifier;
+  if ( m->shortcut_ & FL_CTRL ) macMod |= kMenuControlModifier;
   
   //SetMenuItemKeyGlyph( mh, miCnt, key );
   SetItemCmd( mh, miCnt, toupper(key) );
   SetMenuItemModifiers( mh, miCnt, macMod );
 }
 
+#if 0
+// this function needs to be verified before we compile it back in.
 static void catMenuShortcut( const Fl_Menu_Item *m, char *dst )
 {
   if ( !m->shortcut_ ) 
@@ -158,6 +164,7 @@ static void catMenuShortcut( const Fl_Menu_Item *m, char *dst )
   //if ( isalnum( mm->shortcut_ ) && !( mm->flags & FL_SUBMENU ) )
   //sprintf( buf+strlen(buf), "/%c", mm->shortcut_ );
 }
+#endif
 
 static void setMenuFlags( MenuHandle mh, int miCnt, const Fl_Menu_Item *m )
 {
@@ -173,8 +180,6 @@ static void catMenuFlags( const Fl_Menu_Item *m, char *dst )
 {
   if ( !m->flags ) 
     return;
-  while ( *dst ) 
-    dst++;
   if ( m->flags & FL_MENU_INACTIVE )
     strcat( dst, "(" );
 }
@@ -188,7 +193,7 @@ static void createSubMenu( MenuHandle mh, int &cnt, pFl_Menu_Item &mm )
   int miCnt = 1;
   while ( mm->text )
   {
-    MenuHandle smh;
+    MenuHandle smh = 0;
     buf[1] = 0;
     catMenuFont( mm, buf+1 );
     //catMenuShortcut( mm, buf+1 );
@@ -240,27 +245,32 @@ void Fl_Sys_Menu_Bar::menu(const Fl_Menu_Item *m)
   fl_sys_menu_bar = this;
 
   char buf[255];
-  int cnt = 1; // first menu is no 2. no 1 is the Apple Menu
 
+  int cnt = 1; // first menu is no 2. no 1 is the Apple Menu
   const Fl_Menu_Item *mm = m;
   for (;;)
   {
-    if ( !mm->text )
+    if ( !mm || !mm->text )
       break;
+    char visible = mm->visible() ? 1 : 0;
     buf[1] = 0;
     catMenuText( mm->text, buf+1 );
     buf[0] = strlen( buf+1 );
     MenuHandle mh = NewMenu( ++cnt, (unsigned char*)buf );
+    if ( mm->flags & FL_MENU_INACTIVE ) {
+      ChangeMenuAttributes(mh, kMenuAttrAutoDisable, 0);
+      DisableAllMenuItems(mh);
+      DisableMenuItem(mh, 0);
+    }
     if ( mm->flags & FL_SUBMENU )
       createSubMenu( mh, cnt, ++mm );
-    else if ( mm->flags & FL_SUBMENU_POINTER )
-    {
+    else if ( mm->flags & FL_SUBMENU_POINTER ) {
       const Fl_Menu_Item *smm = (Fl_Menu_Item*)mm->user_data_;
       createSubMenu( mh, cnt, smm );
     }
-    
-    InsertMenu( mh, 0 );
-    if ( mm->flags & FL_MENU_INACTIVE ) DisableMenuItem( mh, 0 );
+    if ( visible ) {
+      InsertMenu( mh, 0 );
+    }
     mm++;
   }
   DrawMenuBar();
@@ -320,5 +330,5 @@ int Fl_Menu_Bar::handle(int event) {
 #endif /* __APPLE__ */
 
 //
-// End of "$Id: Fl_Sys_Menu_Bar.cxx,v 1.1.2.10 2004/11/21 14:32:22 easysw Exp $".
+// End of "$Id: Fl_Sys_Menu_Bar.cxx 6016 2008-01-09 21:32:40Z matt $".
 //
