@@ -1,7 +1,7 @@
 //
-// "$Id: Fl_Text_Buffer.cxx,v 1.9.2.22 2004/09/08 15:42:47 easysw Exp $"
+// "$Id: Fl_Text_Buffer.cxx 6011 2008-01-04 20:32:37Z matt $"
 //
-// Copyright 2001-2004 by Bill Spitzak and others.
+// Copyright 2001-2005 by Bill Spitzak and others.
 // Original code Copyright Mark Edel.  Permission to distribute under
 // the LGPL for the FLTK library granted by Mark Edel.
 //
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <stdio.h>
@@ -31,7 +33,7 @@
 #include <FL/Fl_Text_Buffer.H>
 
 
-#define PREFERRED_GAP_SIZE 80
+#define PREFERRED_GAP_SIZE 1024
 /* Initial size for the buffer gap (empty space
 in the buffer where text might be inserted
 if the user is typing sequential chars ) */
@@ -667,6 +669,11 @@ void Fl_Text_Buffer::secondary_select_rectangular( int start, int end,
   redisplay_selection( &oldSelection, &mSecondary );
 }
 
+int Fl_Text_Buffer::secondary_selection_position( int *start, int *end
+                                      ) {
+  return mSecondary.position( start, end );
+}
+
 int Fl_Text_Buffer::secondary_selection_position( int *start, int *end,
     int *isRect, int *rectStart, int *rectEnd ) {
   return mSecondary.position( start, end, isRect, rectStart,
@@ -707,6 +714,11 @@ void Fl_Text_Buffer::highlight_rectangular( int start, int end,
   redisplay_selection( &oldSelection, &mHighlight );
 }
 
+int Fl_Text_Buffer::highlight_position( int *start, int *end
+                                      ) {
+  return mHighlight.position( start, end );
+}
+
 int Fl_Text_Buffer::highlight_position( int *start, int *end,
                                         int *isRect, int *rectStart, int *rectEnd ) {
   return mHighlight.position( start, end, isRect, rectStart,
@@ -1538,6 +1550,7 @@ void Fl_Text_Buffer::remove_rectangular_( int start, int end, int rectStart,
      the text between rectStart and rectEnd and padding appropriately */
   lineStart = start;
   outPtr = outStr;
+  endOffset = 0;
   while ( lineStart <= mLength && lineStart <= end ) {
     lineEnd = line_end( lineStart );
     line = text_range( lineStart, lineEnd );
@@ -2517,5 +2530,5 @@ Fl_Text_Buffer::outputfile(const char *file, int start, int end, int buflen) {
 
 
 //
-// End of "$Id: Fl_Text_Buffer.cxx,v 1.9.2.22 2004/09/08 15:42:47 easysw Exp $".
+// End of "$Id: Fl_Text_Buffer.cxx 6011 2008-01-04 20:32:37Z matt $".
 //
