@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_Tooltip.cxx,v 1.38.2.31 2004/09/24 16:00:11 easysw Exp $"
+// "$Id: Fl_Tooltip.cxx 5848 2007-05-20 16:18:31Z mike $"
 //
 // Tooltip source file for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl_Tooltip.H>
@@ -58,7 +60,7 @@ public:
 
 Fl_Widget* Fl_Tooltip::widget_ = 0;
 static Fl_TooltipBox *window = 0;
-static int X,Y,W,H;
+static int Y,H;
 
 void Fl_TooltipBox::layout() {
   fl_font(Fl_Tooltip::font(), Fl_Tooltip::size());
@@ -69,21 +71,21 @@ void Fl_TooltipBox::layout() {
 
   // find position on the screen of the widget:
   int ox = Fl::event_x_root();
-  //int ox = X+W/2;
   int oy = Y + H+2;
   for (Fl_Widget* p = Fl_Tooltip::current(); p; p = p->window()) {
-    //ox += p->x();
     oy += p->y();
   }
-  if (ox+ww > Fl::w()) ox = Fl::w() - ww;
-  if (ox < 0) ox = 0;
+  int scr_x, scr_y, scr_w, scr_h;
+  Fl::screen_xywh(scr_x, scr_y, scr_w, scr_h);
+  if (ox+ww > scr_x+scr_w) ox = scr_x+scr_w - ww;
+  if (ox < scr_x) ox = scr_x;
   if (H > 30) {
     oy = Fl::event_y_root()+13;
-    if (oy+hh > Fl::h()) oy -= 23+hh;
+    if (oy+hh > scr_y+scr_h) oy -= 23+hh;
   } else {
-    if (oy+hh > Fl::h()) oy -= (4+hh+H);
+    if (oy+hh > scr_y+scr_h) oy -= (4+hh+H);
   }
-  if (oy < 0) oy = 0;
+  if (oy < scr_y) oy = scr_y;
 
   resize(ox, oy, ww, hh);
 }
@@ -190,7 +192,7 @@ Fl_Tooltip::exit_(Fl_Widget *w) {
   widget_ = 0;
   Fl::remove_timeout(tooltip_timeout);
   Fl::remove_timeout(recent_timeout);
-  if (window) window->hide();
+  if (window && window->visible()) window->hide();
   if (recent_tooltip) {
     if (Fl::event_state() & FL_BUTTONS) recent_tooltip = 0;
     else Fl::add_timeout(Fl_Tooltip::hoverdelay(), recent_timeout);
@@ -204,6 +206,9 @@ Fl_Tooltip::exit_(Fl_Widget *w) {
 void
 Fl_Tooltip::enter_area(Fl_Widget* wid, int x,int y,int w,int h, const char* t)
 {
+  (void)x;
+  (void)w;
+
 #ifdef DEBUG
   printf("Fl_Tooltip::enter_area(wid=%p, x=%d, y=%d, w=%d, h=%d, t=\"%s\")\n",
          wid, x, y, w, h, t ? t : "(null)");
@@ -220,7 +225,7 @@ Fl_Tooltip::enter_area(Fl_Widget* wid, int x,int y,int w,int h, const char* t)
   Fl::remove_timeout(tooltip_timeout);
   Fl::remove_timeout(recent_timeout);
   // remember it:
-  widget_ = wid; X = x; Y = y; W = w; H = h; tip = t;
+  widget_ = wid; Y = y; H = h; tip = t;
   // popup the tooltip immediately if it was recently up:
   if (recent_tooltip) {
     if (window) window->hide();
@@ -229,11 +234,11 @@ Fl_Tooltip::enter_area(Fl_Widget* wid, int x,int y,int w,int h, const char* t)
 #ifdef WIN32
     // possible fix for the Windows titlebar, it seems to want the
     // window to be destroyed, moving it messes up the parenting:
-    if (window) window->hide();
+    if (window && window->visible()) window->hide();
 #endif // WIN32
     tooltip_timeout(0);
   } else {
-    if (window) window->hide();
+    if (window && window->visible()) window->hide();
     Fl::add_timeout(Fl_Tooltip::delay(), tooltip_timeout);
   }
 
@@ -254,5 +259,5 @@ void Fl_Widget::tooltip(const char *tt) {
 }
 
 //
-// End of "$Id: Fl_Tooltip.cxx,v 1.38.2.31 2004/09/24 16:00:11 easysw Exp $".
+// End of "$Id: Fl_Tooltip.cxx 5848 2007-05-20 16:18:31Z mike $".
 //
