@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_display.cxx,v 1.4.2.3 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_display.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // Display function for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // Startup method to set what display to use.
@@ -28,16 +30,25 @@
 
 #include <FL/Fl.H>
 #include <stdlib.h>
-#include <string.h>
+#include "flstring.h"
 
 void Fl::display(const char *d) {
-  char *e = new char[strlen(d)+13];
+#if defined(__APPLE__) || defined(WIN32)
+  (void)d;
+#else
+  static char e[1024];
   strcpy(e,"DISPLAY=");
-  strcpy(e+8,d);
-  for (char *c = e+8; *c!=':'; c++) if (!*c) {strcpy(c,":0.0"); break;}
+  strlcat(e,d,sizeof(e));
+  for (char *c = e+8; *c!=':'; c++) {
+    if (!*c) {
+      strlcat(e,":0.0",sizeof(e));
+      break;
+    }
+  }
   putenv(e);
+#endif // __APPLE__
 }
 
 //
-// End of "$Id: Fl_display.cxx,v 1.4.2.3 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_display.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
