@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_get_key_win32.cxx,v 1.4.2.5 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: Fl_get_key_win32.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // WIN32 keyboard state routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // Return the current state of a key.  Keys are named by fltk symbols,
@@ -28,10 +30,10 @@
 // VK_x symbols.
 
 #include <FL/Fl.H>
-#include <FL/win32.H>
+#include <FL/x.H>
 
 // convert an Fltk (X) keysym to a MSWindows VK symbol:
-// See also the inverse converter in Fl_win32.C
+// See also the inverse converter in Fl_win32.cxx
 // This table is in numeric order by Fltk symbol order for binary search:
 
 static const struct {unsigned short vk, fltk;} vktab[] = {
@@ -132,5 +134,5 @@ int Fl::get_key(int k) {
 }
 
 //
-// End of "$Id: Fl_get_key_win32.cxx,v 1.4.2.5 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: Fl_get_key_win32.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
