@@ -1,9 +1,9 @@
 //
-// "$Id: filename_isdir.cxx,v 1.4.2.5.2.9 2004/04/11 04:38:59 easysw Exp $"
+// "$Id: filename_isdir.cxx 5948 2007-10-07 10:12:32Z matt $"
 //
 // Directory detection routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // Used by fl_file_chooser
@@ -38,15 +40,22 @@ static inline int isdirsep(char c) {return c=='/' || c=='\\';}
 #define isdirsep(c) ((c)=='/')
 #endif
 
+int _fl_filename_isdir_quick(const char* n) {
+  // Do a quick optimization for filenames with a trailing slash...
+  if (*n && isdirsep(n[strlen(n) - 1])) return 1;
+  return fl_filename_isdir(n);
+}
+
 int fl_filename_isdir(const char* n) {
   struct stat	s;
-
-#ifdef WIN32
   char		fn[1024];
   int		length;
+
+  length = strlen(n);
+
+#ifdef WIN32
   // This workaround brought to you by the fine folks at Microsoft!
   // (read lots of sarcasm in that...)
-  length = strlen(n);
   if (length < (int)(sizeof(fn) - 1)) {
     if (length < 4 && isalpha(n[0]) && n[1] == ':' &&
         (isdirsep(n[2]) || !n[2])) {
@@ -62,11 +71,21 @@ int fl_filename_isdir(const char* n) {
       n = fn;
     }
   }
+#else
+  // Matt: Just in case, we strip the slash for other operating
+  // systems as well, avoid bugs by sloppy implementations
+  // of "stat".
+  if (length > 1 && isdirsep(n[length - 1])) {
+    length --;
+    memcpy(fn, n, length);
+    fn[length] = '\0';
+    n = fn;
+  }
 #endif
 
   return !stat(n, &s) && (s.st_mode&0170000)==0040000;
 }
 
 //
-// End of "$Id: filename_isdir.cxx,v 1.4.2.5.2.9 2004/04/11 04:38:59 easysw Exp $".
+// End of "$Id: filename_isdir.cxx 5948 2007-10-07 10:12:32Z matt $".
 //
