@@ -1,9 +1,9 @@
 //
-// "$Id: fl_arc.cxx,v 1.4.2.3.2.8 2004/04/11 04:38:59 easysw Exp $"
+// "$Id: fl_arc.cxx 5349 2006-08-23 14:43:07Z matt $"
 //
 // Arc functions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // Utility for drawing arcs and circles.  They are added to
@@ -30,6 +32,13 @@
 #include <FL/fl_draw.H>
 #include <FL/math.h>
 
+// avoid problems with some platforms that don't 
+// implement hypot.
+static double _fl_hypot(double x, double y) {
+  return sqrt(x*x + y*y);
+}
+
+
 void fl_arc(double x, double y, double r, double start, double end) {
 
   // draw start point accurately:
@@ -42,10 +51,10 @@ void fl_arc(double x, double y, double r, double start, double end) {
   // Maximum arc length to approximate with chord with error <= 0.125
   
   double epsilon; {
-    double r1 = hypot(fl_transform_dx(r,0), // Horizontal "radius"
-		      fl_transform_dy(r,0));
-    double r2 = hypot(fl_transform_dx(0,r), // Vertical "radius"
-		      fl_transform_dy(0,r));
+    double r1 = _fl_hypot(fl_transform_dx(r,0), // Horizontal "radius"
+		          fl_transform_dy(r,0));
+    double r2 = _fl_hypot(fl_transform_dx(0,r), // Vertical "radius"
+		          fl_transform_dy(0,r));
 		      
     if (r1 > r2) r1 = r2;		// r1 = minimum "radius"
     if (r1 < 2.) r1 = 2.;		// radius for circa 9 chords/circle
@@ -74,5 +83,5 @@ void fl_circle(double x,double y,double r) {
 #endif
 
 //
-// End of "$Id: fl_arc.cxx,v 1.4.2.3.2.8 2004/04/11 04:38:59 easysw Exp $".
+// End of "$Id: fl_arc.cxx 5349 2006-08-23 14:43:07Z matt $".
 //
