@@ -1,9 +1,9 @@
 //
-// "$Id: fl_ask.cxx,v 1.8.2.8.2.14 2004/10/18 20:22:25 easysw Exp $"
+// "$Id: fl_ask.cxx 6870 2009-09-13 22:02:30Z AlbrechtS $"
 //
 // Standard dialog functions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // Implementation of fl_message, fl_ask, fl_choice, fl_input
@@ -54,11 +56,18 @@ static const char *iconlabel = "?";
 Fl_Font fl_message_font_ = FL_HELVETICA;
 uchar fl_message_size_ = 14;
 
+static char avoidRecursion = 0;
+
 static Fl_Window *makeform() {
  if (message_form) {
    message_form->size(410,103);
    return message_form;
  }
+ // make sure that the dialog does not become the child of some 
+ // current group
+ Fl_Group *previously_current_group = Fl_Group::current();
+ Fl_Group::current(0);
+ // create a new top level window
  Fl_Window *w = message_form = new Fl_Window(410,103,"");
  // w->clear_border();
  // w->box(FL_UP_BOX);
@@ -73,7 +82,7 @@ static Fl_Window *makeform() {
   o->labelcolor(FL_BLUE);
  }
  button[0] = new Fl_Button(310, 70, 90, 23);
- button[0]->shortcut("^[");
+ button[0]->shortcut(FL_Escape);
  button[0]->align(FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
  button[1] = new Fl_Return_Button(210, 70, 90, 23);
  button[1]->align(FL_ALIGN_INSIDE|FL_ALIGN_WRAP);
@@ -82,6 +91,7 @@ static Fl_Window *makeform() {
  w->resizable(new Fl_Box(60,10,110-60,27));
  w->end();
  w->set_modal();
+ Fl_Group::current(previously_current_group);
  return w;
 }
 
@@ -93,9 +103,10 @@ static Fl_Window *makeform() {
 void resizeform() {
   int	i;
   int	message_w, message_h;
-  int	icon_size;
+  int	text_height;
   int	button_w[3], button_h[3];
   int	x, w, h, max_w, max_h;
+	const int icon_size = 50;
 
   fl_font(fl_message_font_, fl_message_size_);
   message_w = message_h = 0;
@@ -128,8 +139,8 @@ void resizeform() {
         max_h = button_h[i];
     }
 
-  if (input->visible()) icon_size = message_h + 25;
-  else icon_size = message_h;
+  if (input->visible()) text_height = message_h + 25;
+  else text_height = message_h;
 
   max_w = message_w + 10 + icon_size;
   w     = button_w[0] + button_w[1] + button_w[2] - 10;
@@ -140,7 +151,7 @@ void resizeform() {
   message_w = max_w - 10 - icon_size;
 
   w = max_w + 20;
-  h = max_h + 30 + icon_size;
+  h = max_h + 30 + text_height;
 
   message_form->size(w, h);
   message_form->size_range(w, h, w, h);
@@ -167,6 +178,10 @@ static int innards(const char* fmt, va_list ap,
   const char *b1,
   const char *b2)
 {
+  Fl::pushed(0); // stop dragging (STR #2159)
+
+  avoidRecursion = 1;
+
   makeform();
   char buffer[1024];
   if (!strcmp(fmt,"%s")) {
@@ -190,9 +205,19 @@ static int innards(const char* fmt, va_list ap,
 
   resizeform();
 
+  if (button[1]->visible() && !input->visible()) 
+    button[1]->take_focus();
   message_form->hotspot(button[0]);
+  if (b0 && Fl_Widget::label_shortcut(b0))
+    button[0]->shortcut(0);
+  else
+    button[0]->shortcut(FL_Escape);
+
   message_form->show();
-  int r;
+  // deactivate Fl::grab(), because it is incompatible with Fl::readqueue()
+  Fl_Window* g = Fl::grab();
+  if (g) Fl::grab(0);
+  int r = 0;
   for (;;) {
     Fl_Widget *o = Fl::readqueue();
     if (!o) Fl::wait();
@@ -201,8 +226,12 @@ static int innards(const char* fmt, va_list ap,
     else if (o == button[2]) {r = 2; break;}
     else if (o == message_form) {r = 0; break;}
   }
+  if (g) // regrab the previous popup menu, if there was one
+    Fl::grab(g);
   message_form->hide();
   icon->label(prev_icon_label);
+
+  avoidRecursion = 0;
   return r;
 }
 
@@ -261,30 +290,39 @@ void fl_beep(int type) {
 }
 
 void fl_message(const char *fmt, ...) {
+
+  if (avoidRecursion) return;
+
   va_list ap;
 
   fl_beep(FL_BEEP_MESSAGE);
 
   va_start(ap, fmt);
   iconlabel = "i";
-  innards(fmt, ap, 0, fl_ok, 0);
+  innards(fmt, ap, 0, fl_close, 0);
   va_end(ap);
   iconlabel = "?";
 }
 
 void fl_alert(const char *fmt, ...) {
+
+  if (avoidRecursion) return;
+
   va_list ap;
 
   fl_beep(FL_BEEP_ERROR);
 
   va_start(ap, fmt);
   iconlabel = "!";
-  innards(fmt, ap, 0, fl_ok, 0);
+  innards(fmt, ap, 0, fl_close, 0);
   va_end(ap);
   iconlabel = "?";
 }
 
 int fl_ask(const char *fmt, ...) {
+
+  if (avoidRecursion) return 0;
+
   va_list ap;
 
   fl_beep(FL_BEEP_QUESTION);
@@ -297,6 +335,9 @@ int fl_ask(const char *fmt, ...) {
 }
 
 int fl_choice(const char*fmt,const char *b0,const char *b1,const char *b2,...){
+
+  if (avoidRecursion) return 0;
+
   va_list ap;
 
   fl_beep(FL_BEEP_QUESTION);
@@ -325,6 +366,9 @@ static const char* input_innards(const char* fmt, va_list ap,
 }
 
 const char* fl_input(const char *fmt, const char *defstr, ...) {
+
+  if (avoidRecursion) return 0;
+
   fl_beep(FL_BEEP_QUESTION);
 
   va_list ap;
@@ -335,6 +379,9 @@ const char* fl_input(const char *fmt, const char *defstr, ...) {
 }
 
 const char *fl_password(const char *fmt, const char *defstr, ...) {
+
+  if (avoidRecursion) return 0;
+
   fl_beep(FL_BEEP_PASSWORD);
 
   va_list ap;
@@ -345,5 +392,5 @@ const char *fl_password(const char *fmt, const char *defstr, ...) {
 }
 
 //
-// End of "$Id: fl_ask.cxx,v 1.8.2.8.2.14 2004/10/18 20:22:25 easysw Exp $".
+// End of "$Id: fl_ask.cxx 6870 2009-09-13 22:02:30Z AlbrechtS $".
 //
