@@ -1,9 +1,9 @@
 //
-// "$Id: fl_color_win32.cxx,v 1.14.2.3 2001/01/22 15:13:40 easysw Exp $"
+// "$Id: fl_color_win32.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // WIN32 color functions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // The fltk "colormap".  This allows ui colors to be stored in 8-bit
@@ -34,7 +36,7 @@
 
 #include <config.h>
 #include <FL/Fl.H>
-#include <FL/win32.H>
+#include <FL/x.H>
 #include <FL/fl_draw.H>
 
 static unsigned fl_cmap[256] = {
@@ -47,17 +49,32 @@ Fl_XMap fl_xmap[256];
 Fl_XMap* fl_current_xmap;
 
 HPALETTE fl_palette;
-HPEN tmppen=0;
-HBRUSH tmpbrush=0;
+static HGDIOBJ tmppen=0;
+static HPEN savepen=0;
+
+void fl_cleanup_pens(void) {
+  for (int i=0; i<256; i++) {
+    if (fl_xmap[i].pen) DeleteObject(fl_xmap[i].pen);
+  }
+}
+
+void fl_save_pen(void) {
+    if(!tmppen) tmppen = CreatePen(PS_SOLID, 1, 0);
+    savepen = (HPEN)SelectObject(fl_gc, tmppen);
+}
+
+void fl_restore_pen(void) {
+    if (savepen) SelectObject(fl_gc, savepen);
+    DeleteObject(tmppen);
+    tmppen = 0;
+    savepen = 0;
+}
 
 static void clear_xmap(Fl_XMap& xmap) {
   if (xmap.pen) {
-    if(!tmppen) tmppen = CreatePen(PS_SOLID, 1, 0);
-    if(!tmpbrush) tmpbrush = CreateSolidBrush(0);
-    HPEN oldpen = (HPEN)SelectObject(fl_gc, tmppen); // Push out the current pen of the gc
+    HGDIOBJ tmppen = GetStockObject(BLACK_PEN);
+    HGDIOBJ oldpen = SelectObject(fl_gc, tmppen);       // Push out the current pen of the gc
     if(oldpen != xmap.pen) SelectObject(fl_gc, oldpen); // Put it back if it is not the one we are about to delete
-    SelectObject(fl_gc, tmpbrush); // Push out the old pen of the gc
-    //fl_current_xmap = 0;
     DeleteObject((HGDIOBJ)(xmap.pen));
     xmap.pen = 0;
     xmap.brush = -1;
@@ -66,34 +83,45 @@ static void clear_xmap(Fl_XMap& xmap) {
 
 static void set_xmap(Fl_XMap& xmap, COLORREF c) {
   xmap.rgb = c;
-  xmap.pen = CreatePen(PS_SOLID, 1, xmap.rgb);
+  if (xmap.pen) {
+      HGDIOBJ oldpen = SelectObject(fl_gc,GetStockObject(BLACK_PEN)); // replace current pen with safe one
+      if (oldpen != xmap.pen)SelectObject(fl_gc,oldpen);              // if old one not xmap.pen, need to put it back
+      DeleteObject(xmap.pen);                                         // delete pen
+  }
+  xmap.pen = CreatePen(PS_SOLID, 1, xmap.rgb);                        // get a pen into xmap.pen
   xmap.brush = -1;
 }
 
 Fl_Color fl_color_;
 
 void fl_color(Fl_Color i) {
-  fl_color_ = i;
-  Fl_XMap &xmap = fl_xmap[i];
-  if (!xmap.pen) {
+  if (i & 0xffffff00) {
+    unsigned rgb = (unsigned)i;
+    fl_color((uchar)(rgb >> 24), (uchar)(rgb >> 16), (uchar)(rgb >> 8));
+  } else {
+    fl_color_ = i;
+    Fl_XMap &xmap = fl_xmap[i];
+    if (!xmap.pen) {
 #if USE_COLORMAP
-    if (fl_palette) {
-      set_xmap(xmap, PALETTEINDEX(i));
-    } else {
+      if (fl_palette) {
+	set_xmap(xmap, PALETTEINDEX(i));
+      } else {
 #endif
-      unsigned c = fl_cmap[i];
-      set_xmap(xmap, RGB(uchar(c>>24), uchar(c>>16), uchar(c>>8)));
+	unsigned c = fl_cmap[i];
+	set_xmap(xmap, RGB(uchar(c>>24), uchar(c>>16), uchar(c>>8)));
 #if USE_COLORMAP
+      }
+#endif
     }
-#endif
+    fl_current_xmap = &xmap;
+    SelectObject(fl_gc, (HGDIOBJ)(xmap.pen));
   }
-  fl_current_xmap = &xmap;
-  SelectObject(fl_gc, (HGDIOBJ)(xmap.pen));
 }
 
 void fl_color(uchar r, uchar g, uchar b) {
   static Fl_XMap xmap;
   COLORREF c = RGB(r,g,b);
+  fl_color_ = fl_rgb_color(r, g, b);
   if (!xmap.pen || c != xmap.rgb) {
     clear_xmap(xmap);
     set_xmap(xmap, c);
@@ -103,6 +131,10 @@ void fl_color(uchar r, uchar g, uchar b) {
 }
 
 HBRUSH fl_brush() {
+  return fl_brush_action(0);
+}
+
+HBRUSH fl_brush_action(int action) {
   Fl_XMap *xmap = fl_current_xmap;
   // Wonko: we use some statistics to cache only a limited number
   // of brushes:
@@ -113,6 +145,15 @@ HBRUSH fl_brush() {
     Fl_XMap* backref;
   } brushes[FL_N_BRUSH];
 
+  if (action) {
+    SelectObject(fl_gc, GetStockObject(BLACK_BRUSH));  // Load stock object
+    for (int i=0; i<FL_N_BRUSH; i++) {
+      if (brushes[i].brush)
+        DeleteObject(brushes[i].brush); // delete all brushes in array
+    }
+    return NULL;
+  }
+
   int i = xmap->brush; // find the associated brush
   if (i != -1) { // if the brush was allready allocated
     if (brushes[i].brush == NULL) goto CREATE_BRUSH;
@@ -135,7 +176,10 @@ HBRUSH fl_brush() {
       }
     }
     i = imin;
-    DeleteObject(brushes[i].brush);
+    HGDIOBJ tmpbrush = GetStockObject(BLACK_BRUSH);  // get a stock brush
+    HGDIOBJ oldbrush = SelectObject(fl_gc,tmpbrush); // load in into current context
+    if (oldbrush != brushes[i].brush) SelectObject(fl_gc,oldbrush);  // reload old one
+    DeleteObject(brushes[i].brush);      // delete the one in list
     brushes[i].brush = NULL;
     brushes[i].backref->brush = -1;
   }
@@ -206,5 +250,5 @@ fl_select_palette(void)
 #endif
 
 //
-// End of "$Id: fl_color_win32.cxx,v 1.14.2.3 2001/01/22 15:13:40 easysw Exp $".
+// End of "$Id: fl_color_win32.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
