@@ -1,9 +1,9 @@
 //
-// "$Id: fl_dnd_mac.cxx,v 1.1.2.7 2004/09/09 21:34:47 matthiaswm Exp $"
+// "$Id: fl_dnd_mac.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // Drag & Drop code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -71,7 +71,9 @@ int Fl::dnd()
   Fl_Widget *w = Fl::pushed();
   if ( w )
   {
-    w->handle( FL_RELEASE );
+    int old_event = Fl::e_number;
+    w->handle(Fl::e_number = FL_RELEASE);
+    Fl::e_number = old_event;
     Fl::pushed( 0 );
   }
 
@@ -84,5 +86,5 @@ int Fl::dnd()
   
 
 //
-// End of "$Id: fl_dnd_mac.cxx,v 1.1.2.7 2004/09/09 21:34:47 matthiaswm Exp $".
+// End of "$Id: fl_dnd_mac.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
