@@ -1,9 +1,9 @@
 //
-// "$Id: fl_labeltype.cxx,v 1.6.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: fl_labeltype.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // Label drawing routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // Drawing code for the (one) common label types.
@@ -31,6 +33,7 @@
 #include <FL/Fl_Widget.H>
 #include <FL/Fl_Group.H>
 #include <FL/fl_draw.H>
+#include <FL/Fl_Image.H>
 
 void
 fl_no_label(const Fl_Label*,int,int,int,int,Fl_Align) {}
@@ -40,13 +43,17 @@ fl_normal_label(const Fl_Label* o, int X, int Y, int W, int H, Fl_Align align)
 {
   fl_font(o->font, o->size);
   fl_color((Fl_Color)o->color);
-  fl_draw(o->value, X, Y, W, H, align);
+  fl_draw(o->value, X, Y, W, H, align, o->image);
 }
 
 void
 fl_normal_measure(const Fl_Label* o, int& W, int& H) {
   fl_font(o->font, o->size);
   fl_measure(o->value, W, H);
+  if (o->image) {
+    if (o->image->w() > W) W = o->image->w();
+    H += o->image->h();
+  }
 }
 
 #define MAX_LABELTYPE 16
@@ -54,16 +61,15 @@ fl_normal_measure(const Fl_Label* o, int& W, int& H) {
 static Fl_Label_Draw_F* table[MAX_LABELTYPE] = {
   fl_normal_label,
   fl_no_label,
-  fl_normal_label,	// _FL_SYMBOL_LABEL,
   fl_normal_label,	// _FL_SHADOW_LABEL,
   fl_normal_label,	// _FL_ENGRAVED_LABEL,
   fl_normal_label,	// _FL_EMBOSSED_LABEL,
-  fl_no_label,		// _FL_BITMAP_LABEL,
-  fl_no_label,		// _FL_PIXMAP_LABEL,
-  fl_no_label,		// _FL_IMAGE_LABEL,
+  fl_no_label,		// _FL_MULTI_LABEL,
+  fl_no_label,		// _FL_ICON_LABEL,
   // FL_FREE_LABELTYPE+n:
   fl_no_label, fl_no_label, fl_no_label,
-  fl_no_label, fl_no_label, fl_no_label, fl_no_label,
+  fl_no_label, fl_no_label, fl_no_label,
+  fl_no_label, fl_no_label, fl_no_label
 };
 
 static Fl_Label_Measure_F* measure[MAX_LABELTYPE];
@@ -77,12 +83,16 @@ void Fl::set_labeltype(Fl_Labeltype t,Fl_Label_Draw_F* f,Fl_Label_Measure_F*m)
 
 // draw label with arbitrary alignment in arbitrary box:
 void Fl_Label::draw(int X, int Y, int W, int H, Fl_Align align) const {
-  if (!value) return;
+  if (!value && !image) return;
   table[type](this, X, Y, W, H, align);
 }
 
 void Fl_Label::measure(int& W, int& H) const {
-  if (!value) return;
+  if (!value && !image) {
+    W = H = 0;
+    return;
+  }
+
   Fl_Label_Measure_F* f = ::measure[type]; if (!f) f = fl_normal_measure;
   f(this, W, H);
 }
@@ -103,11 +113,13 @@ void Fl_Widget::draw_label(int X, int Y, int W, int H) const {
 }
 
 // Anybody can call this to force the label to draw anywhere:
-extern char fl_draw_shortcut;
 void Fl_Widget::draw_label(int X, int Y, int W, int H, Fl_Align a) const {
   if (flags()&SHORTCUT_LABEL) fl_draw_shortcut = 1;
   Fl_Label l1 = label_;
-  if (!active_r()) l1.color = inactive((Fl_Color)l1.color);
+  if (!active_r()) {
+    l1.color = fl_inactive((Fl_Color)l1.color);
+    if (l1.deimage) l1.image = l1.deimage;
+  }
   l1.draw(X,Y,W,H,a);
   fl_draw_shortcut = 0;
 }
@@ -117,5 +129,5 @@ void Fl_Widget::draw_label(int X, int Y, int W, int H, Fl_Align a) const {
 #include <FL/Fl_Input_.H>
 
 //
-// End of "$Id: fl_labeltype.cxx,v 1.6.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: fl_labeltype.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
