@@ -1,9 +1,9 @@
 //
-// "$Id: fl_line_style.cxx,v 1.3.2.3.2.17 2004/08/31 22:00:48 matthiaswm Exp $"
+// "$Id: fl_line_style.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // Line style code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
@@ -67,6 +69,7 @@ void fl_line_style(int style, int width, char* dashes) {
   }
   HPEN oldpen = (HPEN)SelectObject(fl_gc, newpen);
   DeleteObject(oldpen);
+  DeleteObject(fl_current_xmap->pen);
   fl_current_xmap->pen = newpen;
 #elif defined(__APPLE_QD__)
   // QuickDraw supports pen size and pattern, but no arbitrary line styles.
@@ -158,5 +161,5 @@ void fl_line_style(int style, int width, char* dashes) {
 
 
 //
-// End of "$Id: fl_line_style.cxx,v 1.3.2.3.2.17 2004/08/31 22:00:48 matthiaswm Exp $".
+// End of "$Id: fl_line_style.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
