@@ -1,9 +1,9 @@
 //
-// "$Id: fl_oval_box.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: fl_oval_box.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // Oval box drawing code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 
@@ -41,7 +43,7 @@ static void fl_oval_frame(int x, int y, int w, int h, Fl_Color c) {
 }
 
 static void fl_oval_box(int x, int y, int w, int h, Fl_Color c) {
-  fl_oval_flat_box(x,y,w-1,h-1,c);
+  fl_oval_flat_box(x,y,w,h,c);
   fl_oval_frame(x,y,w,h,FL_BLACK);
 }
 
@@ -51,7 +53,7 @@ static void fl_oval_shadow_box(int x, int y, int w, int h, Fl_Color c) {
 }
 
 extern void fl_internal_boxtype(Fl_Boxtype, Fl_Box_Draw_F*);
-Fl_Boxtype define_FL_OVAL_BOX() {
+Fl_Boxtype fl_define_FL_OVAL_BOX() {
   fl_internal_boxtype(_FL_OSHADOW_BOX,fl_oval_shadow_box);
   fl_internal_boxtype(_FL_OVAL_FRAME,fl_oval_frame);
   fl_internal_boxtype(_FL_OFLAT_BOX,fl_oval_flat_box);
@@ -60,5 +62,5 @@ Fl_Boxtype define_FL_OVAL_BOX() {
 }
 
 //
-// End of "$Id: fl_oval_box.cxx,v 1.4.2.3 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: fl_oval_box.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
