@@ -1,9 +1,9 @@
 //
-// "$Id: fl_rect.cxx,v 1.10.2.4.2.18 2004/09/09 21:34:47 matthiaswm Exp $"
+// "$Id: fl_rect.cxx 5692 2007-02-12 16:41:41Z matt $"
 //
 // Rectangle drawing routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // These routines from fl_draw.H are used by the standard boxtypes
@@ -29,6 +31,7 @@
 // that minimal update works.
 
 #include <config.h>
+#include <FL/Fl.H>
 #include <FL/Fl_Widget.H>
 #include <FL/fl_draw.H>
 #include <FL/x.H>
@@ -237,8 +240,7 @@ void fl_line(int x, int y, int x1, int y1) {
   MoveTo(x, y); 
   LineTo(x1, y1);
 #elif defined(__APPLE_QUARTZ__)
-  if (( x==x1 || y==y1 ) && fl_quartz_line_width_!=1.0f )  
-    CGContextSetShouldAntialias(fl_gc, false);
+  if (fl_quartz_line_width_==1.0f ) CGContextSetShouldAntialias(fl_gc, false);
   CGContextMoveToPoint(fl_gc, x, y);
   CGContextAddLineToPoint(fl_gc, x1, y1);
   CGContextStrokePath(fl_gc);
@@ -261,10 +263,12 @@ void fl_line(int x, int y, int x1, int y1, int x2, int y2) {
   LineTo(x1, y1);
   LineTo(x2, y2);
 #elif defined(__APPLE_QUARTZ__)
+  if (fl_quartz_line_width_==1.0f ) CGContextSetShouldAntialias(fl_gc, false);
   CGContextMoveToPoint(fl_gc, x, y);
   CGContextAddLineToPoint(fl_gc, x1, y1);
   CGContextAddLineToPoint(fl_gc, x2, y2);
   CGContextStrokePath(fl_gc);
+  if (fl_quartz_line_width_==1.0f ) CGContextSetShouldAntialias(fl_gc, true);
 #else
   XPoint p[3];
   p[0].x = x;  p[0].y = y;
@@ -345,8 +349,10 @@ void fl_polygon(int x, int y, int x1, int y1, int x2, int y2) {
   MoveTo(x, y);
   LineTo(x1, y1);
   LineTo(x2, y2);
+  LineTo(x, y);
   ClosePoly();
   PaintPoly(poly);
+  FramePoly(poly);
   KillPoly(poly);
 #elif defined(__APPLE_QUARTZ__)
   CGContextMoveToPoint(fl_gc, x, y);
@@ -376,8 +382,10 @@ void fl_polygon(int x, int y, int x1, int y1, int x2, int y2, int x3, int y3) {
   LineTo(x1, y1);
   LineTo(x2, y2);
   LineTo(x3, y3);
+  LineTo(x, y);
   ClosePoly();
   PaintPoly(poly);
+  FramePoly(poly);
   KillPoly(poly);
 #elif defined(__APPLE_QUARTZ__)
   CGContextMoveToPoint(fl_gc, x, y);
@@ -445,9 +453,11 @@ void fl_restore_clip() {
 #ifdef WIN32
   SelectClipRgn(fl_gc, r); //if r is NULL, clip is automatically cleared
 #elif defined(__APPLE_QD__)
-#  if 1
-  if ( fl_window ) 
-  {
+# if 1
+  // This code is required to allow true subwindows to work on Mac.
+  // During regular operation however, this seems overkill.
+  // See also: Fl_Window::make_current()
+  if ( fl_window ) {
     GrafPtr port = GetWindowPort( fl_window );
     if ( port ) { // port will be NULL if we are using a GWorld (and fl_window_region is invalid)
       RgnHandle portClip = NewRgn();
@@ -457,16 +467,23 @@ void fl_restore_clip() {
       SetPortClipRegion( port, portClip );
       DisposeRgn( portClip );
     }
+  } else {
+    if (r) 
+      SetClip(r);
+    else {
+      Rect rect; rect.left=0; rect.top=0; rect.right=0x7fff; rect.bottom=0x7fff;
+      ClipRect(&rect);
+    }
   }
-#  else
+# else
   if (r) SetClip(r);
   else {
     Rect rect; rect.left=0; rect.top=0; rect.right=0x7fff; rect.bottom=0x7fff;
     ClipRect(&rect);
   }
-#  endif
+# endif
 #elif defined(__APPLE_QUARTZ__)
-  if ( fl_window )
+  if ( fl_window ) // clipping for a true window
   {
     GrafPtr port = GetWindowPort( fl_window );
     if ( port ) { 
@@ -480,6 +497,16 @@ void fl_restore_clip() {
       Fl_X::q_fill_context();
       DisposeRgn( portClip );
     }
+  } else if (fl_gc) { // clipping for an offscreen drawing world (CGBitmap)
+    Rect portRect;
+    portRect.top = 0;
+    portRect.left = 0;
+    portRect.bottom = CGBitmapContextGetHeight(fl_gc);
+    portRect.right = CGBitmapContextGetWidth(fl_gc);
+    Fl_X::q_clear_clipping();
+    if (r)
+      ClipCGContextToRegion(fl_gc, &portRect, r);
+    Fl_X::q_fill_context();
   }
 #else
   if (r) XSetRegion(fl_display, fl_gc, r);
@@ -534,12 +561,14 @@ void fl_push_clip(int x, int y, int w, int h) {
 #endif
   }
   if (rstackptr < STACK_MAX) rstack[++rstackptr] = r;
+  else Fl::warning("fl_push_clip: clip stack overflow!\n");
   fl_restore_clip();
 }
 
 // make there be no clip (used by fl_begin_offscreen() only!)
 void fl_push_no_clip() {
   if (rstackptr < STACK_MAX) rstack[++rstackptr] = 0;
+  else Fl::warning("fl_push_no_clip: clip stack overflow!\n");
   fl_restore_clip();
 }
 
@@ -548,14 +577,13 @@ void fl_pop_clip() {
   if (rstackptr > 0) {
     Fl_Region oldr = rstack[rstackptr--];
     if (oldr) XDestroyRegion(oldr);
-  }
+  } else Fl::warning("fl_pop_clip: clip stack underflow!\n");
   fl_restore_clip();
 }
 
 // does this rectangle intersect current clip?
 int fl_not_clipped(int x, int y, int w, int h) {
-  if (x+w <= 0 || y+h <= 0 || x > Fl_Window::current()->w()
-      || y > Fl_Window::current()->h()) return 0;
+  if (x+w <= 0 || y+h <= 0) return 0;
   Fl_Region r = rstack[rstackptr];
 #ifdef WIN32
   if (!r) return 1;
@@ -653,5 +681,5 @@ int fl_clip_box(int x, int y, int w, int h, int& X, int& Y, int& W, int& H){
 }
 
 //
-// End of "$Id: fl_rect.cxx,v 1.10.2.4.2.18 2004/09/09 21:34:47 matthiaswm Exp $".
+// End of "$Id: fl_rect.cxx 5692 2007-02-12 16:41:41Z matt $".
 //
