@@ -1,9 +1,9 @@
 //
-// "$Id: fl_round_box.cxx,v 1.6.2.3.2.5 2004/04/11 04:39:00 easysw Exp $"
+// "$Id: fl_round_box.cxx 6537 2008-12-02 17:21:36Z matt $"
 //
 // Round box drawing routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // Box drawing code for an obscure box type.
@@ -39,7 +41,7 @@ void fl_arc_i(int x,int y,int w,int h,double a1,double a2) {
 
 enum {UPPER_LEFT, LOWER_RIGHT, CLOSED, FILL};
 
-static void draw(int which, int x,int y,int w,int h, int inset, uchar color)
+static void draw(int which, int x,int y,int w,int h, int inset, unsigned int color)
 {
   if (inset*2 >= w) inset = (w-1)/2;
   if (inset*2 >= h) inset = (h-1)/2;
@@ -49,7 +51,7 @@ static void draw(int which, int x,int y,int w,int h, int inset, uchar color)
   h -= 2*inset;
   int d = w <= h ? w : h;
   if (d <= 1) return;
-  fl_color((Fl_Color)color);
+  fl_color(color);
   void (*f)(int,int,int,int,double,double);
   f = (which==FILL) ? fl_pie : fl_arc_i;
   if (which >= CLOSED) {
@@ -69,11 +71,11 @@ static void draw(int which, int x,int y,int w,int h, int inset, uchar color)
       fl_rectf(x+d/2, y, w-(d&-2), h);
   } else {
     if (w < h) {
-      if (which != UPPER_LEFT) fl_yxline(x+w-1, y+d/2, y+h-d/2);
-      if (which != LOWER_RIGHT) fl_yxline(x, y+d/2, y+h-d/2);
+      if (which != UPPER_LEFT) fl_yxline(x+w-1, y+d/2-1, y+h-d/2+1);
+      if (which != LOWER_RIGHT) fl_yxline(x, y+d/2-1, y+h-d/2+1);
     } else if (w > h) {
-      if (which != UPPER_LEFT) fl_xyline(x+d/2, y+h-1, x+w-d/2);
-      if (which != LOWER_RIGHT) fl_xyline(x+d/2, y, x+w-d/2);
+      if (which != UPPER_LEFT) fl_xyline(x+d/2-1, y+h-1, x+w-d/2+1);
+      if (which != LOWER_RIGHT) fl_xyline(x+d/2-1, y, x+w-d/2+1);
     }
   }
 }
@@ -116,5 +118,5 @@ Fl_Boxtype fl_define_FL_ROUND_UP_BOX() {
 }
 
 //
-// End of "$Id: fl_round_box.cxx,v 1.6.2.3.2.5 2004/04/11 04:39:00 easysw Exp $".
+// End of "$Id: fl_round_box.cxx 6537 2008-12-02 17:21:36Z matt $".
 //
