@@ -1,5 +1,5 @@
 //
-// "$Id: fl_shortcut.cxx 5981 2007-11-19 16:21:19Z matt $"
+// "$Id: fl_shortcut.cxx 6803 2009-06-28 22:25:51Z matt $"
 //
 // Shortcut support routines for the Fast Light Tool Kit (FLTK).
 //
@@ -222,6 +222,10 @@ char Fl_Widget::label_shortcut(const char *t) {
 }
 
 int Fl_Widget::test_shortcut(const char *t) {
+  #ifdef WIN32
+  // on MSWindows, users expect shortcuts to work only when the Alt modifier is pressed
+  if (Fl::event_state(FL_ALT)==0) return 0;
+  #endif
   char c = Fl::event_text()[0];
   if (!c || !t) return 0;
   if (c == label_shortcut(t))
@@ -235,5 +239,5 @@ int Fl_Widget::test_shortcut() {
 }
 
 //
-// End of "$Id: fl_shortcut.cxx 5981 2007-11-19 16:21:19Z matt $".
+// End of "$Id: fl_shortcut.cxx 6803 2009-06-28 22:25:51Z matt $".
 //
