@@ -1,9 +1,9 @@
 //
-// "$Id: fl_vertex.cxx,v 1.5.2.3.2.12 2004/08/31 22:00:49 matthiaswm Exp $"
+// "$Id: fl_vertex.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // Portable drawing routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // Portable drawing code for drawing arbitrary shapes with
@@ -32,6 +34,7 @@
 #include <config.h>
 #include <FL/fl_draw.H>
 #include <FL/x.H>
+#include <FL/Fl.H>
 #include <FL/math.h>
 #include <stdlib.h>
 
@@ -39,12 +42,22 @@ struct matrix {double a, b, c, d, x, y;};
 
 static matrix m = {1, 0, 0, 1, 0, 0};
 
-static matrix stack[10];
+static matrix stack[32];
 static int sptr = 0;
 
-void fl_push_matrix() {stack[sptr++] = m;}
+void fl_push_matrix() {
+  if (sptr==32)
+    Fl::error("fl_push_matrix(): matrix stack overflow.");
+  else
+    stack[sptr++] = m;
+}
 
-void fl_pop_matrix() {m = stack[--sptr];}
+void fl_pop_matrix() {
+  if (sptr==0)
+    Fl::error("fl_pop_matrix(): matrix stack underflow.");
+  else 
+    m = stack[--sptr];
+}
 
 void fl_mult_matrix(double a, double b, double c, double d, double x, double y) {
   matrix o;
@@ -308,5 +321,5 @@ void fl_circle(double x, double y,double r) {
 }
 
 //
-// End of "$Id: fl_vertex.cxx,v 1.5.2.3.2.12 2004/08/31 22:00:49 matthiaswm Exp $".
+// End of "$Id: fl_vertex.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
