@@ -1,9 +1,9 @@
 //
-// "$Id: gl_draw.cxx,v 1.7.2.5 2001/03/14 17:20:02 spitzak Exp $"
+// "$Id: gl_draw.cxx 5190 2006-06-09 16:16:34Z mike $"
 //
 // OpenGL drawing support routines for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,13 +20,15 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 // Functions from <FL/gl.h>
-// See also Fl_Gl_Window and gl_start.C
+// See also Fl_Gl_Window and gl_start.cxx
 
-#include <config.h>
+#include "flstring.h"
 #if HAVE_GL
 
 #include <FL/Fl.H>
@@ -35,7 +37,10 @@
 #include <FL/fl_draw.H>
 #include "Fl_Gl_Choice.H"
 #include "Fl_Font.H"
-#include <string.h>
+
+#if USE_XFT
+extern XFontStruct* fl_xxfont();
+#endif // USE_XFT
 
 int   gl_height() {return fl_height();}
 int   gl_descent() {return fl_descent();}
@@ -48,21 +53,80 @@ void  gl_font(int fontid, int size) {
   if (!fl_fontsize->listbase) {
 #ifdef WIN32
     int base = fl_fontsize->metr.tmFirstChar;
-    int size = fl_fontsize->metr.tmLastChar-base+1;
+    int count = fl_fontsize->metr.tmLastChar-base+1;
     HFONT oldFid = (HFONT)SelectObject(fl_gc, fl_fontsize->fid);
     fl_fontsize->listbase = glGenLists(256);
-    wglUseFontBitmaps(fl_gc, base, size, fl_fontsize->listbase+base); 
+    wglUseFontBitmaps(fl_gc, base, count, fl_fontsize->listbase+base); 
     SelectObject(fl_gc, oldFid);
-#else
-    int base = fl_xfont->min_char_or_byte2;
-    int size = fl_xfont->max_char_or_byte2-base+1;
+#elif defined(__APPLE_QD__)
+    // undefined characters automatically receive an empty GL list in aglUseFont
     fl_fontsize->listbase = glGenLists(256);
-    glXUseXFont(fl_xfont->fid, base, size, fl_fontsize->listbase+base);
+    aglUseFont(aglGetCurrentContext(), fl_fontsize->font, fl_fontsize->face,
+               fl_fontsize->size, 0, 256, fl_fontsize->listbase);
+#elif defined(__APPLE_QUARTZ__)
+    short font, face, size;
+    uchar fn[256]; 
+    fn[0]=strlen(fl_fontsize->q_name); 
+    strcpy((char*)(fn+1), fl_fontsize->q_name);
+    GetFNum(fn, &font);
+    face = 0;
+    size = fl_fontsize->size;
+    fl_fontsize->listbase = glGenLists(256);
+    aglUseFont(aglGetCurrentContext(), font, face,
+               size, 0, 256, fl_fontsize->listbase);
+#else
+#  if USE_XFT
+    fl_xfont = fl_xxfont();
+#  endif // USE_XFT
+    int base = fl_xfont->min_char_or_byte2;
+    int count = fl_xfont->max_char_or_byte2-base+1;
+    fl_fontsize->listbase = glGenLists(256);
+    glXUseXFont(fl_xfont->fid, base, count, fl_fontsize->listbase+base);
 #endif
   }
   glListBase(fl_fontsize->listbase);
 }
 
+
+void gl_remove_displaylist_fonts()
+{
+# if HAVE_GL
+
+  // clear variables used mostly in fl_font
+  fl_font_ = 0;
+  fl_size_ = 0;
+
+  for (int j = 0 ; j < FL_FREE_FONT ; ++j)
+  {
+    Fl_FontSize* past = 0;
+    Fl_Fontdesc* s    = fl_fonts + j ;
+    Fl_FontSize* f    = s->first;
+    while (f != 0) {
+      if(f->listbase) {
+        if(f == s->first) {
+          s->first = f->next;
+        }
+        else {
+          past->next = f->next;
+        }
+
+        // It would be nice if this next line was in a descturctor somewhere
+        glDeleteLists(f->listbase, 256);
+
+        Fl_FontSize* tmp = f;
+        f = f->next;
+        delete tmp;
+      }
+      else {
+        past = f;
+        f = f->next;
+      }
+    }
+  }
+
+#endif
+}
+
 void gl_draw(const char* str, int n) {
   glCallLists(n, GL_UNSIGNED_BYTE, str);
 }
@@ -155,5 +219,5 @@ void gl_draw_image(const uchar* b, int x, int y, int w, int h, int d, int ld) {
 #endif
 
 //
-// End of "$Id: gl_draw.cxx,v 1.7.2.5 2001/03/14 17:20:02 spitzak Exp $".
+// End of "$Id: gl_draw.cxx 5190 2006-06-09 16:16:34Z mike $".
 //
