@@ -1,9 +1,9 @@
 //
-// "$Id: CubeMain.cxx,v 1.2.2.5.2.4 2004/04/11 04:39:00 easysw Exp $"
+// "$Id: CubeMain.cxx 5519 2006-10-11 03:12:15Z mike $"
 //
 // CubeView class definitions for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,10 +20,12 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
-#include <config.h>
+#include "config.h"
 #include <FL/Fl.H>
 #include "CubeViewUI.h"
 
@@ -43,5 +45,5 @@ main(int argc, char **argv) {
 
 
 //
-// End of "$Id: CubeMain.cxx,v 1.2.2.5.2.4 2004/04/11 04:39:00 easysw Exp $".
+// End of "$Id: CubeMain.cxx 5519 2006-10-11 03:12:15Z mike $".
 //
