@@ -1,5 +1,5 @@
 //
-// "$Id: boxtype.cxx 4626 2005-11-02 13:57:56Z matt $"
+// "$Id: boxtype.cxx 5671 2007-02-08 07:58:47Z matt $"
 //
 // Boxtype test program for the Fast Light Tool Kit (FLTK).
 //
@@ -28,13 +28,13 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <FL/Fl.H>
-#include <FL/Fl_Single_Window.H>
+#include <FL/Fl_Double_Window.H>
 #include <FL/Fl_Box.H>
 
 int N = 0;
 #define W 200
 #define H 50
-#define ROWS 10
+#define ROWS 14
 
 Fl_Window *window;
 
@@ -50,7 +50,7 @@ void bt(const char *name, Fl_Boxtype type, int square=0) {
 }
 
 int main(int argc, char ** argv) {
-  window = new Fl_Single_Window(4*W,ROWS*H);
+  window = new Fl_Double_Window(4*W,ROWS*H);
   window->box(FL_FLAT_BOX);
 #if 0 // this code uses the command line arguments to set arbitrary color schemes
   Fl::args(argc, argv);
@@ -60,6 +60,7 @@ int main(int argc, char ** argv) {
   Fl::get_system_colors();
   Fl::background(113,113,198);
 #else // this code uses the nice bright blue background to show box vs. frame types
+  Fl::args(argc, argv);
   Fl::get_system_colors();
   window->color(12);// light blue
 #endif
@@ -94,12 +95,28 @@ int main(int argc, char ** argv) {
   bt("FL_ROUND_DOWN_BOX",FL_ROUND_DOWN_BOX);
   bt("FL_DIAMOND_UP_BOX",FL_DIAMOND_UP_BOX);
   bt("FL_DIAMOND_DOWN_BOX",FL_DIAMOND_DOWN_BOX);
+
   bt("FL_PLASTIC_UP_BOX",FL_PLASTIC_UP_BOX);
   bt("FL_PLASTIC_DOWN_BOX",FL_PLASTIC_DOWN_BOX);
   bt("FL_PLASTIC_UP_FRAME",FL_PLASTIC_UP_FRAME);
   bt("FL_PLASTIC_DOWN_FRAME",FL_PLASTIC_DOWN_FRAME);
   bt("FL_PLASTIC_THIN_UP_BOX",FL_PLASTIC_THIN_UP_BOX);
   bt("FL_PLASTIC_THIN_DOWN_BOX",FL_PLASTIC_THIN_DOWN_BOX);
+  N += 2;
+  bt("FL_PLASTIC_ROUND_UP_BOX",FL_PLASTIC_ROUND_UP_BOX);
+  bt("FL_PLASTIC_ROUND_DOWN_BOX",FL_PLASTIC_ROUND_DOWN_BOX);
+  N += 2;
+
+  bt("FL_GTK_UP_BOX",FL_GTK_UP_BOX);
+  bt("FL_GTK_DOWN_BOX",FL_GTK_DOWN_BOX);
+  bt("FL_GTK_UP_FRAME",FL_GTK_UP_FRAME);
+  bt("FL_GTK_DOWN_FRAME",FL_GTK_DOWN_FRAME);
+  bt("FL_GTK_THIN_UP_BOX",FL_GTK_THIN_UP_BOX);
+  bt("FL_GTK_THIN_DOWN_BOX",FL_GTK_THIN_DOWN_BOX);
+  bt("FL_GTK_THIN_UP_FRAME",FL_GTK_THIN_UP_FRAME);
+  bt("FL_GTK_THIN_DOWN_FRAME",FL_GTK_THIN_DOWN_FRAME);
+  bt("FL_GTK_ROUND_UP_BOX",FL_GTK_ROUND_UP_BOX);
+  bt("FL_GTK_ROUND_DOWN_BOX",FL_GTK_ROUND_DOWN_BOX);
   window->resizable(window);
   window->end();
   window->show();
@@ -107,5 +124,5 @@ int main(int argc, char ** argv) {
 }
 
 //
-// End of "$Id: boxtype.cxx 4626 2005-11-02 13:57:56Z matt $".
+// End of "$Id: boxtype.cxx 5671 2007-02-08 07:58:47Z matt $".
 //
