@@ -1,11 +1,11 @@
 //
-// "$Id: cube.cxx,v 1.4.2.5.2.7 2004/04/11 04:39:01 easysw Exp $"
+// "$Id: cube.cxx 5519 2006-10-11 03:12:15Z mike $"
 //
 // Another forms test program for the Fast Light Tool Kit (FLTK).
 //
 // Modified to have 2 cubes to test multiple OpenGL contexts
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -22,10 +22,12 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
-#include <config.h>
+#include "config.h"
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Box.H>
@@ -52,6 +54,7 @@ public:
 
 class cube_box : public Fl_Gl_Window {
   void draw();
+  int handle(int);
 public:
   double lasttime;
   int wire;
@@ -121,6 +124,14 @@ void cube_box::draw() {
   glEnable(GL_DEPTH_TEST);
 }
 
+int cube_box::handle(int e) {
+  switch (e) {
+  case FL_ENTER: cursor(FL_CURSOR_CROSS); break;
+  case FL_LEAVE: cursor(FL_CURSOR_DEFAULT); break;
+  }
+  return Fl_Gl_Window::handle(e);
+}
+
 #endif
 
 Fl_Window *form;
@@ -184,5 +195,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: cube.cxx,v 1.4.2.5.2.7 2004/04/11 04:39:01 easysw Exp $".
+// End of "$Id: cube.cxx 5519 2006-10-11 03:12:15Z mike $".
 //
