@@ -1,5 +1,5 @@
 //
-// "$Id: file_chooser.cxx 4410 2005-07-11 20:31:59Z matt $"
+// "$Id: file_chooser.cxx 6092 2008-04-11 12:57:37Z matt $"
 //
 // File chooser test program.
 //
@@ -211,7 +211,7 @@ multi_callback(void)
 Fl_Image *			// O - Page image or NULL
 pdf_check(const char *name,	// I - Name of file
           uchar      *header,	// I - Header data
-	  int        headerlen)	// I - Length of header data
+	  int)			// I - Length of header data (unused)
 {
   const char	*home;		// Home directory
   char		preview[1024],	// Preview filename
@@ -242,7 +242,7 @@ pdf_check(const char *name,	// I - Name of file
 Fl_Image *			// O - Page image or NULL
 ps_check(const char *name,	// I - Name of file
          uchar      *header,	// I - Header data
-	 int        headerlen)	// I - Length of header data
+	 int)			// I - Length of header data (unused)
 {
   const char	*home;		// Home directory
   char		preview[1024],	// Preview filename
@@ -315,8 +315,9 @@ show_callback(void)
 
   fc->show();
 
-  while (fc->visible())
+  while (fc->visible()) {
     Fl::wait();
+  }
 
   count = fc->count();
   if (count > 0)
@@ -340,5 +341,5 @@ show_callback(void)
 
 
 //
-// End of "$Id: file_chooser.cxx 4410 2005-07-11 20:31:59Z matt $".
+// End of "$Id: file_chooser.cxx 6092 2008-04-11 12:57:37Z matt $".
 //
