@@ -1,5 +1,5 @@
 //
-// "$Id: forms.cxx 4288 2005-04-16 00:13:17Z mike $"
+// "$Id: forms.cxx 5519 2006-10-11 03:12:15Z mike $"
 //
 // Another forms demo for the Fast Light Tool Kit (FLTK).
 //
@@ -69,7 +69,7 @@ static VN_struct btypes[]=
    {-1}
 };
 
-#include "srs.xbm"
+#include "pixmaps/srs.xbm"
 
 /*************** Callback **********************/
 
@@ -233,5 +233,5 @@ main (int argc, char *argv[])
 }
 
 //
-// End of "$Id: forms.cxx 4288 2005-04-16 00:13:17Z mike $".
+// End of "$Id: forms.cxx 5519 2006-10-11 03:12:15Z mike $".
 //
