@@ -1,5 +1,5 @@
 //
-// "$Id: fullscreen.cxx 4666 2005-11-28 15:35:18Z matt $"
+// "$Id: fullscreen.cxx 5845 2007-05-20 00:01:06Z mike $"
 //
 // Fullscreen test program for the Fast Light Tool Kit (FLTK).
 //
@@ -91,8 +91,8 @@ void shape_window::draw() {
   glClear(GL_COLOR_BUFFER_BIT);
   glColor3f(.5,.6,.7);
   glBegin(GL_POLYGON);
-  for (int i=0; i<sides; i++) {
-    double ang = i*2*M_PI/sides;
+  for (int j = 0; j < sides; j ++) {
+    double ang = j*2*M_PI/sides;
     glVertex3f(cos(ang),sin(ang),0);
   }
   glEnd();
@@ -247,5 +247,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: fullscreen.cxx 4666 2005-11-28 15:35:18Z matt $".
+// End of "$Id: fullscreen.cxx 5845 2007-05-20 00:01:06Z mike $".
 //
