@@ -1,9 +1,9 @@
 //
-// "$Id: hello.cxx,v 1.4.2.3.2.3 2004/04/11 04:39:01 easysw Exp $"
+// "$Id: hello.cxx 6102 2008-04-21 19:54:34Z matt $"
 //
 // Hello, World! program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,12 +20,15 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Box.H>
+#include <FL/filename.H>
 
 int main(int argc, char **argv) {
   Fl_Window *window = new Fl_Window(300,180);
@@ -39,5 +42,6 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: hello.cxx,v 1.4.2.3.2.3 2004/04/11 04:39:01 easysw Exp $".
+// End of "$Id: hello.cxx 6102 2008-04-21 19:54:34Z matt $".
 //
+
