@@ -1,5 +1,5 @@
 //
-// "$Id: input.cxx 4626 2005-11-02 13:57:56Z matt $"
+// "$Id: input.cxx 5519 2006-10-11 03:12:15Z mike $"
 //
 // Input field test program for the Fast Light Tool Kit (FLTK).
 //
@@ -61,7 +61,7 @@ void button_cb(Fl_Widget *,void *) {
 
 void color_cb(Fl_Widget* button, void* v) {
   Fl_Color c;
-  switch ((int)v) {
+  switch ((long)v) {
   case 0: c = FL_BACKGROUND2_COLOR; break;
   case 1: c = FL_SELECTION_COLOR; break;
   default: c = FL_FOREGROUND_COLOR; break;
@@ -138,5 +138,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: input.cxx 4626 2005-11-02 13:57:56Z matt $".
+// End of "$Id: input.cxx 5519 2006-10-11 03:12:15Z mike $".
 //
