@@ -1,9 +1,9 @@
 //
-// "$Id: mandelbrot.cxx,v 1.8.2.4.2.4 2004/04/11 04:39:01 easysw Exp $"
+// "$Id: mandelbrot.cxx 6474 2008-10-19 20:15:48Z matt $"
 //
 // Mandelbrot set demo for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include "mandelbrot_ui.h"
@@ -31,12 +33,12 @@
 Drawing_Window mbrot;
 Drawing_Window jbrot;
 
-void idle() {
-  if (!mbrot.d->idle() && !(jbrot.d && jbrot.d->idle())) Fl::set_idle(0);
+void idle(void*) {
+  if (!mbrot.d->idle() && !(jbrot.d && jbrot.d->idle())) Fl::remove_idle(idle);
 }
 
 void set_idle() {
-  Fl::set_idle(idle);
+  Fl::add_idle(idle);
 }
 
 static void window_callback(Fl_Widget*, void*) {exit(0);}
@@ -197,5 +199,5 @@ void Drawing_Area::resize(int XX,int YY,int WW,int HH) {
 }
 
 //
-// End of "$Id: mandelbrot.cxx,v 1.8.2.4.2.4 2004/04/11 04:39:01 easysw Exp $".
+// End of "$Id: mandelbrot.cxx 6474 2008-10-19 20:15:48Z matt $".
 //
