@@ -1,5 +1,5 @@
 //
-// "$Id: menubar.cxx 5519 2006-10-11 03:12:15Z mike $"
+// "$Id: menubar.cxx 6610 2008-12-29 17:24:51Z matt $"
 //
 // Menubar test program for the Fast Light Tool Kit (FLTK).
 //
@@ -171,6 +171,41 @@ Fl_Menu_Item pulldown[] = {
 
 Fl_Menu_* menus[4];
 
+class MyWindow : public Fl_Window {
+public:
+  MyWindow(int w, int h, const char *l=0)
+  : Fl_Window(w, h, l) { }
+  int handle(int event) {
+    switch (event) {
+    case FL_ENTER:
+//	puts("ENTER"); break;
+    case FL_LEAVE:
+//	puts("LEAVE"); break;
+      break;
+    }
+//    printf("%d\n", event);
+    return Fl_Window::handle(event);
+  }
+};
+
+
+class MyButton : public Fl_Button {
+public:
+  MyButton(int x, int y,int w, int h, const char *l=0)
+  : Fl_Button(x, y, w, h, l) { }
+  int handle(int event) {
+    switch (event) {
+    case FL_ENTER:
+	puts("ENTER"); break;
+    case FL_LEAVE:
+	puts("LEAVE"); break;
+    }
+    printf("%d\n", event);
+    return Fl_Button::handle(event);
+  }
+};
+
+
 int main(int argc, char **argv) {
   //Fl::set_color(Fl_Color(15),0,0,128);
   for (int i=0; i<99; i++) {
@@ -178,11 +213,12 @@ int main(int argc, char **argv) {
     sprintf(buf,"item %d",i);
     hugemenu[i].text = strdup(buf);
   }
-  Fl_Window window(WIDTH,400);
+  MyWindow window(WIDTH,400);
   window.callback(window_cb);
   Fl_Menu_Bar menubar(0,0,WIDTH,30); menubar.menu(menutable);
   menubar.callback(test_cb);
   menus[0] = &menubar;
+  MyButton bb(100, 125, 120, 25, "Erwinn");
   Fl_Menu_Button mb1(100,100,120,25,"&menubutton"); mb1.menu(pulldown);
   mb1.tooltip("this is a menu button");
   mb1.callback(test_cb);
@@ -207,5 +243,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: menubar.cxx 5519 2006-10-11 03:12:15Z mike $".
+// End of "$Id: menubar.cxx 6610 2008-12-29 17:24:51Z matt $".
 //
