@@ -1,9 +1,9 @@
 //
-// "$Id: pixmap.cxx,v 1.4.2.3.2.5 2004/04/11 04:39:01 easysw Exp $"
+// "$Id: pixmap.cxx 5519 2006-10-11 03:12:15Z mike $"
 //
 // Pixmap label test program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
@@ -29,7 +31,7 @@
 #include <FL/Fl_Pixmap.H>
 #include <stdio.h>
 
-#include "porsche.xpm"
+#include "pixmaps/porsche.xpm"
 
 #include <FL/Fl_Toggle_Button.H>
 
@@ -96,5 +98,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: pixmap.cxx,v 1.4.2.3.2.5 2004/04/11 04:39:01 easysw Exp $".
+// End of "$Id: pixmap.cxx 5519 2006-10-11 03:12:15Z mike $".
 //
