@@ -1,5 +1,5 @@
 //
-// "$Id: scroll.cxx 4288 2005-04-16 00:13:17Z mike $"
+// "$Id: scroll.cxx 5845 2007-05-20 00:01:06Z mike $"
 //
 // Fl_Scroll test program for the Fast Light Tool Kit (FLTK).
 //
@@ -71,7 +71,7 @@ void box_cb(Fl_Widget* o, void*) {
 }
 
 void type_cb(Fl_Widget*, void* v) {
-  thescroll->type(int(v));
+  thescroll->type((uchar)((long)v));
   thescroll->redraw();
 }
 
@@ -87,7 +87,7 @@ Fl_Menu_Item choices[] = {
 };
 
 void align_cb(Fl_Widget*, void* v) {
-  thescroll->scrollbar.align(int(v));
+  thescroll->scrollbar.align((uchar)((long)v));
   thescroll->redraw();
 }
 
@@ -140,5 +140,5 @@ int main(int argc, char** argv) {
 }
 
 //
-// End of "$Id: scroll.cxx 4288 2005-04-16 00:13:17Z mike $".
+// End of "$Id: scroll.cxx 5845 2007-05-20 00:01:06Z mike $".
 //
