@@ -1,9 +1,9 @@
 //
-// "$Id: shape.cxx,v 1.5.2.4.2.3 2004/04/11 04:39:01 easysw Exp $"
+// "$Id: shape.cxx 5845 2007-05-20 00:01:06Z mike $"
 //
 // Tiny OpenGL demo program for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2004 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,10 +20,12 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
-#include <config.h>
+#include "config.h"
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
 #include <FL/Fl_Hor_Slider.H>
@@ -58,8 +60,8 @@ void shape_window::draw() {
   glClear(GL_COLOR_BUFFER_BIT);
   glColor3f(.5,.6,.7);
   glBegin(GL_POLYGON);
-  for (int i=0; i<sides; i++) {
-    double ang = i*2*M_PI/sides;
+  for (int j=0; j<sides; j++) {
+    double ang = j*2*M_PI/sides;
     glVertex3f(cos(ang),sin(ang),0);
   }
   glEnd();
@@ -111,5 +113,5 @@ int main(int argc, char **argv) {
 }
 
 //
-// End of "$Id: shape.cxx,v 1.5.2.4.2.3 2004/04/11 04:39:01 easysw Exp $".
+// End of "$Id: shape.cxx 5845 2007-05-20 00:01:06Z mike $".
 //
