@@ -1,5 +1,5 @@
 //
-// "$Id: subwindow.cxx,v 1.5.2.5 2001/01/22 15:13:41 easysw Exp $"
+// "$Id: subwindow.cxx 6547 2008-12-06 11:15:29Z AlbrechtS $"
 //
 // Nested window test program for the Fast Light Tool Kit (FLTK).
 //
@@ -8,7 +8,7 @@
 // Buttons and pop-up menu should work, indicating that mouse positions
 // are being correctly translated.
 //
-// Copyright 1998-2001 by Bill Spitzak and others.
+// Copyright 1998-2005 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -25,7 +25,9 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+// Please report all bugs and problems on the following page:
+//
+//     http://www.fltk.org/str.php
 //
 
 #include <stdlib.h>
@@ -37,6 +39,21 @@
 #include <FL/Fl_Box.H>
 #include <FL/Fl_Input.H>
 
+#ifdef DEBUG
+#include <FL/names.h>
+#endif
+
+// Define DEBUG_POS for a subwindow positioning test. This will draw
+// the last typed character at the cursor position, if no input widget
+// currently has the focus.
+// Note: The typed character is saved in the derived testwindow class,
+// regardless of the definition of DEBUG_POS. Only drawing the character
+// depends on this define.
+
+#ifdef DEBUG_POS
+#include <FL/fl_draw.H>
+#endif
+
 class EnterExit : public Fl_Box {
   int handle(int);
 public:
@@ -49,55 +66,39 @@ int EnterExit::handle(int e) {
   else return 0;
 }
 
-#ifdef DEBUG
-const char *eventnames[] = {
-"zero",
-"FL_PUSH",
-"FL_RELEASE",
-"FL_ENTER",
-"FL_LEAVE",
-"FL_DRAG",
-"FL_FOCUS",
-"FL_UNFOCUS",
-"FL_KEYBOARD",
-"9",
-"FL_MOVE",
-"FL_SHORTCUT",
-"12",
-"FL_DEACTIVATE",
-"FL_ACTIVATE",
-"FL_HIDE",
-"FL_SHOW",
-"FL_PASTE",
-"FL_SELECTIONCLEAR",
-};
-#endif
-
 class testwindow : public Fl_Window {
   int handle(int);
   void draw();
   int cx, cy; char key;
+  Fl_Cursor crsr;
 public:
   testwindow(Fl_Boxtype b,int x,int y,const char *l)
-    : Fl_Window(x,y,l) {box(b); key = 0;}
+    : Fl_Window(x,y,l), crsr(FL_CURSOR_DEFAULT) {box(b); key = 0;}
   testwindow(Fl_Boxtype b,int x,int y,int w,int h,const char *l)
     : Fl_Window(x,y,w,h,l) {box(b); key = 0;}
+  void use_cursor(Fl_Cursor c) { crsr = c; }
 };
 
-#include <FL/fl_draw.H>
-
 void testwindow::draw() {
 #ifdef DEBUG
   printf("%s : draw\n",label());
 #endif
   Fl_Window::draw();
+#ifdef DEBUG_POS
   if (key) fl_draw(&key, 1, cx, cy);
+#endif
 }
 
 int testwindow::handle(int e) {
 #ifdef DEBUG
-  if (e != FL_MOVE) printf("%s : %s\n",label(),eventnames[e]);
+  if (e != FL_MOVE) printf("%s : %s\n",label(),fl_eventnames[e]);
 #endif
+  if (crsr!=FL_CURSOR_DEFAULT) {
+    if (e == FL_ENTER) 
+      cursor(crsr);
+    if (e == FL_LEAVE) 
+      cursor(FL_CURSOR_DEFAULT);
+  }
   if (Fl_Window::handle(e)) return 1;
   if (e == FL_FOCUS) return 1;
   if (e == FL_PUSH) {Fl::focus(this); return 1;}
@@ -151,22 +152,25 @@ const char* bigmess =
 #endif
 ;
 
-int main(int, char **) {
+int main(int argc, char **argv) {
   testwindow *window =
     new testwindow(FL_UP_BOX,400,400,"outer");
   new Fl_Toggle_Button(310,310,80,80,"&outer");
   new EnterExit(10,310,80,80,"enterexit");
-  new Fl_Input(150,310,150,25,"input:");
+  new Fl_Input(160,310,140,25,"input1:");
+  new Fl_Input(160,340,140,25,"input2:");
   (new Fl_Menu_Button(5,150,80,25,"menu&1"))->add(bigmess);
   testwindow *subwindow =
     new testwindow(FL_DOWN_BOX,100,100,200,200,"inner");
   new Fl_Toggle_Button(110,110,80,80,"&inner");
   new EnterExit(10,110,80,80,"enterexit");
-  (new Fl_Menu_Button(50,50,80,25,"menu&2"))->add(bigmess);
-  new Fl_Input(45,80,150,25,"input:");
+  (new Fl_Menu_Button(50,20,80,25,"menu&2"))->add(bigmess);
+  new Fl_Input(55,50,140,25,"input1:");
+  new Fl_Input(55,80,140,25,"input2:");
   subwindow->resizable(subwindow);
   window->resizable(subwindow);
   subwindow->end();
+  subwindow->use_cursor(FL_CURSOR_HAND);
   (new Fl_Box(FL_NO_BOX,0,0,400,100,
 	     "A child Fl_Window with children of it's own may "
 	     "be useful for imbedding controls into a GL or display "
@@ -178,10 +182,10 @@ int main(int, char **) {
   popup->type(Fl_Menu_Button::POPUP3);
   popup->add("This|is|a popup|menu");
   popup->add(bigmess);
-  window->show();
+  window->show(argc, argv);
   return Fl::run();
 }
 
 //
-// End of "$Id: subwindow.cxx,v 1.5.2.5 2001/01/22 15:13:41 easysw Exp $".
+// End of "$Id: subwindow.cxx 6547 2008-12-06 11:15:29Z AlbrechtS $".
 //
